using System;
using System.Collections;
using System.Collections.Generic;
using System.Text;
using Habanero.Base;
using Habanero.BO;
using Habanero.Faces.Base;
using Replace_it.BO;

namespace Replace_it.UI
{
    public class ComputerPartsGridManager : ControlManager
    {
        private IReadOnlyGridControl _computerPartsGrid;

        public ComputerPartsGridManager(IControlFactory controlFactory) : base(controlFactory)
        {
        }

        protected override void InitialiseControl()
        {
            SetupComputerPartsGrid();
            AddGridFilters();
            LoadComputerPartsGrid();
        }

        private void SetupComputerPartsGrid()
        {
            _computerPartsGrid = _controlFactory.CreateReadOnlyGridControl();
            BorderLayoutManager manager = _controlFactory.CreateBorderLayoutManager(_control);
            manager.AddControl(_computerPartsGrid, BorderLayoutManager.Position.Centre);
        }

        private void AddGridFilters()
        {
            BusinessObjectCollection<ComputerPartType> computerPartTypes = Broker.GetBusinessObjectCollection<ComputerPartType>("", "Code");
            List<string> options = new List<string>();
            computerPartTypes.ForEach(delegate(ComputerPartType type) { options.Add(type.Code); });
            _computerPartsGrid.FilterControl.AddStringFilterComboBox("Category:", "PartCode", options, false);
            _computerPartsGrid.FilterControl.AddStringFilterTextBox("Description:", "Description");
        }

        private void LoadComputerPartsGrid()
        {
            BusinessObjectCollection<ComputerPart> computerParts = new BusinessObjectCollection<ComputerPart>();
            computerParts.LoadAll();
            _computerPartsGrid.SetBusinessObjectCollection(computerParts);
        }
    }
}
