using System;
using System.IO;
using System.Reflection;
using System.Windows.Forms;
using Habanero.Base;
using Habanero.BO;
using Habanero.DB;
using Habanero.Faces.Win;
using log4net;
using log4net.Config;
using Replace_it.BO;

namespace Replace_it
{
    static class Program
    {
        private static readonly ILog log = LogManager.GetLogger("ReplaceIT.Program");

        [STAThread]
        static void Main()
        {
            HabaneroApp mainApp = new HabaneroAppWin("Replace_it", "v2.4.0");
            DOMConfigurator.Configure();
            try
            {
                log.Debug("---------------------------------------------------------------------");
                log.Debug("ReplaceIT Starting");
                log.Debug("---------------------------------------------------------------------");
                if (!mainApp.Startup()) return;
            
                Application.EnableVisualStyles();
                Application.SetCompatibleTextRenderingDefault(false);

                if (BORegistry.DataAccessor is DataAccessorInMemory) LoadDataIntoMemory();
                //SaveFromDBToXml();
                FormWin programForm = new FormWin();
                new UIManagerWin().SetupMainForm(programForm);


                Application.Run(programForm);

                if (BORegistry.DataAccessor is DataAccessorInMemory) SaveDataToXml();

            }
            catch (Exception ex)
            {
                log.Error(ex.Message);
                GlobalRegistry.UIExceptionNotifier.Notify(ex,
					"An error has occurred in the application.",
					"Application Error");
            }
        }

        private static void SaveDataToXml()
        {
            DataAccessorInMemory dataAccessorInMemory = (DataAccessorInMemory) BORegistry.DataAccessor;
            DatabaseConfig config = DatabaseConfig.ReadFromConfigFile();
            using (Stream replaceitStream = File.Create(config.Database))
            {
                DataStoreInMemoryXmlWriter writer = new DataStoreInMemoryXmlWriter(replaceitStream);
                writer.Write(dataAccessorInMemory.DataStoreInMemory);
            }
        }

        private static void LoadDataIntoMemory()
        {
            DatabaseConfig config = DatabaseConfig.ReadFromConfigFile();
            using (Stream replaceItStream = File.OpenRead(config.Database))
            {
                DataStoreInMemoryXmlReader reader = new DataStoreInMemoryXmlReader(replaceItStream);
                DataStoreInMemory replaceitDataStore = new DataStoreInMemory();
                replaceitDataStore.AllObjects = reader.Read();
                BORegistry.DataAccessor = new DataAccessorInMemory(replaceitDataStore);
            }
        }

     
    }
}