using System.Collections.Generic;
using FireStarter.CodeGeneration.Generator;
using FireStarter.CodeGeneration.Parser;
using NUnit.Framework;

namespace FireStarter.Test.CodeGeneration.Parser
{
    [TestFixture]
    public class TestForEachExpressionParser
    {
        private static readonly List<IActionParser> _defaultActionParsers = new List<IActionParser>() { new ValueExpressionParser(), new ForEachExpressionParser() };

        [Test]
        public void Test_Parse()
        {
            //---------------Set up test pack-------------------
            string propertyName = TestUtils.GetRandomString();
            string innerContent = TestUtils.GetRandomString();
            string text = string.Format("FOREACH({0}):{1}", propertyName, innerContent);
            IParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            TemplateExpression templateExpression = parser.Parse(ref text, _defaultActionParsers);
            //---------------Test Result -----------------------
            ForEachExpression forEachExpression;
            ParserTestUtils.AssertExpressionIsForEach(propertyName, templateExpression, out forEachExpression);
            ParserTestUtils.AssertExpressionIsLiteral(innerContent, forEachExpression.Part);
            Assert.IsNull(forEachExpression.Next);
            Assert.AreEqual("", text);
        }

        [Test]
        public void Test_IsExpressionMatch()
        {
            //---------------Set up test pack-------------------
            string propertyName = TestUtils.GetRandomString();
            string innerContent = TestUtils.GetRandomString();
            string text = string.Format("FOREACH({0}):{1}", propertyName, innerContent);
            ForEachExpressionParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            bool isExpressionMatch = parser.IsExpressionMatch(text);
            //---------------Test Result -----------------------
            Assert.IsTrue(isExpressionMatch);
        }

        [Test]
        public void Test_IsExpressionMatch_WithWhitespaceBeforeColon()
        {
            //---------------Set up test pack-------------------
            string propertyName = TestUtils.GetRandomString();
            string text = string.Format("FOREACH({0}){1}:{2}", propertyName, TestUtils.GetRandomWhiteSpace(), TestUtils.GetRandomString());
            ForEachExpressionParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            bool isExpressionMatch = parser.IsExpressionMatch(text);
            //---------------Test Result -----------------------
            Assert.IsTrue(isExpressionMatch);
        }

        [Test]
        public void Test_IsExpressionMatch_Fails_IfJustForEach()
        {
            //---------------Set up test pack-------------------
            const string text = "FOREACH";
            ForEachExpressionParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            bool isExpressionMatch = parser.IsExpressionMatch(text);
            //---------------Test Result -----------------------
            Assert.IsFalse(isExpressionMatch);
        }

        [Test]
        public void Test_IsExpressionMatch_Fails_IfNoColon()
        {
            //---------------Set up test pack-------------------
            string propertyName = TestUtils.GetRandomString();
            string innerContent = TestUtils.GetRandomString();
            string text = string.Format("FOREACH({0}){1}", propertyName, innerContent);
            ForEachExpressionParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            bool isExpressionMatch = parser.IsExpressionMatch(text);
            //---------------Test Result -----------------------
            Assert.IsFalse(isExpressionMatch);
        }

        [Test]
        public void Test_IsExpressionMatch_Fails_IfPatternDoesntMatch()
        {
            //---------------Set up test pack-------------------
            string propertyName = TestUtils.GetRandomString();
            string text = propertyName;
            ForEachExpressionParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            bool isExpressionMatch = parser.IsExpressionMatch(text);
            //---------------Test Result -----------------------
            Assert.IsFalse(isExpressionMatch);
        }

        [Test]
        public void Test_IsExpressionMatch_Fails_IfNull()
        {
            //---------------Set up test pack-------------------
            const string text = null;
            ForEachExpressionParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            bool isExpressionMatch = parser.IsExpressionMatch(text);
            //---------------Test Result -----------------------
            Assert.IsFalse(isExpressionMatch);
        }

        [Test]
        public void Test_IsExpressionMatch_Fails_IfEmpty()
        {
            //---------------Set up test pack-------------------
            const string text = "";
            ForEachExpressionParser parser = new ForEachExpressionParser();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            bool isExpressionMatch = parser.IsExpressionMatch(text);
            //---------------Test Result -----------------------
            Assert.IsFalse(isExpressionMatch);
        } 
    }
}
