﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using FireStarter.CodeGeneration;
using FireStarterModeller.BO;
using NUnit.Framework;

namespace FireStarter.Test.CodeGeneration
{
    [TestFixture]
    public class TestBaseTestProjectGenerator: TestProjectGeneratorBase
    {
        protected override IProjectGenerator CreateProjectGenerator()
        {
            return new BaseTestProjectGenerator();
        }

        protected override IProjectGenerator CreateProjectReadyToGenerate()
        {
            BaseTestProjectGenerator projectGenerator = new BaseTestProjectGenerator();
            CreateSolutionGenerator(projectGenerator);
            projectGenerator.ProjectPath = GetRootedPath(Path.Combine(TEST_PROJECTS_FOLDER, GetRandomString()));
            projectGenerator.ProjectName = "TestProject.Test.Base";
            //projectGenerator.SolutionGenerator.ClassDefsString = TestUtils.GetRandomString();
            //projectGenerator.BoProjectFileName = GetRootedPath(GetRandomProjectFileName());
            return projectGenerator;
        }

        private void AssertProjectGeneratedCorrectly(IProjectGenerator testProjectGenerator)
        {
            string libFolderName = testProjectGenerator.SolutionGenerator.HabaneroReferencePath;
            libFolderName = Utilities.GetRelativePath(libFolderName, testProjectGenerator.ProjectPath);
            string projectFileName = testProjectGenerator.ProjectFileName;
            Assert.IsTrue(Directory.Exists(testProjectGenerator.ProjectPath));
            Assert.IsTrue(File.Exists(projectFileName));
            ProjectFileManager projectFileManager = new ProjectFileManager(projectFileName, CodeTemplateEnum.CSharpVisualStudio2005);
            Assert.AreEqual(OutputType.Library, projectFileManager.OutputType);
            string projectFileContents = File.ReadAllText(projectFileName);
            StringAssert.Contains(@"<Compile Include=""Properties\AssemblyInfo.cs"" />", projectFileContents);
            AssertHasReference(projectFileContents, string.Format(@"{0}\Habanero.Base.dll", libFolderName));
            AssertHasReference(projectFileContents, string.Format(@"{0}\Habanero.BO.dll", libFolderName));
            AssertHasReference(projectFileContents, string.Format(@"{0}\Habanero.DB.dll", libFolderName));
            AssertHasReference(projectFileContents, string.Format(@"{0}\nunit.framework.dll", libFolderName));
        }

        [Test]
        public void TestCreateBoProjectGenerator()
        {
            //---------------Set up test pack-------------------

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IProjectGenerator projectGenerator = CreateProjectGenerator();
            //---------------Test Result -----------------------
            Assert.IsNotNull(projectGenerator);
            Assert.IsInstanceOf(typeof(BaseTestProjectGenerator), projectGenerator);
        }

        [Test]
        public void TestGenerate_Creates_ProjectFile_IfDoesNotExist()
        {
            //---------------Set up test pack-------------------
            IProjectGenerator projectGenerator = CreateProjectReadyToGenerate();
            //---------------Assert Precondition----------------
            Assert.IsFalse(File.Exists(projectGenerator.ProjectFileName));
            //---------------Execute Test ----------------------
            projectGenerator.Generate();
            //---------------Test Result -----------------------
            AssertProjectGeneratedCorrectly(projectGenerator);
        }
    }
}
