using System;
using Habanero.Base;
using Habanero.Base.Exceptions;

namespace FireStarterModeller.BO
{
    /// <summary>
    /// A <see cref="IBusinessObjectRule"/> that validates that a <see cref="DMClass"/>
    /// has an <see cref="DMObjectIdentity"/> and that the <see cref="DMClass.ObjectIdentity"/> 
    /// has at least one <see cref="DMProperty"/>
    /// </summary>
    public class DMClassMustHaveObjectID : IBusinessObjectRule
    {
        private DMClass _dmClass;

//        public DMClassMustHaveObjectID(DMClass dmClass)
//        {
//            if (dmClass == null) throw new ArgumentNullException("dmClass");
//            _dmClass = dmClass;
//        }

        #region Implementation of IBusinessObjectRule

        /// <summary>
        /// Indicates whether the property value is valid against the rules
        /// </summary>
        /// <returns>
        /// Returns true if valid
        /// </returns>
        public bool IsValid()
        {
            throw new HabaneroApplicationException("The IsValid is no longer Valid you should use the IsValid(bo) method");
        }

        public bool IsValid(IBusinessObject bo)
        {
            _dmClass = (DMClass) bo;
            DMObjectIdentity objectIdentity = _dmClass.ObjectIdentity;
            if (objectIdentity == null || objectIdentity.Properties.Count == 0)
            {
                return (GetSuperClassObjectIdentity(_dmClass));
            }
            return objectIdentity.Properties.Count > 0;
        }

        private bool GetSuperClassObjectIdentity(DMClass dmClass)
        {
            DMInheritanceRelationship superClassRelationship = dmClass.InheritanceSuperClassRelationship;
            if (superClassRelationship != null && superClassRelationship.SuperClass != null)
            {
                DMClass superClass = superClassRelationship.SuperClass;
                DMObjectIdentity superClassObjectIdentity = superClass.ObjectIdentity;
                if (superClassObjectIdentity != null && superClassObjectIdentity.Properties.Count > 0) return true;
                return GetSuperClassObjectIdentity(superClass);
            }
            return false;
        }

        /// <summary>
        /// Returns the rule name
        /// </summary>
        public string Name
        {
            get { return "DMClassMustHaveObjectID"; }
        }

        /// <summary>
        /// Returns the error message for if the rule fails.
        /// </summary>
        public string Message
        {
            get { return "The Entity/Class '" + _dmClass + "' must have an ObjectID with at least one Property"; }
        }
        /// <summary>
        /// The <see cref="P:Habanero.Base.IBusinessObjectRule.ErrorLevel" /> for this BusinessObjectRule e.g. Warning, Error. 
        /// </summary>
        public ErrorLevel ErrorLevel
        {
            get { return Habanero.Base.ErrorLevel.Warning; }
        }

        #endregion
    }
}