using System;
using System.Text.RegularExpressions;
using Habanero.Base;
using Habanero.Base.Exceptions;

namespace FireStarterModeller.BO
{
    public class DMPropertyNameDoesNotStartWithNumberRule : IBusinessObjectRule
    {
        private DMProperty _dmProperty;

//        public DMPropertyNameDoesNotStartWithNumberRule(DMProperty dmProperty)
//        {
//            if (dmProperty == null) throw new ArgumentNullException("dmProperty");
//            _dmProperty = dmProperty;
//        }

        #region Implementation of IBusinessObjectRule

        /// <summary>
        /// Indicates whether the property value is valid against the rules
        /// </summary>
        /// <returns>
        /// Returns true if valid
        /// </returns>
        public bool IsValid()
        {
            throw new HabaneroApplicationException("The IsValid is no longer Valid you should use the IsValid(bo) method");
        }

        public bool IsValid(IBusinessObject bo)
        {
            _dmProperty = (DMProperty) bo;
            if (String.IsNullOrEmpty(_dmProperty.PropertyName)) return true;
            return Regex.IsMatch(_dmProperty.PropertyName, "^[a-zA-Z]{1}.*$");
        }

        /// <summary>
        /// Returns the rule name
        /// </summary>
        public string Name
        {
            get { return "DMPropertyNameDoesNotStartWithNumber"; }
        }

        /// <summary>
        /// Returns the error message for if the rule fails.
        /// </summary>
        public string Message
        {
            get { return "The Property Name '" + _dmProperty.ToString() + "' on Class '" + _dmProperty.Class + "' cannot start with a number."; }
        }
        /// <summary>
        /// The <see cref="P:Habanero.Base.IBusinessObjectRule.ErrorLevel" /> for this BusinessObjectRule e.g. Warning, Error. 
        /// </summary>
        public ErrorLevel ErrorLevel
        {
            get { return Habanero.Base.ErrorLevel.Error; }
        }

        #endregion
    }
}