using System;
using System.Collections;
using Habanero.Base;

namespace FireStarterModeller.BO
{
    public class DMPropertyPropertyTypeIntCanOnlyBeAutoIncrementing : IBusinessObjectRule
    {
        private readonly DMProperty _dmProperty;
       

        public DMPropertyPropertyTypeIntCanOnlyBeAutoIncrementing(DMProperty dmProperty)
        {
            if (dmProperty == null) throw new ArgumentNullException("dmProperty");
            _dmProperty = dmProperty;
            
        }

        #region Implementation of IBusinessObjectRule

        /// <summary>
        /// Indicates whether the property value is valid against the rules
        /// </summary>
        /// <returns>
        /// Returns true if valid
        /// </returns>
        public bool IsValid()
        {
            if (_dmProperty.PropertyType == null) return true;
            return String.IsNullOrEmpty(_dmProperty.PropertyType.PropertyTypeName) || CheckValidity();
        }

        private bool CheckValidity()
        {
            return _dmProperty.PropertyType.PropertyTypeName.StartsWith("Int") || _dmProperty.AutoIncrementing != true;
        }

        public bool IsValid(IBusinessObject bo)
        {
            return IsValid();
        }

        /// <summary>
        /// Returns the rule name
        /// </summary>
        public string Name
        {
            get { return "DMPropertyPropertyTypeIntCanOnlyBeAutoIncrementing"; }
        }

        /// <summary>
        /// Returns the error message for if the rule fails.
        /// </summary>
        public string Message
        {
            get { return "The Property '" + _dmProperty.ToString() + "' on class '" + _dmProperty.Class.ToString() + "' with Property Type '" + _dmProperty.PropertyType.ToString() + "' cannot be AutoIncrementing"; }
        }
        /// <summary>
        /// The <see cref="P:Habanero.Base.IBusinessObjectRule.ErrorLevel" /> for this BusinessObjectRule e.g. Warning, Error. 
        /// </summary>
        public ErrorLevel ErrorLevel
        {
            get { return Habanero.Base.ErrorLevel.Error; }
        }

        #endregion
    }
}