using System;
using Habanero.Base;
using Habanero.Base.Exceptions;

namespace FireStarterModeller.BO
{
    /// <summary>
    /// A <see cref="IBusinessObjectRule"/> that validates that a <see cref="DMClass"/>
    /// has an <see cref="DMObjectIdentity"/> and that the <see cref="DMClass.ObjectIdentity"/> 
    /// has at least one <see cref="DMProperty"/>
    /// </summary>
    public class DMUniqueConstraintMustHaveProps : IBusinessObjectRule
    {
        private DMUniqueConstraint _dmUniqueConstraint;

//        public DMUniqueConstraintMustHaveProps(DMUniqueConstraint dmUniqueConstraint)
//        {
//            if (dmUniqueConstraint == null) throw new ArgumentNullException("dmUniqueConstraint");
//            _dmUniqueConstraint = dmUniqueConstraint;
//        }

        #region Implementation of IBusinessObjectRule

        /// <summary>
        /// Indicates whether the property value is valid against the rules
        /// </summary>
        /// <returns>
        /// Returns true if valid
        /// </returns>
        public bool IsValid()
        {
            throw new HabaneroApplicationException("The IsValid is no longer Valid you should use the IsValid(bo) method");
        }

        public bool IsValid(IBusinessObject bo)
        {
            _dmUniqueConstraint = (DMUniqueConstraint) bo;
            return _dmUniqueConstraint.UniqueConstraintProperties.Count > 0;
        }

        /// <summary>
        /// Returns the rule name
        /// </summary>
        public string Name
        {
            get { return "DMUniqueConstraintMustHaveProps"; }
        }

        /// <summary>
        /// Returns the error message for if the rule fails.
        /// </summary>
        public string Message
        {
            get { return "The UniqueConstraint '" + _dmUniqueConstraint.ToString() + "' on Class '" + _dmUniqueConstraint .Class + "' must have at least one Property"; }
        }
        /// <summary>
        /// The <see cref="P:Habanero.Base.IBusinessObjectRule.ErrorLevel" /> for this BusinessObjectRule e.g. Warning, Error. 
        /// </summary>
        public ErrorLevel ErrorLevel
        {
            get { return Habanero.Base.ErrorLevel.Warning; }
        }

        #endregion
    }
}