// ------------------------------------------------------------------------------
// This class was auto-generated for use with the Habanero Enterprise Framework.
// ------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using FireStarter.Base;
using Habanero.Base;
using Habanero.Base.Exceptions;
using Habanero.BO;
using Habanero.BO.ClassDefinition;

namespace FireStarterModeller.BO
{
    [Serializable]
    public partial class DMPropRuleParameter : IModelElement, IMergableBo
    {
        const string DATETIME_TODAY = "Today";
        const string DATETIME_NOW = "Now";

        public DMPropRuleParameter() : this(null)
        {
        }

        protected internal DMPropRuleParameter(ClassDef def) : base(def)
        {
            this.Props["Value"].Updated += PropertyValue_UpdatedHandler;
        }


        protected DMPropRuleParameter(SerializationInfo info, StreamingContext context) : base(info, context)
        {
            this.Props["Value"].Updated += PropertyValue_UpdatedHandler;
        }

        private void PropertyValue_UpdatedHandler(object sender, BOPropEventArgs e)
        {
            string message;
            ValidatePropValue(e.Prop, out message);
            if (!string.IsNullOrEmpty(message))
            {
                throw new UserException(message);
            }
        }

        private void ValidatePropValue(IBOProp prop, out string message)
        {
            message = "";
            DMPropRule propRule = this.PropRule;
            if (propRule == null) return;
            if (propRule.DMProperty == null) return;
            if (string.IsNullOrEmpty(this.Value)) return;
            DMProperty dmProperty = propRule.DMProperty;
            if (dmProperty.PropertyTypeString == DMPropertyType.PROP_TYPE_DATETIME)
            {
                ValidatePropRuleParametersForDateTime(prop, out message);
            }
            else if (this.KeyName == "minLength" || this.KeyName == "maxLength")
            {
                ValidatePropRuleParametersForString(prop, out message);
            }
            else if (this.KeyName == "min" || this.KeyName == "max")
            {
                ValidatePropRuleParametersForNumber(prop, out message);
            }
        }

        private void ValidatePropRuleParametersForNumber(IBOProp prop, out string message)
        {
            message = "";
            decimal decValue;
            if (!decimal.TryParse(Convert.ToString(this.Value), out decValue))
            {
                message = string.Format
                    ("The value '{0}' is not valid for '{1}'. " + "The value must be numeric.", prop.Value, this.KeyName);
                return;
                //throw new UserException(message);
            }
            if (this.KeyName == "min") ValidateMinParameter("min", "max", out message);
            if (this.KeyName == "max") ValidateMaxParameter("max", "min", out message);
        }

        private void ValidatePropRuleParametersForDateTime(IBOProp prop, out string message)
        {
            message = "";
            if (this.KeyName == "min" || this.KeyName == "max")
            {

                if (this.Value == DATETIME_TODAY || this.Value == DATETIME_NOW) return;
                DateTime dateTimeValue;
                if (!DateTime.TryParse(Convert.ToString(this.Value), out dateTimeValue))
                {
                    message = string.Format
                        ("The value '{0}' is not valid for '{1}'. " + "The value must be a DateTime.", prop.Value,
                         this.KeyName);
                    return;
                    //throw new UserException(message);
                }

                if (this.KeyName == "min") ValidateMinDateParameter(out message);
                if (this.KeyName == "max") ValidateMaxDateParameter(out message);
                //ValidateMinDateParameter(out message);
                //if (string.IsNullOrEmpty(message))
                //{
                //    ValidateMaxDateParameter(out message);
                //}
            }
        }

        private void ValidatePropRuleParametersForString(IBOProp prop, out string message)
        {
            message = "";
            int intValue;
            if (!int.TryParse(Convert.ToString(this.Value), out intValue))
            {
                message = string.Format
                    ("The value '{0}' is not valid for '{1}'. " + "The value must be an integer.", prop.Value, this.KeyName);
                return;
                //throw new UserException(message);
            }
            if (intValue < 0)
            {
                message = string.Format
                    ("The value '{0}' is not valid for '{1}'. " + "The value must be greater than zero.", prop.Value,
                     this.KeyName);
                return;
                //throw new UserException(message);
            }
            if (this.KeyName == "minLength") ValidateMinParameter("minLength", "maxLength", out message);
            if (this.KeyName == "maxLength") ValidateMaxParameter("maxLength", "minLength", out message);
        }

        private void ValidateMinParameter(string key, string compareKey, out string message)
        {
            message = "";
            if (this.KeyName != key) return;
            foreach (DMPropRuleParameter propRuleParameter in this.PropRule.PropRuleParameters)
            {
                if (propRuleParameter.KeyName != compareKey) continue;
                if (propRuleParameter.Value == null) return;
                decimal decValue;
                if (!decimal.TryParse(Convert.ToString(propRuleParameter.Value), out decValue))
                {
                    message = string.Format
                        ("The value '{0}' is not valid for '{1}'. " + "The value must be numeric.",
                         propRuleParameter.Value, propRuleParameter.KeyName);
                    return;
                    //throw new UserException(message);
                }

                if (Convert.ToDecimal(this.Value) > Convert.ToDecimal(propRuleParameter.Value))
                {
                    message = "The value '" + this.Value + "' is not valid for '" + this.KeyName
                              + "'. The value cannot be greater than the '" + propRuleParameter.KeyName + "'";
                    return;
                    //throw new UserException(message);
                }
            }
        }


        private void ValidateMaxParameter(string key, string compareKey, out string message)
        {
            message = "";
            if (this.KeyName != key) return;
            foreach (DMPropRuleParameter propRuleParameter in this.PropRule.PropRuleParameters)
            {
                if (propRuleParameter.KeyName != compareKey) continue;
                if (propRuleParameter.Value == null) return;
                decimal decValue;
                if (!decimal.TryParse(Convert.ToString(propRuleParameter.Value), out decValue))
                {
                    message = string.Format
                        ("The value '{0}' is not valid for '{1}'. " + "The value must be numeric.",
                         propRuleParameter.Value, propRuleParameter.KeyName);
                    return;
                    //throw new UserException(message);
                }
                if (Convert.ToDecimal(propRuleParameter.Value) > Convert.ToDecimal(this.Value))
                {
                    message = "The value '" + this.Value + "' is not valid for '" + this.KeyName
                              + "'. The value cannot be less than the '" + propRuleParameter.KeyName + "'";
                    return;
                    //throw new UserException(message);
                }
            }
        }

        private void ValidateMaxDateParameter(out string message)
        {
            message = "";
            if (this.KeyName != "max") return;
            foreach (DMPropRuleParameter propRuleParameter in this.PropRule.PropRuleParameters)
            {
                if (propRuleParameter.KeyName != "min") continue;
                if (propRuleParameter.Value == null) return;
                if (this.Value == DATETIME_TODAY || propRuleParameter.Value == DATETIME_TODAY) return;
                if (this.Value == DATETIME_NOW || propRuleParameter.Value == DATETIME_NOW) return;
                if (Convert.ToDateTime(propRuleParameter.Value) > Convert.ToDateTime(this.Value))
                {
                    message = "The value '" + this.Value + "' is not valid for '" + this.KeyName
                              + "'. The value cannot be less than the '" + propRuleParameter.KeyName + "'";
                    return;
                    //throw new UserException(message);
                }
            }
        }


        private void ValidateMinDateParameter(out string message)
        {
            message = "";
            if (this.KeyName != "min") return;
            foreach (DMPropRuleParameter propRuleParameter in this.PropRule.PropRuleParameters)
            {
                if (propRuleParameter.KeyName != "max") continue;
                if (propRuleParameter.Value == null) return;
                
                if (this.Value == DATETIME_TODAY || propRuleParameter.Value == DATETIME_TODAY) return;
                if (this.Value == DATETIME_NOW || propRuleParameter.Value == DATETIME_NOW) return;
                if (Convert.ToDateTime(this.Value) > Convert.ToDateTime(propRuleParameter.Value))
                {
                    message = "The value '" + this.Value + "' is not valid for '" + this.KeyName
                              + "'. The value cannot be greater than the '" + propRuleParameter.KeyName + "'";
                    return;
                    //throw new UserException(message);
                }
            }
        }


        public override string ToString()
        {
            if (string.IsNullOrEmpty(KeyName))
            {
                return this.PropRuleParameterID == null
                           ? "-unnamed PropRule-"
                           : Convert.ToString(this.PropRuleParameterID);
            }
            return this.KeyName;
        }


        //protected override bool AreCustomRulesValid(out IList<IBOError> errors)
        //{
        //    bool isValid = base.AreCustomRulesValid(out errors);
        //    string message;
        //    ValidatePropValue(this.Props["Value"], out message);
        //    if (!string.IsNullOrEmpty(message))
        //    {
        //        isValid = false;
        //        errors.Add(new BOError(message, ErrorLevel.Error));
        //    }
        //    return isValid;
        //}

        protected override bool AreCustomRulesValid(out string customRuleErrors)
        {
            bool isValid = base.AreCustomRulesValid(out customRuleErrors);
            string message;
            ValidatePropValue(this.Props["Value"], out message);
            if (!string.IsNullOrEmpty(message))
            {
                isValid = false;
                customRuleErrors += message;
            }
            return isValid;
        }


        IModelElement IModelElement.Parent
        {
            get { return this.PropRule; }
            set
            {
                throw new HabaneroDeveloperException
                    ("Set the parent of the Property rule parameter by setting the related Property rule object", "");
            }
        }

        #region Implementation of IMergableBo

        ChangeType IMergableBo.PendingChangeType
        {
            get;
            set;
        }

        bool IMergableBo.PendingChangeApproved
        {
            get { return true; }
            set { }
        }

        #endregion
    }
}