// ------------------------------------------------------------------------------
// This partial class was auto-generated for use with the Habanero Architecture.
// ------------------------------------------------------------------------------

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Runtime.Serialization;
using FireStarter.Base;
using Habanero.Base;
using Habanero.BO;
using Habanero.BO.ClassDefinition;

namespace FireStarterModeller.BO
{
    public interface IDMRelationshipProperty : IBusinessObject
    {
        string OwnerPropertyName { get; set; }
        string RelatedPropertyName { get; set; }
        IDMRelationshipProperty ReverseRelationshipProperty { get; set; }
        IDMRelationship Relationship { get; set; }
        IDMProperty Property { get; set; }
        IDMProperty RelatedProperty { get; set; }
        IDMProperty OwnerProperty { get; set; }
    }

    [Serializable]
    public partial class DMRelationshipProperty : IRelPropDef, IDMRelationshipProperty, IMergableBo
    {
        private string _relatedPropertyName;
        private string _ownerPropertyName;

        public static bool Loading { private get; set; }

        public DMRelationshipProperty() : this((IClassDef) null)
        {
            if (!Loading)
            {
                new DMRelationshipProperty(this);
            }
            SetupRelationshipLinkLogic();
        }

        // ReSharper disable DoNotCallOverridableMethodsInConstructor
        public DMRelationshipProperty(IDMRelationshipProperty reverseRelationshipProperty)
            : this((IClassDef) null)
        {
            this.ReverseRelationshipProperty = reverseRelationshipProperty;
            if (reverseRelationshipProperty != null)
            {
                reverseRelationshipProperty.ReverseRelationshipProperty = this;
            }
            SetupRelationshipLinkLogic();
        }

        // ReSharper restore DoNotCallOverridableMethodsInConstructor

        protected internal DMRelationshipProperty(IClassDef def) : base(def)
        {
        }
        protected DMRelationshipProperty(ConstructForFakes constructForFakes)
            : base(constructForFakes)
        {
        }
        protected DMRelationshipProperty(SerializationInfo info, StreamingContext context) : base(info, context)
        {
        }

//        private bool _busySettingUpRelationshipLink = false;
        private void SetupRelationshipLinkLogic()
        {
            if(Relationships == null) return;
            ISingleRelationship singleRelationship = (ISingleRelationship) Relationships["Relationship"];
            if (singleRelationship == null) return;
            singleRelationship.RelatedBusinessObjectChanged += (sender, e) =>
                           {
                               if (Loading) return;
                               IDMRelationship dmRelationship = Relationship;
                               DMRelationship reverseRelationship = null;
                               if (dmRelationship != null)
                                   reverseRelationship = (DMRelationship) dmRelationship.ReverseRelationship;
                               if (ReverseRelationshipProperty != null &&
                                   ReverseRelationshipProperty.Relationship != reverseRelationship)
                               {
                                   //Debug.Print( Prn(this) + " --> " + Prn(ReverseRelationshipProperty) + " ==> " + Prn(reverseRelationship != null ? reverseRelationship.RelationshipID : null));
                                   ReverseRelationshipProperty.Relationship = reverseRelationship;
                               }
                           };
        }

        public override string ToString()
        {
            if (this.Relationship == null && this.OwnerProperty == null)
            {
                return this.RelationshipPropertyID == null
                           ? "-unnamed RelationshipProperty-"
                           : Convert.ToString(this.RelationshipPropertyID);
            }
            return (this.Relationship == null ? "" : this.Relationship.ToString()) + " - "
                   + (this.OwnerProperty == null ? "" : this.OwnerProperty.ToString());
        }

        protected override void LoadBusinessObjectRules(IList<IBusinessObjectRule> boRules)
        {
            base.LoadBusinessObjectRules(boRules);
            boRules.Add(new DMRelationshipPropertyTypesAreSame());
        }

        string IRelPropDef.RelatedClassPropName
        {
            get { return RelatedPropertyName; }
        }

        public IPropDef OwnerPropDef
        {
            get { return (IPropDef) this.OwnerProperty; }
        }

        public virtual string RelatedPropertyName
        {
            get { return RelatedProperty == null ? _relatedPropertyName : RelatedProperty.PropertyName; }
            set { _relatedPropertyName = value; }
        }

        public virtual string OwnerPropertyName
        {
            get { return OwnerProperty == null ? _ownerPropertyName : OwnerProperty.PropertyName; }
            set { _ownerPropertyName = value; }
        }

        public virtual IDMProperty OwnerProperty
        {
            get { return this.Property; }
            set { this.Property = value; }
        }

        public virtual IDMProperty RelatedProperty
        {
            get { return (this.ReverseRelationshipProperty != null ? this.ReverseRelationshipProperty.Property : null); }
            set { this.ReverseRelationshipProperty.Property = value; }
        }

/*
        public Guid? OwnerPropertyID
        {
            get { return this.PropertyID; }
            set { this.PropertyID = value; }
        }*/

/*        public Guid? RelatedPropertyID
        {
            get { return this.ReverseRelationshipProperty.PropertyID; }
            set { this.ReverseRelationshipProperty.PropertyID = value; }
        }*/

        /// <summary>
        /// Returns the Class associated with the Owner Property
        /// </summary>
        public virtual DMClass OwnerClass
        {
            get { return (DMClass) (OwnerProperty == null ? null : OwnerProperty.Class); }
        }

        public bool AreBothPropTypesSame()
        {
            if (this.RelatedProperty == null || this.OwnerProperty == null) return false;

            return this.OwnerProperty.PropertyTypeString == this.RelatedProperty.PropertyTypeString;
        }


        public virtual IDMRelationshipProperty ReverseRelationshipProperty
        {
            get { return Relationships.GetRelatedObject<DMRelationshipProperty>("ReverseRelationshipProperty"); }
            set { Relationships.SetRelatedObject("ReverseRelationshipProperty", value); }
        }

        public virtual IDMProperty Property
        {
            get { return Relationships.GetRelatedObject<DMProperty>("Property"); }
            set { Relationships.SetRelatedObject("Property", value); }
        }

        public virtual IDMRelationship Relationship
        {
            get { return Relationships.GetRelatedObject<DMRelationship>("Relationship"); }
            set { Relationships.SetRelatedObject("Relationship", value); }
        }

        #region Implementation of IMergableBo

        ChangeType IMergableBo.PendingChangeType
        {
            get;
            set;
        }

        bool IMergableBo.PendingChangeApproved
        {
            get { return true; }
            set { }
        }

        public bool IsOwnerPropCompulsory
        {
            get
            {
                return this.OwnerProperty != null && this.OwnerProperty.IsCompulsory;
            }
        }

        #endregion
    }
}