using System.Linq;
using FireStarter.Base;
using Habanero.Util;

namespace FireStarterModeller.BO
{
    public class RelationshipNameGenerator : IRelationshipNameGenerator
    {
        public string CreateRelationshipName(IDMRelationship relationship)
        {
            if (!string.IsNullOrEmpty(relationship.RelationshipName)) return relationship.RelationshipName;
            return RenameRelationship(relationship);
        }

        public string RenameRelationship(IDMRelationship relationship)
        {
            if (relationship.IsMultiple)
            {
                return StringUtilities.Pluralize(relationship.RelatedClassName);
            }
            if (relationship.RelationshipProperties == null) return relationship.RelatedClassName;
            //If this is a one to one and the relationship has the Primary
            // Key then use the Related Class Name
            // Otherwise use the PropertyName.
            if (relationship.HasAllPkProps() || relationship.RelationshipProperties.Count == 0)
            {
                return relationship.RelatedClassName;
            }
            string ownerPropertyName = relationship.RelationshipProperties.First().OwnerPropertyName;
            if (string.IsNullOrEmpty(ownerPropertyName))
            {
                return relationship.RelatedClassName;
            }
            return ownerPropertyName.EndsWith("ID")
                       ? RemoveIDSuffix(ownerPropertyName)
                       : ownerPropertyName;
        }

        private static string RemoveIDSuffix(string parentPropertyName)
        {
            return parentPropertyName.Substring(0, parentPropertyName.LastIndexOf("ID"));
        }
    }
}