using System;
using System.IO;
using FireStarterModeller.BO;
//using FireStarterModeller.Logic.LGMISServiceReference;
using Habanero.Base;
using Habanero.BO;
using Habanero.Util;

namespace FireStarterModeller.Logic
{
    public class ConfigurationExporter
    {
        private readonly string _copyFile = @"C:\systems\SITA\TraditionalAffairs\SerialiseData\FireStarterModeller.db4o";

        private readonly string _sourceFile = @"C:\systems\Habanero Tools\FireStarter_Modeller\trunk\source\FireStarterModeller\bin\Debug\SerialiseData\FireStarterModeller.db4o";

        /// <summary>
        /// Provides a delegate to Update the BusinessObject with appropriate values during publishing
        /// </summary>
        public delegate void UpdateBusinessObjectStatus(IBusinessObject businessObject);

        private UpdateBusinessObjectStatus _updateBOStatusDelegate; //= this.UpdateBoStatusToNew;
        private readonly string _backUpFile;

        public ConfigurationExporter()
        {
            _copyFile = DataFileSettings.Default.CopyFile;
            _sourceFile = DataFileSettings.Default.SourceFile;
            //IDictionary configuration = (IDictionary)ConfigurationManager.GetSection("DataFiles");
            //_copyFile = (string) configuration["copyFile"];
            //_sourceFile = (string)configuration["sourceFile"];
            _updateBOStatusDelegate = this.UpdateBoStatusToNew;
            _backUpFile = _copyFile + ".bak" + DateTime.Today.ToString("yyyymmdd");
        }

        public void ExportSolution(DMSolution solution, bool callWebService)
        {
//            solution.Save();
            //The following line is a hack we seemed to have a problem where all the related objects
            // were not loading up when required.
//            SolutionLoader.LoadAllSolutionBusinessObjects(solution);
            foreach (DMClass dmClass in solution.AllClasses)
            {
                this.CreateNewUIForm(dmClass);
            }
            if (File.Exists(_copyFile))
            {
                //File.Copy(_copyFile, _backUpFile, true);
                //SetupExportDataAccessor();
                DeleteSolutionIfExists(solution);
                UpdateBOStatus(solution, this.UpdateBoStatusToNew);
                solution.Save();
            }else
            {
                File.Copy(_sourceFile,_copyFile);
            }
//            if (callWebService)
//            {
//                LGMISWebServiceSoapClient lgmisServiceClient = new LGMISWebServiceSoapClient();
//                lgmisServiceClient.ReloadClassDefs();
//            }

        }

        public void UpdateBOStatus(DMSolution solution, UpdateBusinessObjectStatus updateBOStatusDelegate)
        {
            if (solution == null) throw new ArgumentNullException("solution");
            if (updateBOStatusDelegate == null) throw new ArgumentNullException("updateBOStatusDelegate");
            _updateBOStatusDelegate = updateBOStatusDelegate;
            _updateBOStatusDelegate(solution);
            UpdatePropertyTypes(solution);
            UpdateGridColumnControlTypes(solution);
            UpdateControlTypes(solution);
            UpdateFilterControlTypes(solution);
            UpdateUIControlMapperTypes(solution);
            UpdateModules(solution);
        }

        //private void SetupExportDataAccessor()
        //{
        //    DB4ORegistry.DB.Close();
        //    Db4oFactory.Configure().ObjectClass(typeof (BusinessObjectDTO)).CascadeOnUpdate(true);
        //    DB4ORegistry.DB = Db4oFactory.OpenFile(_copyFile);
        //    BORegistry.DataAccessor = new DataAccessorDB4O(DB4ORegistry.DB);
        //}

        //public void RestoreDataAccessor()
        //{ 
        //    DB4ORegistry.DB.Close();
        //    Db4oFactory.Configure().ObjectClass(typeof (BusinessObjectDTO)).CascadeOnUpdate(true);
        //    DB4ORegistry.DB = Db4oFactory.OpenFile(_sourceFile);
        //    BORegistry.DataAccessor = new DataAccessorDB4O(DB4ORegistry.DB);
        //}

        private void DeleteSolutionIfExists(DMSolution solution)
        {
            BusinessObjectManager.Instance.ClearLoadedObjects();
            DMSolution existingSolution = null;
            try
            {
                existingSolution = Broker.GetBusinessObject<DMSolution>(solution.ID);
            }
            catch (BusObjDeleteConcurrencyControlException)
            {
            }
            if (existingSolution != null)
            {
//                SolutionLoader.LoadAllSolutionBusinessObjects(existingSolution);
                BusinessObjectCollection<DMAssembly> modules = existingSolution.Assemblies;
                ITransactionCommitter committer = BORegistry.DataAccessor.CreateTransactionCommitter();
                DeleteModules(modules, committer);
                existingSolution.MarkForDelete();
                committer.AddBusinessObject(existingSolution);
                committer.CommitTransaction();
                BusinessObjectManager.Instance.ClearLoadedObjects();
            }
        }

        public static void UpdateBOAndGeneratedPropToDirty(IBusinessObject businessObject)
        {
            if (businessObject != null)
            {
                //                BOStatus boStatus = (BOStatus)businessObject.Status;
                //                ReflectionUtilities.SetPropertyValue(boStatus, "IsDirty", true);
                if (businessObject.Props.Contains("Generated"))
                {
                    IBOProp generatedProp = businessObject.Props["Generated"];
                    generatedProp.Value = false;
                    generatedProp.BackupPropValue();
                    generatedProp.Value = true;
                }
            }
        }
        private void DeleteModules(BusinessObjectCollection<DMAssembly> modules, ITransactionCommitter committer)
        {
            for (int index = modules.Count - 1; index >= 0; index--)
            {
                DMAssembly module = modules[index];
                BusinessObjectCollection<DMClass> entities = module.Classes;

                MarkEntitiesAsNotGenerated(entities, committer);
                module.MarkForDelete();
                committer.AddBusinessObject(module);
            }
        }

        private void MarkEntitiesAsNotGenerated(BusinessObjectCollection<DMClass> entities, ITransactionCommitter committer)
        {
            for (int i = entities.Count - 1; i >= 0; i--)
            {
                DMClass entity = entities[i];

                //                MarkRelationshipsAsNotGenerated(entity.UIViews);
                MarkRelationshipsAsNotGenerated(entity.OwnerRelationships);
                MarkRelationshipsAsNotGenerated(entity.RelatedRelationships);
                MarkUniqueConstraintsAsNotGenerated(entity.UniqueConstraints);
                MarkPropsAsNotGenerated(entity.Properties);
                entity.Generated = false;
                entity.MarkForDelete();
                committer.AddBusinessObject(entity);
//                entity.Save();
            }
        }

        private void MarkUniqueConstraintsAsNotGenerated(BusinessObjectCollection<DMUniqueConstraint> uniqueConstraints)
        {
            for (int counter = uniqueConstraints.Count - 1; counter >= 0; counter--)
            {
                DMUniqueConstraint uniqueConstraint = uniqueConstraints[counter];
                uniqueConstraint.Generated = false;
                uniqueConstraint.MarkForDelete();
//                uniqueConstraint.Save();
            }
        }
//
//        private void DeleteUIViews(BusinessObjectCollection<UIView> uiViews)
//        {
//            for (int uiViewCounter = uiViews.Count - 1; uiViewCounter >= 0; uiViewCounter--)
//            {
//                UIView uiView = uiViews[uiViewCounter];
//                DeleteUIGridInfo(uiView.UIGridInfo);
//                DeleteUIFormInfo(uiView.UIFormInfo);
//                uiView.MarkForDelete();
//                uiView.Save();
//            }
//        }

//        private void DeleteUIFormInfo(UIFormInfo uiFormInfo)
//        {
//            uiFormInfo.MarkForDelete();
//            uiFormInfo.Save();
//        }
//
//        private void DeleteUIGridInfo(UIGridInfo uiGridInfo)
//        {
//            uiGridInfo.MarkForDelete();
//            uiGridInfo.Save();
//        }
//
//        private void DeleteGridColumnInfos(BusinessObjectCollection<UIGridColumnControlType> gridColumnControlTypes)
//        {
//            foreach (UIGridColumnControlType uiGridColumnControlType in gridColumnControlTypes)
//            {
//                BusinessObjectCollection<UIGridColumnInfo> gridColumnInfos = uiGridColumnControlType.UIColumns;
//                for (int gridColumnInfoCounter = gridColumnInfos.Count - 1;
//                     gridColumnInfoCounter >= 0;
//                     gridColumnInfoCounter--)
//                {
//                    UIGridColumnInfo gridColumnInfo = gridColumnInfos[gridColumnInfoCounter];
//                    gridColumnInfo.MarkForDelete();
//                    gridColumnInfo.Save();
//                }
//            }
//        }


        private static void MarkPropsAsNotGenerated(BusinessObjectCollection<DMProperty> properties)
        {
            for (int propertyCounter = properties.Count - 1; propertyCounter >= 0; propertyCounter--)
            {
                DMProperty property = properties[propertyCounter];
                property.Generated = false;
//                property.MarkForDelete();
//                property.Save();
            }
        }

        private static void MarkRelationshipsAsNotGenerated(BusinessObjectCollection<DMRelationship> relatedRelationships)
        {
            for (int relatedRelationshipCounter = relatedRelationships.Count - 1;
                 relatedRelationshipCounter >= 0;
                 relatedRelationshipCounter--)
            {
                DMRelationship dmRelationship = relatedRelationships[relatedRelationshipCounter];
                dmRelationship.Generated = false;
                dmRelationship.MarkForDelete();
//                dmRelationship.Save();
            }
        }
//
//        private static void DeleteOwnerRelationships(BusinessObjectCollection<DMRelationship> ownerRelationships)
//        {
//            for (int ownerRelationshipCounter = ownerRelationships.Count - 1;
//                 ownerRelationshipCounter >= 0;
//                 ownerRelationshipCounter--)
//            {
//                DMRelationship dmRelationship = ownerRelationships[ownerRelationshipCounter];
//                dmRelationship.Generated = false;
////                dmRelationship.MarkForDelete();
////                dmRelationship.Save();
//            }
//        }

        private void UpdatePropertyTypes(DMSolution solution)
        {
            if (solution != null)
            {
                MultipleRelationship<DMPropertyType> propertyTypeRelationship =
                    (MultipleRelationship<DMPropertyType>) solution.Relationships["PropertyTypes"];
                BusinessObjectCollection<DMPropertyType> propertyTypes =
                    propertyTypeRelationship.CurrentBusinessObjectCollection;
                foreach (DMPropertyType propertyType in propertyTypes)
                {
                    _updateBOStatusDelegate(propertyType);
                }
            }
        }

        private void UpdateGridColumnControlTypes(DMSolution solution)
        {
            if (solution != null)
            {
                MultipleRelationship<UIGridColumnControlType> uiGridColumnControlTypesRelationship =
                    (MultipleRelationship<UIGridColumnControlType>) solution.Relationships["UIGridColumnControlTypes"];
                BusinessObjectCollection<UIGridColumnControlType> uiGridColumnControlTypes =
                    uiGridColumnControlTypesRelationship.CurrentBusinessObjectCollection;
                foreach (UIGridColumnControlType uiGridColumnControlType in uiGridColumnControlTypes)
                {
                    _updateBOStatusDelegate(uiGridColumnControlType);
                }
            }
        }

        private void UpdateControlTypes(DMSolution solution)
        {
            if (solution != null)
            {
                MultipleRelationship<UIControlType> uiControlTypesRelationship =
                    (MultipleRelationship<UIControlType>) solution.Relationships["UIControlTypes"];
                BusinessObjectCollection<UIControlType> uiControlTypes =
                    uiControlTypesRelationship.CurrentBusinessObjectCollection;
                foreach (UIControlType uiControlType in uiControlTypes)
                {
                    _updateBOStatusDelegate(uiControlType);
                }
            }
        }

        private void UpdateFilterControlTypes(DMSolution solution)
        {
            if (solution != null)
            {
                MultipleRelationship<UITypeDefinition> uiTypeDefinitionRelationship =
                    (MultipleRelationship<UITypeDefinition>) solution.Relationships["UIFilterControlTypes"];
                BusinessObjectCollection<UITypeDefinition> uiTypeDefinitions =
                    uiTypeDefinitionRelationship.CurrentBusinessObjectCollection;
                foreach (UITypeDefinition uiTypeDefinition in uiTypeDefinitions)
                {
                    _updateBOStatusDelegate(uiTypeDefinition);
                }
            }
        }

        private void UpdateUIControlMapperTypes(DMSolution solution)
        {
            if (solution != null)
            {
                MultipleRelationship<UIControlMapperType> controlMapperTypeRelationship =
                    (MultipleRelationship<UIControlMapperType>) solution.Relationships["UIControlMapperTypes"];
                BusinessObjectCollection<UIControlMapperType> controlMapperTypes =
                    controlMapperTypeRelationship.CurrentBusinessObjectCollection;
                foreach (UIControlMapperType controlMapperType in controlMapperTypes)
                {
                    _updateBOStatusDelegate(controlMapperType);
                }
            }
        }


        private void UpdateModules(DMSolution solution)
        {
            if (solution != null)
            {
                MultipleRelationship<DMAssembly> moduleRelationship =
                    (MultipleRelationship<DMAssembly>) solution.Relationships["Assemblies"];
                BusinessObjectCollection<DMAssembly> assemblies = moduleRelationship.CurrentBusinessObjectCollection;
                foreach (DMAssembly assembly in assemblies)
                {
                    _updateBOStatusDelegate(assembly);
                    UpdateClasses(assembly);
                }
            }
        }

        private void UpdateClasses(DMAssembly assembly)
        {
            if (assembly != null)
            {
                MultipleRelationship<DMClass> entityRelationship =
                    (MultipleRelationship<DMClass>) assembly.Relationships["Classes"];
                BusinessObjectCollection<DMClass> classes = entityRelationship.CurrentBusinessObjectCollection;
                foreach (DMClass dmClass in classes)
                {
                    _updateBOStatusDelegate(dmClass);
//                    dmClass.Generated = true;
                    UpdateProperties(dmClass);
                    SingleRelationship<DMObjectIdentity> relationship =
                        (SingleRelationship<DMObjectIdentity>) dmClass.Relationships["ObjectIdentity"];
                    DMObjectIdentity objectIdentity = relationship.GetRelatedObject();
                    if (objectIdentity != null)
                    {
                        _updateBOStatusDelegate(objectIdentity);
//                        objectIdentity.Generated = true;
                    }
                    UpdateUniqueConstraints(dmClass);
                    UpdateOwnerRelationship(dmClass);
                    UpdateRelatedRelationship(dmClass);
                    UpdateUIViews(dmClass);
                }
            }
        }

        private void UpdateProperties(DMClass dmClass)
        {
            if (dmClass != null)
            {
                MultipleRelationship<DMProperty> propertyRelationship =
                    (MultipleRelationship<DMProperty>) dmClass.Relationships["Properties"];
                BusinessObjectCollection<DMProperty> properties = propertyRelationship.CurrentBusinessObjectCollection;
                foreach (DMProperty dmProperty in properties)
                {
                    _updateBOStatusDelegate(dmProperty);
//                    dmProperty.Generated = true;
                    UpdateDefaultPropRule(dmProperty);
                }
            }
        }

        private void UpdateDefaultPropRule(DMProperty dmProperty)
        {
            if (dmProperty != null)
            {
                SingleRelationship<DMPropRule> defaultPropRuleRelationship =
                    (SingleRelationship<DMPropRule>) dmProperty.Relationships["DefaultPropRule"];
                DMPropRule propRule = defaultPropRuleRelationship.GetRelatedObject();
                _updateBOStatusDelegate(propRule);
                if (propRule != null)
                {
                    MultipleRelationship<DMPropRuleParameter> relationship =
                        (MultipleRelationship<DMPropRuleParameter>) propRule.Relationships["PropRuleParameters"];
                    foreach (DMPropRuleParameter propRuleParameter in relationship.CurrentBusinessObjectCollection)
                    {
                        _updateBOStatusDelegate(propRuleParameter);
                    }
                }
            }
        }

        private void UpdateUniqueConstraints(DMClass dmClass)
        {
            if (dmClass != null)
            {
                MultipleRelationship<DMUniqueConstraint> ucRelationship =
                    (MultipleRelationship<DMUniqueConstraint>) dmClass.Relationships["UniqueConstraints"];
                BusinessObjectCollection<DMUniqueConstraint> dmUniqueConstraints =
                    ucRelationship.CurrentBusinessObjectCollection;
                foreach (DMUniqueConstraint dmUniqueConstraint in dmUniqueConstraints)
                {
                    _updateBOStatusDelegate(dmUniqueConstraint);
                    UpdateUniqueConstraintProps(dmUniqueConstraint);
                }
            }
        }

        private void UpdateUniqueConstraintProps(DMUniqueConstraint constraint)
        {
            if (constraint != null)
            {
                MultipleRelationship<DMUniqueConstraintProperty> ucPropRelationship =
                    (MultipleRelationship<DMUniqueConstraintProperty>)
                    constraint.Relationships["UniqueConstraintProperties"];
                BusinessObjectCollection<DMUniqueConstraintProperty> dmUniqueConstraintProperties =
                    ucPropRelationship.CurrentBusinessObjectCollection;
                foreach (DMUniqueConstraintProperty uniqueConstraintProperty in dmUniqueConstraintProperties)
                {
                    _updateBOStatusDelegate(uniqueConstraintProperty);
                }
            }
        }

        private void UpdateOwnerRelationship(DMClass dmClass)
        {
            if (dmClass != null)
            {
                MultipleRelationship<DMRelationship> ownerMultipleRelationship =
                    (MultipleRelationship<DMRelationship>) dmClass.Relationships["OwnerRelationships"];
                BusinessObjectCollection<DMRelationship> ownerRelationships =
                    ownerMultipleRelationship.CurrentBusinessObjectCollection;
                foreach (DMRelationship ownerRelationship in ownerRelationships)
                {
                    _updateBOStatusDelegate(ownerRelationship);
//                    ownerRelationship.Generated = true;
                    UpdateRelationshipProps(ownerRelationship);
                }
            }
        }

        private void UpdateRelatedRelationship(DMClass dmClass)
        {
            if (dmClass != null)
            {
                MultipleRelationship<DMRelationship> relatedMultipleRelationship =
                    (MultipleRelationship<DMRelationship>) dmClass.Relationships["RelatedRelationships"];
                BusinessObjectCollection<DMRelationship> relatedRelationships =
                    relatedMultipleRelationship.CurrentBusinessObjectCollection;
                foreach (DMRelationship relationship in relatedRelationships)
                {
                    _updateBOStatusDelegate(relationship);
//                    relationship.Generated = true;
                    UpdateRelationshipProps(relationship);
                }
            }
        }

        private void UpdateRelationshipProps(DMRelationship relationship)
        {
            if (relationship != null)
            {
                MultipleRelationship<DMRelationshipProperty> relationshipPropertiesRelationship =
                    (MultipleRelationship<DMRelationshipProperty>) relationship.Relationships["RelationshipProperties"];
                BusinessObjectCollection<DMRelationshipProperty> relationshipProperties =
                    relationshipPropertiesRelationship.CurrentBusinessObjectCollection;
                foreach (DMRelationshipProperty relationshipProperty in relationshipProperties)
                {
                    _updateBOStatusDelegate(relationshipProperty);
//                    relationshipProperty.Generated = true;
                }
            }
        }

        private void UpdateUIViews(DMClass dmClass)
        {
            if (dmClass != null)
            {
                MultipleRelationship<UIView> uiViewRelationship =
                    (MultipleRelationship<UIView>) dmClass.Relationships["UIViews"];
                BusinessObjectCollection<UIView> uiViews = uiViewRelationship.CurrentBusinessObjectCollection;
                foreach (UIView uiView in uiViews)
                {
                    _updateBOStatusDelegate(uiView);
                    UpdateUIForm(uiView);
                    UpdateUIGrid(uiView);
                }
            }
        }

        private void UpdateUIGrid(UIView uiView)
        {
            if (uiView != null)
            {
                SingleRelationship<UIGridInfo> relationship =
                    (SingleRelationship<UIGridInfo>) uiView.Relationships["UIGridInfo"];
                UIGridInfo uiGridInfo = relationship.GetRelatedObject();
                _updateBOStatusDelegate(uiGridInfo);
                UpdateUIColumns(uiGridInfo);
                UpdateUIGridFilter(uiGridInfo);
            }
        }

        private void UpdateUIGridFilter(UIGridInfo uiGridInfo)
        {
            if (uiGridInfo != null)
            {
                SingleRelationship<UIGridFilter> relationship =
                    (SingleRelationship<UIGridFilter>) uiGridInfo.Relationships["UIGridFilter"];
                UIGridFilter uiGridFilter = relationship.GetRelatedObject();
                _updateBOStatusDelegate(uiGridFilter);
                UpdateUIGridFilterProps(uiGridFilter);
            }
        }

        private void UpdateUIGridFilterProps(UIGridFilter uiGridFilter)
        {
            if (uiGridFilter != null)
            {
                MultipleRelationship<UIGridFilterProp> relationship =
                    (MultipleRelationship<UIGridFilterProp>) uiGridFilter.Relationships["UIGridFilterProps"];
                BusinessObjectCollection<UIGridFilterProp> gridFilterProps =
                    relationship.CurrentBusinessObjectCollection;
                foreach (UIGridFilterProp uiGridFilterProp in gridFilterProps)
                {
                    _updateBOStatusDelegate(uiGridFilterProp);
                    MultipleRelationship<UIParameter> uiParameterRelationship =
                        (MultipleRelationship<UIParameter>) uiGridFilterProp.Relationships["UIParameters"];
                    BusinessObjectCollection<UIParameter> uiParameters =
                        uiParameterRelationship.CurrentBusinessObjectCollection;
                    UpdateUIParameters(uiParameters);
                }
            }
        }

        private void UpdateUIColumns(UIGridInfo uiGridInfo)
        {
            if (uiGridInfo != null)
            {
                MultipleRelationship<UIGridColumnInfo> relationship =
                    (MultipleRelationship<UIGridColumnInfo>) uiGridInfo.Relationships["UIColumns"];
                BusinessObjectCollection<UIGridColumnInfo> gridColumnInfos =
                    relationship.CurrentBusinessObjectCollection;
                foreach (UIGridColumnInfo uiGridColumnInfo in gridColumnInfos)
                {
                    _updateBOStatusDelegate(uiGridColumnInfo);
                    MultipleRelationship<UIParameter> uiParameterRelationship =
                        (MultipleRelationship<UIParameter>) uiGridColumnInfo.Relationships["UIParameters"];
                    BusinessObjectCollection<UIParameter> uiParameters =
                        uiParameterRelationship.CurrentBusinessObjectCollection;
                    UpdateUIParameters(uiParameters);
                }
            }
        }

        private void UpdateUIParameters(BusinessObjectCollection<UIParameter> uiParameters)
        {
            foreach (UIParameter uiParameter in uiParameters)
            {
                _updateBOStatusDelegate(uiParameter);
            }
        }

        private void UpdateUIForm(UIView uiView)
        {
            if (uiView != null)
            {
                SingleRelationship<UIFormInfo> uiFormRelationship =
                    (SingleRelationship<UIFormInfo>) uiView.Relationships["UIFormInfo"];
                UIFormInfo uiFormInfo = uiFormRelationship.GetRelatedObject();
                _updateBOStatusDelegate(uiFormInfo);
                UpdateUITabs(uiFormInfo);
            }
        }

        private void UpdateUITabs(UIFormInfo uiFormInfo)
        {
            if (uiFormInfo != null)
            {
                MultipleRelationship<UITab> uiViewRelationship =
                    (MultipleRelationship<UITab>) uiFormInfo.Relationships["UITabs"];
                BusinessObjectCollection<UITab> uiTabs = uiViewRelationship.CurrentBusinessObjectCollection;
                foreach (UITab uiTab in uiTabs)
                {
                    _updateBOStatusDelegate(uiTab);
                    UpdateUIColumnLayouts(uiTab);
                }
            }
        }

        private void UpdateUIColumnLayouts(UITab uiTab)
        {
            if (uiTab != null)
            {
                MultipleRelationship<UIColumnLayout> relationship =
                    (MultipleRelationship<UIColumnLayout>) uiTab.Relationships["UIColumnLayouts"];
                BusinessObjectCollection<UIColumnLayout> uiColumnLayouts = relationship.CurrentBusinessObjectCollection;
                foreach (UIColumnLayout uiColumnLayout in uiColumnLayouts)
                {
                    _updateBOStatusDelegate(uiColumnLayout);
                    UpdateUIFields(uiColumnLayout);
                }
            }
        }

        private void UpdateUIFields(UIColumnLayout columnLayout)
        {
            if (columnLayout != null)
            {
                MultipleRelationship<UIField> relationship =
                    (MultipleRelationship<UIField>) columnLayout.Relationships["UIFields"];
                BusinessObjectCollection<UIField> uiFields = relationship.CurrentBusinessObjectCollection;
                foreach (UIField uiField in uiFields)
                {
                    _updateBOStatusDelegate(uiField);
                }
            }
        }


        private void UpdateBoStatusToNew(IBusinessObject businessObject)
        {
            if (businessObject != null)
            {
                BOStatus boStatus = (BOStatus) businessObject.Status;
                ReflectionUtilities.SetPropertyValue(boStatus, "IsNew", true);
                ReflectionUtilities.SetPropertyValue(boStatus, "IsDirty", true);
                if (boStatus.IsDeleted)
                {
                    ReflectionUtilities.SetPropertyValue(boStatus, "IsDeleted", false);
                }
                if (businessObject.Props.Contains("Generated"))
                {
                    IBOProp generatedProp = businessObject.Props["Generated"];
                    generatedProp.Value = true;
                }
            }
        }

        public void CreateNewUIForm(DMClass dmClass)
        {
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            UIBOEditorCreator boEditorCreator = new UIBOEditorCreator();
            if (dmClass.UIViews.Count == 0 )
            {
                //TODO: Throw some error
                return;
            }
            UIView view = dmClass.UIViews[0];
            UIFormInfo formInfo = view.UIFormInfo;
            ITransactionCommitter committer = BORegistry.DataAccessor.CreateTransactionCommitter();
            if (formInfo != null)
            {
                formInfo.MarkForDelete();
                committer.AddBusinessObject(formInfo);
            }
            UIFormInfo createdFormInfo =  boEditorCreator.CreateBOEditor(dmClass);
            view.UIFormInfo = createdFormInfo;
            committer.AddBusinessObject(createdFormInfo);
            committer.CommitTransaction();
        }
    }
}