using System;
using System.Collections.Generic;
using System.Linq;
using FireStarterModeller.BO;
using Habanero.BO;

namespace FireStarterModeller.Logic.Wizards
{
    public class DMClassDeleter
    {
        public IDMClass ClassDM { get; private set; }

        public IEnumerable<IDMRelationship> AllRelationshipWithRelatedFKs
        {
            get { return this.ClassDM.RelationshipsDM
                    .Where(relationship => relationship.RelatedBOHasForeignKey)
                    .Cast<IDMRelationship>(); }
        }

        public IList<IDMRelationship> RelationshipsWithFKToBeDeleted
        {
            get; private set;
        }

        public IEnumerable<DMLookupListBusinessObject> AllLookupLists
        {
            get { return this.ClassDM.DMLookupListBusinessObjects; }
        }

        public IList<DMLookupListBusinessObject> LookupListsWithPropsToBeDeleted
        {
            get; private set;
        }

        public DMClassDeleter(IDMClass dmClass)
        {
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            this.ClassDM = dmClass;
            RelationshipsWithFKToBeDeleted = new List<IDMRelationship>();
            LookupListsWithPropsToBeDeleted = new List<DMLookupListBusinessObject>();
        }
        /// <summary>
        /// Deletes the DMClass and all its Related Business Objects
        /// </summary>
        public void Delete()
        {


            var relPropsToBeDeleted = this.RelationshipsWithFKToBeDeleted
                    .SelectMany(relationship => relationship.RelationshipProperties);
            foreach (var relationshipProperty in relPropsToBeDeleted.ToList())
            {
                new DMRelationshipPropertyDeleter(relationshipProperty).Delete();
            }
            foreach (var dmRelationship in this.ClassDM.RelationshipsDM.ToArray())
            {
                dmRelationship.MarkForDelete();
            }
            var lookupPropsToBeDeleted = this.LookupListsWithPropsToBeDeleted
                    .Select(o => o.DMProperty)
                    .Where(prop => prop != null);
            foreach (var prop in lookupPropsToBeDeleted.ToList())
            {
                prop.MarkForDelete();
            }
            foreach (var lookupList in this.ClassDM.DMLookupListBusinessObjects.ToArray())
            {
                lookupList.MarkForDelete();
            }
            if (this.ClassDM.UniqueConstraints != null)
            {
                foreach (var uniqueConstraint in this.ClassDM.UniqueConstraints.ToArray())
                {
                    uniqueConstraint.MarkForDelete();
                }
            }
            this.ClassDM.MarkForDelete();
        }
        /// <summary>
        /// Generally Based on Some User Interface the user can select which 
        /// </summary>
        /// <param name="relationship"></param>
        public void AddRelationshipWithFKToBeDeleted(IDMRelationship relationship)
        {
            RelationshipsWithFKToBeDeleted.Add(relationship);
        }

        public void AddLookupListsWithPropsToBeDeleted(DMLookupListBusinessObject lookupList)
        {
            LookupListsWithPropsToBeDeleted.Add(lookupList);
        }

        public void MarkAllRelationshipsFKsToBeDeleted()
        {
            foreach (var relationship in AllRelationshipWithRelatedFKs)
            {
                this.AddRelationshipWithFKToBeDeleted(relationship);
            }
        }

        public void MarkAllLookupListsPropsToBeDeleted()
        {
            foreach (var lookupList in AllLookupLists)
            {
                this.AddLookupListsWithPropsToBeDeleted(lookupList);
            }
        }
    }
}