using System;
using FireStarterModeller.BO;
using FireStarterModeller.Logic;
using FireStarterModeller.Test.BO;
using Habanero.Base.Exceptions;
using NUnit.Framework;

namespace FireStarterModeller.Test.Logic
{
    [TestFixture]
    public class TestCreateUIViewFromDomainModel
    {
        #region Setup/Teardown

        [TestFixtureSetUp]
        public virtual void TestFixtureSetup()
        {
            TestUtilsLogic.SetupFixture();
        }

        [SetUp]
        public virtual void SetupTest()
        {
            TestUtilsLogic.SetupTest();
        }

        [TearDown]
        public virtual void TearDownTest(){}
        #endregion

        [Test]
        public void Test_CreateField_WhenPropNull_ShouldRaiseError()
        {
            //---------------Set up test pack-------------------
            UIViewCreator uiViewCreator = new UIViewCreator();
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            try
            {
                uiViewCreator.CreateUIView(null);
                Assert.Fail("expected ArgumentNullException");
            }
                //---------------Test Result -----------------------
            catch (ArgumentNullException ex)
            {
                StringAssert.Contains("Value cannot be null", ex.Message);
                StringAssert.Contains("dmClass", ex.ParamName);
            }
        }

        [Test]
        public void Test_CreateField_WhenClassSolutionNull_ShouldRaiseError()
        {
            //---------------Set up test pack-------------------
            DMClass dmClass = TestUtilsDMClass.CreateUnsavedDefaultDMClass();
            dmClass.ClassNameBO = TestUtilsShared.GetRandomString();
            UIViewCreator uiViewCreator = new UIViewCreator();
            //---------------Assert Precondition----------------
            Assert.IsNull(dmClass.Solution);
            //---------------Execute Test ----------------------
            try
            {
                uiViewCreator.CreateUIView(dmClass);
                Assert.Fail("expected HabaneroArgumentExceptifsdon");
            }
                //---------------Test Result -----------------------
            catch (HabaneroArgumentException ex)
            {
                StringAssert.Contains("The argument 'dmClass.Solution' is not valid. You cannot " 
                        + "create a UIView for a DMClass that is not associated with a Solution - (DMClass ", ex.Message);
            }
        }

        [Test]
        public void Test_CreateField_WhenString_ShouldHaveTextBoxPropertyAndControlType()
        {
            //---------------Set up test pack-------------------
            SolutionCreator creator = new SolutionCreator();
            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass(creator.CreateSolution());
            property.OrdinalPosition = 3;
            TestUtilsDMProperty.SetDMPropStringType(property);
            DMClass dmClass = (DMClass) property.Class;
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass, false);
            TestUtilsDMProperty.CreateUnsavedValidDMProperty(dmClass, false);
            UIViewCreator uiViewCreator = new UIViewCreator();
            //---------------Assert Precondition----------------
            UICreatorTestUtils.AssertPropertyTypeIsString(property);
            Assert.IsNotNull(property.OrdinalPosition);
            Assert.Greater(property.OrdinalPosition, 0);
            Assert.AreEqual(0, dmClass.UIViews.Count);
            Assert.AreEqual(3, dmClass.Properties.Count);
            //---------------Execute Test ----------------------
            UIView uiView = uiViewCreator.CreateUIView(dmClass);
            //---------------Test Result -----------------------
            Assert.IsNotNull(uiView);
            Assert.IsNotNull(uiView.Class);
            Assert.AreSame(dmClass, uiView.Class);
            Assert.AreEqual(1, dmClass.UIViews.Count);

            //----------------Verify BOEditor Created Correctly---------
            UIFormInfo formInfo = uiView.UIFormInfo;
            Assert.IsNotNull(formInfo);
            Assert.AreEqual(3, formInfo.UIFields.Count);

            //---------------Verify GridFilter Created Correctly-------
            UIGridFilter gridFilter = uiView.UIGridInfo.UIGridFilter;
            Assert.IsNotNull(gridFilter);
            Assert.AreEqual(3, gridFilter.UIGridFilterProps.Count);

            //---------------Verify GridInfo Created Correctly---------
            UIGridInfo gridInfo = uiView.UIGridInfo;
            Assert.IsNotNull(gridInfo);
            Assert.AreEqual(3, gridInfo.UIColumns.Count);

        }
//        [Test]
//        public void Test_CreateField_WhenInt_ShouldHaveTextBoxType()
//        {
//            //---------------Set up test pack-------------------
//            SolutionCreator creator = new SolutionCreator();
//            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass(creator.CreateSolution());
//            TestUtilsDMProperty.SetDMPropIntType(property);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            UICreatorTestUtils.AssertPropertyTypeIsInt(property);
//            //---------------Execute Test ----------------------
//            UIView uiView = uiViewCreator.CreateUIView(property);
//            //---------------Test Result -----------------------
//            Assert.IsNotNull(formField);
//            Assert.IsNotNull(formField.Property);
//            Assert.AreSame(property, formField.Property);
//
//            UICreatorTestUtils.AssertFieldControlTypeIsTextBox(formField.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateTwoFormFieldDefs_String_ShouldBothHaveSameControlType()
//        {
//            //---------------Set up test pack-------------------
//            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass();
//            TestUtilsDMProperty.SetDMPropStringType(property);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            UICreatorTestUtils.AssertPropertyTypeIsString(property);
//            //---------------Execute Test ----------------------
//            UIView uiView = uiViewCreator.CreateUIView(property);
//            UIView uiView2 = uiViewCreator.CreateUIView(property);
//            //---------------Test Result -----------------------
//            Assert.AreNotSame(formField, formField2);
//            Assert.AreSame(formField.UIControlType, formField2.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateTwoFormFieldDefs_Boolean_ShouldBothHaveSameControlType()
//        {
//            //---------------Set up test pack-------------------
//            SolutionCreator creator = new SolutionCreator();
//            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass(creator.CreateSolution());
//            TestUtilsDMProperty.SetDMPropBooleanType(property);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            UICreatorTestUtils.AssertPropertyTypeIsBool(property);
//            //---------------Execute Test ----------------------
//            UIView uiView = uiViewCreator.CreateUIView(property);
//            UIView uiView2 = uiViewCreator.CreateUIView(property);
//            //---------------Test Result -----------------------
//            Assert.AreNotSame(formField, formField2);
//            Assert.AreSame(formField.UIControlType, formField2.UIControlType);
//            UICreatorTestUtils.AssertControlTypeIsCheckBox(formField.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateField_Bool_ShouldBeCheckBoxControlType()
//        {
//            //---------------Set up test pack-------------------
//            SolutionCreator creator = new SolutionCreator();
//            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass(creator.CreateSolution());
//            TestUtilsDMProperty.SetDMPropBooleanType(property);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            UICreatorTestUtils.AssertPropertyTypeIsBool(property);
//            //---------------Execute Test ----------------------
//            UIView uiView = uiViewCreator.CreateUIView(property);
//            //---------------Test Result -----------------------
//            Assert.AreSame(property, formField.Property);
//            UICreatorTestUtils.AssertControlTypeIsCheckBox(formField.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateField_Lookup_ShouldBeComboBoxControlType()
//        {
//            //---------------Set up test pack-------------------
//            SolutionCreator creator = new SolutionCreator();
//            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass(creator.CreateSolution());
//            TestUtilsDMProperty.SetDMPropWithLookupList(property);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            UICreatorTestUtils.AssertPropertyTypeIsLookup(property);
//            //---------------Execute Test ----------------------
//            UIView uiView = uiViewCreator.CreateUIView(property);
//            //---------------Test Result -----------------------
//            UICreatorTestUtils.AssertControlTypeIsComboBox(formField, formField.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateTwoFormFieldDefs_Lookup_ShouldBothHaveSameControlType()
//        {
//            //---------------Set up test pack-------------------
//            SolutionCreator creator = new SolutionCreator();
//            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass(creator.CreateSolution());
//            TestUtilsDMProperty.SetDMPropWithLookupList(property);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            UICreatorTestUtils.AssertPropertyTypeIsLookup(property);
//            //---------------Execute Test ----------------------
//            UIView uiView = uiViewCreator.CreateUIView(property);
//            UIView uiView2 = uiViewCreator.CreateUIView(property);
//            //---------------Test Result -----------------------
//            Assert.AreNotSame(formField, formField2);
//            Assert.AreSame(formField.UIControlType, formField2.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateField_DateTime_ShouldBeDateTimeControlType()
//        {
//            //---------------Set up test pack-------------------
//            SolutionCreator creator = new SolutionCreator();
//            DMProperty property = TestUtilsDMProperty.CreateUnsavedDefaultDMPropertyWithClass(creator.CreateSolution());
//            TestUtilsDMProperty.SetDMPropWithDateTime(property);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            UICreatorTestUtils.AssertPropertyTypeIsDateTime(property);
//            //---------------Execute Test ----------------------
//            UIView uiView = uiViewCreator.CreateUIView(property);
//            //---------------Test Result -----------------------
//            UICreatorTestUtils.AssertControlTypeIsDateTimePicker(formField.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateBOEditorPanelInfo_OneProperty_ShouldHaveAppropriateControlInfo()
//        {
//            //---------------Set up test pack-------------------
//            DMClass dmClass = UICreatorTestUtils.CreateClassWithOneLookupProperty();
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            Assert.IsNotNull(dmClass);
//            Assert.AreEqual(1, dmClass.Properties.Count);
//            //---------------Execute Test ----------------------
//            UIFormInfo boEditorPanelInfo = uiViewCreator.CreateBOEditor(dmClass.Properties);
//            //---------------Test Result -----------------------
//            Assert.IsNotNull(boEditorPanelInfo);
//            Assert.AreEqual(1, boEditorPanelInfo.UITabs.Count);
//            UITab tab = boEditorPanelInfo.UITabs[0];
//            Assert.AreEqual(1, tab.UIColumnLayouts.Count);
//            UIColumnLayout layout = tab.UIColumnLayouts[0];
//            Assert.AreEqual(1, layout.UIFields.Count);
//            UIView uiView = layout.UIFields[0];
//            UICreatorTestUtils.AssertControlTypeIsComboBox(formField, formField.UIControlType);
//            Assert.IsFalse(boEditorPanelInfo.Generated.Value);
//        }
//
//        [Test]
//        public void Test_CreateBOEditor_TwoProperty_ShouldHaveAppropriateControlInfo()
//        {
//            //---------------Set up test pack-------------------
//            DMClass dmClass = UICreatorTestUtils.CreateClassWithOneLookupProperty();
//            DMProperty dmProperty = dmClass.Properties.CreateBusinessObject();
//            TestUtilsDMProperty.SetDMPropBooleanType(dmProperty);
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            Assert.IsNotNull(dmClass);
//            Assert.AreEqual(2, dmClass.Properties.Count);
//            //---------------Execute Test ----------------------
//            UIFormInfo boEditorPanelInfo = uiViewCreator.CreateBOEditor(dmClass.Properties);
//            //---------------Test Result -----------------------
//            Assert.IsNotNull(boEditorPanelInfo);
//            ReadOnlyCollection<UIField> fields = boEditorPanelInfo.UIFields;
//            Assert.AreEqual(2, fields.Count);
//            UIView uiView1 = fields[0];
//            UICreatorTestUtils.AssertControlTypeIsComboBox(formField1, formField1.UIControlType);
//            UIView uiView = fields[1];
//            UICreatorTestUtils.AssertControlTypeIsCheckBox(formField.UIControlType);
//        }
//
//        [Test]
//        public void Test_CreateBOEditor_TwoProperty_OneObjectID_ShouldNotHaveObjectIDControl()
//        {
//            //---------------Set up test pack-------------------
//            DMClass dmClass = UICreatorTestUtils.CreateClassWithOneLookupProperty();
//            DMProperty dmProperty = dmClass.Properties.CreateBusinessObject();
//            TestUtilsDMProperty.SetDMPropBooleanType(dmProperty);
//            dmClass.CreateObjectIdentity(dmClass.Properties.CreateBusinessObject());
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            Assert.IsNotNull(dmClass);
//            Assert.AreEqual(3, dmClass.Properties.Count);
//            //---------------Execute Test ----------------------
//            UIFormInfo boEditorPanelInfo = uiViewCreator.CreateBOEditor(dmClass.Properties);
//            //---------------Test Result -----------------------
//            Assert.IsNotNull(boEditorPanelInfo);
//            Assert.AreEqual(2, boEditorPanelInfo.UIFields.Count);
//        }
//
//        [Test]
//        public void Test_CreateBOEditor_TwoProperty_OneKeepValuePrivate_ShouldHaveKeepvaluePrivateControl()
//        {
//            //---------------Set up test pack-------------------
//            DMClass dmClass = UICreatorTestUtils.CreateClassWithOneLookupProperty();
//            DMProperty dmProperty = dmClass.Properties.CreateBusinessObject();
//            TestUtilsDMProperty.SetDMPropBooleanType(dmProperty);
//            DMProperty property = dmClass.Properties.CreateBusinessObject();
//            property.KeepValuePrivate = true;
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//            Assert.IsNotNull(dmClass);
//            Assert.AreEqual(3, dmClass.Properties.Count);
//            Assert.IsTrue(property.KeepValuePrivate.Value);
//            //---------------Execute Test ----------------------
//            UIFormInfo boEditorPanelInfo = uiViewCreator.CreateBOEditor(dmClass.Properties);
//            //---------------Test Result -----------------------
//            Assert.IsNotNull(boEditorPanelInfo);
//            Assert.AreEqual(3, boEditorPanelInfo.UIFields.Count);
//            //TODO Brett 02 Apr 2009: Make sure that the Keep Value private is a password TextBox.
//        }
//
//        [Test]
//        public void Test_CreateUIGridFilter_WhenPropertiesNull_ShouldRaiseError()
//        {
//            //---------------Set up test pack-------------------
//            UIViewCreator uiViewCreator = new UIViewCreator();
//            //---------------Assert Precondition----------------
//
//            //---------------Execute Test ----------------------
//            try
//            {
//                uiViewCreator.CreateBOEditor(null);
//                Assert.Fail("expected ArgumentNullException");
//            }
//                //---------------Test Result -----------------------
//            catch (ArgumentNullException ex)
//            {
//                StringAssert.Contains("Value cannot be null", ex.Message);
//                StringAssert.Contains("properties", ex.ParamName);
//            }
//        }
    }
    internal class UIViewCreator
    {
//        public UIField CreateUIView(DMProperty property)
//        {
//            if (property == null) throw new ArgumentNullException("property");
//            ValidatePropertySetupCorrectly(property);
//            UIView uiView = new UIField { Property = property };
//            formField.OrdinalPosition = property.OrdinalPosition;
//            formField.UIControlType = GetControlType(property);
//            return formField;
//        }
//
//        private static void ValidatePropertySetupCorrectly(DMProperty property)
//        {
//            if (property.Class == null)
//            {
//                throw new HabaneroArgumentException
//                    ("property.Class", "You cannot create a formField for a property that is not associated with a class (Property " + property.PropertyName + "'");
//            }
//            if (property.Class.Solution == null)
//            {
//                throw new HabaneroArgumentException
//                    ("property.Class.Solution", "You cannot create a formField for a property that is not associated with a Solution - Property '" + property.PropertyName + "'");
//            }
//        }
//
//        //Idea: This should be moved into some sort of a a mapper class? Brett 22 03 2009
//        private static UIControlType GetControlType(DMProperty property)
//        {
//            UIControlType type;
//            if (property.HasLookupList)
//            {
//                type = GetComboBoxControlType(property.Class.Solution);
//            }
//            else
//            {
//                switch (property.PropertyType.PropertyTypeName)
//                {
//                    case "Boolean":
//                        type = GetCheckBoxControlType(property.Class.Solution);
//                        break;
//                    case "DateTime":
//                        type = GetDateTimeControlType(property.Class.Solution);
//                        break;
//                    default:
//                        type = GetTextBoxControlType(property.Class.Solution);
//                        break;
//                }
//            }
//            return type;
//        }
//
//        private static UIControlType GetDateTimeControlType(DMSolution solution)
//        {
//            return GetControlType(solution, "System.Windows.Forms", "DateTimePicker");
//        }
//
//        private static UIControlType GetComboBoxControlType(DMSolution solution)
//        {
//            return GetControlType(solution, "System.Windows.Forms", "ComboBox");
//        }
//        private static UIControlType GetControlType(DMSolution solution, string assemblyName, string formFieldTypeName)
//        {
//            return solution.GetUIFormFieldControlType(assemblyName, formFieldTypeName);
//        }
//        private static UIControlType GetTextBoxControlType(DMSolution solution)
//        {
//            return GetControlType(solution, "System.Windows.Forms", "TextBox");
//        }
//        private static UIControlType GetCheckBoxControlType(DMSolution solution)
//        {
//            return GetControlType(solution, "System.Windows.Forms", "CheckBox");
//        }
//
//        public UIFormInfo CreateBOEditor(BusinessObjectCollection<DMProperty> properties)
//        {
//            if (properties == null) throw new ArgumentNullException("properties");
//            UIFormInfo boEditor = new UIFormInfo();
//            UITab tab = boEditor.UITabs.CreateBusinessObject();
//            UIColumnLayout columnLayout = tab.UIColumnLayouts.CreateBusinessObject();
//
//            foreach (DMProperty property in properties)
//            {
//                if (property.IsPartOfObjectIdentity) continue;
//                columnLayout.UIFields.Add(CreateUIView(property));
//            }
//            return boEditor;
//        }
        public UIView CreateUIView(DMClass dmClass)
        {
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            if (dmClass.Solution == null)
            {
                throw new HabaneroArgumentException
                    ("dmClass.Solution", "You cannot create a UIView for a " 
                    + "DMClass that is not associated with a Solution - (DMClass '" + dmClass.ClassNameBO + "')");
            }
            UIView view = new UIView();
            view.Class = dmClass;
            UIBOEditorCreator boEditorCreator = new UIBOEditorCreator();

            UIFormInfo boEditor = boEditorCreator.CreateBOEditor(dmClass);
            view.UIFormInfo = boEditor;
            
            UIGridCreator gridCreator = new UIGridCreator();
            UIGridInfo gridInfo = gridCreator.CreateGrid(dmClass, dmClass.Properties);
            view.UIGridInfo = gridInfo;

            UIGridFilterCreator gridFilterCreator = new UIGridFilterCreator((DMSolution) dmClass.Solution);
            UIGridFilter gridFilterInfo = gridFilterCreator.CreateUIGridFilter(dmClass.Properties);
            gridInfo.UIGridFilter = gridFilterInfo;
            return view;
        }
    }

}