﻿using System.Windows.Forms;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using FireStarterModeller.Test.BO;
using FireStarterModeller.UI;
using Habanero.BO;
using Habanero.BO.ClassDefinition;
using Habanero.BO.Loaders;
using Habanero.DB;
using Habanero.Faces.Base;
using Habanero.Faces.Win;
using NUnit.Framework;
using DockStyle=System.Windows.Forms.DockStyle;

namespace FireStarterModeller.Test.UI.CodeGeneration
{
    [TestFixture]
    public class TestDatabaseConnectionInfoControl
    {
        [TestFixtureSetUp]
        public void SetUpFixture()
        {
            GlobalUIRegistry.ControlFactory = new ControlFactoryWin();
            ClassDef.ClassDefs.Clear();
            ClassDef.ClassDefs.Add(new XmlClassDefsLoader(BOBroker.GetClassDefsXml(), new DtdLoader()).LoadClassDefs());
        }

        [SetUp]
        public void Setup()
        {
            BORegistry.DataAccessor = new DataAccessorInMemory();
        }

        [Test]
        public void Test_Construct()
        {
            //---------------Set up test pack-------------------
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            DatabaseConnectionInfoControl connectionInfoControl = new DatabaseConnectionInfoControl();
            //---------------Test Result -----------------------
            Assert.AreEqual(1, connectionInfoControl.Controls.Count);
            Assert.IsInstanceOf(typeof(IBOGridAndEditorControl), connectionInfoControl.Controls[0]);
            IBOGridAndEditorControl boGridAndEditorControl = (IBOGridAndEditorControl) connectionInfoControl.Controls[0];
            BOEditorControlWin<DatabaseConnectionInfo> boEditorControl = (BOEditorControlWin<DatabaseConnectionInfo>)boGridAndEditorControl.IBOEditorControl;
            Assert.AreEqual(1, boEditorControl.Controls.Count);
            Assert.IsInstanceOf(typeof (IPanel), boEditorControl.Controls[0]);
            IPanel panel = (IPanel) boEditorControl.Controls[0];
            Assert.IsInstanceOf( typeof(IButtonGroupControl), panel.Controls[ panel.Controls.Count-1]);

            IButtonGroupControl buttonGroupControl = (IButtonGroupControl) panel.Controls[panel.Controls.Count - 1];
            Assert.AreEqual(1, buttonGroupControl.Controls.Count);
            Assert.AreEqual("Test Connection", buttonGroupControl.Controls[0].Text);
        }

        [Test]
        [Ignore("Unable to test MessageBox when TestButton clicked but works visually")]
        public void Test_TestConnectionButtonClick_ShouldCheckConnectionDetailsSetupCorrectly()
        {
            //---------------Set up test pack-------------------
            DatabaseConnectionInfo databaseConnectionInfo = TestUtilsDatabaseConnectionInfo.CreateSavedDatabaseConnectionInfo();
            DatabaseConnectionInfoControl connectionInfoControl = new DatabaseConnectionInfoControl();
            IBOGridAndEditorControl boGridAndEditorControl = GetGridAndBOEditorControl(connectionInfoControl);
            //---------------Assert Precondition----------------
            Assert.IsNotNull(boGridAndEditorControl.IBOEditorControl.BusinessObject);
            Assert.AreSame(databaseConnectionInfo, boGridAndEditorControl.IBOEditorControl.BusinessObject);
            //---------------Execute Test ----------------------
            connectionInfoControl.TestConnectionButton.PerformClick();
            //---------------Test Result -----------------------
        }

        private static IBOGridAndEditorControl GetGridAndBOEditorControl(DatabaseConnectionInfoControl connectionInfoControl)
        {
            return (IBOGridAndEditorControl)connectionInfoControl.Controls[0];
        }
    }

}
