
// ------------------------------------------------------------------------------

// This partial class was auto-generated for use with the Habanero Architecture.

// ------------------------------------------------------------------------------


using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Runtime.Serialization;
using FireStarter.Base;
using Habanero.Base.Exceptions;
using Habanero.BO.ClassDefinition;
using Habanero.Util;

namespace Firestarter.DB.Schema.BO
{
    [Serializable]
    public partial class DBPrimaryKey : IMergableBo
    {
        private List<DBColumn> _listColumns;

        public DBPrimaryKey()
        {
        }

        protected internal DBPrimaryKey(ClassDef def) : base(def)
        {
        }

        protected DBPrimaryKey(SerializationInfo info, StreamingContext context) : base(info, context)
        {
        }

        public override string ToString()
        {
            string tableName = this.Table == null ? "" : this.Table.TableNameDB;
            string primaryKeyName = "Primary";
            List<DBPrimaryKeyColumn> dbPrimaryKeyColumns = this.PrimaryKeyColumns.GetList();
            dbPrimaryKeyColumns.Sort((column, keyColumn) => Comparer<string>.Default.Compare(column.ColumnName, keyColumn.ColumnName));
            if (dbPrimaryKeyColumns.Count > 0)
            {
                primaryKeyName += " (";
                string columnNames = "";
                foreach (DBPrimaryKeyColumn primaryKeyColumn in dbPrimaryKeyColumns)
                {
                    if (primaryKeyColumn.Column == null) continue;
                    columnNames = StringUtilities.AppendMessage(columnNames, primaryKeyColumn.Column.ColumnName, " - ");
                }
                primaryKeyName += columnNames + ")";
            }
            return tableName + " - " + primaryKeyName;
        }

        public DBColumn GetColumnInfo(string columnName)
        {
            DBPrimaryKeyColumn dbPrimaryKeyColumn = GetPrimaryKeyColumnInfo(columnName);
            return dbPrimaryKeyColumn != null ? dbPrimaryKeyColumn.Column : null ;
        }

        public DBPrimaryKeyColumn GetPrimaryKeyColumnInfo(string columnName)
        {
            foreach (DBPrimaryKeyColumn info in this.PrimaryKeyColumns)
            {
                if (info.Column.ColumnName == columnName) return info;
            }
            return null;
        }

        public DBPrimaryKeyColumn AddColumnInfo(DBColumn dbColumn)
        {
            if (dbColumn == null) throw new ArgumentNullException("dbColumn");
            DBPrimaryKeyColumn dbPrimaryKeyColumn = this.GetPrimaryKeyColumnInfo(dbColumn.ColumnName);
            if (dbPrimaryKeyColumn != null)
            {
                if (dbPrimaryKeyColumn.Column == dbColumn)
                {
                    return dbPrimaryKeyColumn;
                }
                string name = this.Table == null? "": this.Table.TableNameDB;
                string message = string.Format("A column with name '{0}' was could not be added to the " + 
                                               "primary key for the business object '{1}' since it already exists in the business object"
                                               , dbColumn.ColumnName, name);
                throw new HabaneroDeveloperException(message,message);
            }
            dbPrimaryKeyColumn = new DBPrimaryKeyColumn { Column = dbColumn };
            this.PrimaryKeyColumns.Add(dbPrimaryKeyColumn);
            return dbPrimaryKeyColumn;
        }
        /// <summary>
        /// Returns a list of column infos associated with this PrimaryKey
        /// </summary>
        public ReadOnlyCollection<DBColumn> Columns
        {
            get
            {
                if (_listColumns == null)
                {
                    _listColumns = new List<DBColumn>();
                }else
                {
                    _listColumns.Clear();
                }
                
                foreach (DBPrimaryKeyColumn keyColumnInfo in this.PrimaryKeyColumns)
                {
                    _listColumns.Add(keyColumnInfo.Column);
                }
                return new ReadOnlyCollection<DBColumn>(_listColumns);
            }
        }

        /// <summary>
        /// Whether this change is approved to be applied to the domain model
        /// </summary>
        public bool PendingChangeApproved
        {
            get { return Approved ?? false; }
            set { Approved = value; }
        }
    }
}