using System;
using System.Data;
using System.Data.Common;
using System.Data.SqlClient;

namespace Firestarter.DB.Schema.Logic
{
    public class DBSchemaExtractorSQLServer : DBSchemaExtractor
    {
        public DBSchemaExtractorSQLServer(IDbConnection idbConnection) : base(idbConnection)
        {
        }

        /// <summary>
        /// Gets the current Catalog Name from the database connection.
        /// </summary>
        public override string CatalogName
        {
            get { return _dbConnection.Database; }
        }

        /// <summary>
        /// Gets the current Schema Name from the database connection.
        /// </summary>
        public override string SchemaName
        {
            get { return "dbo"; }
        }

        protected override DbDataAdapter CreateDataAdapter(string sql)
        {
            return new SqlDataAdapter(sql, (SqlConnection) _dbConnection);
        }

        /// <summary>
        /// This method is the specific for the database implementation for fetching the 
        /// DataTable representing the databases for the connection with a specified database name.
        /// This database name might be used to filter the catalog, schema, or both.
        /// This needs to be overridden because there is a difference in columns
        /// General:  CATALOG_NAME, SCHEMA_NAME
        /// SQLSVR:   database_name,dbid,create_date
        /// </summary>
        /// <param name="databaseName">The name of the database to filter by.</param>
        /// <returns>A DataTable populated with the Databases information.</returns>
        public override DataTable GetDatabasesDataTable(string databaseName)
        {
            DataTable dataTable = base.GetDatabasesDataTable(databaseName); 
            dataTable.Columns["database_name"].ColumnName = DBSchemaTemplate.Naming.CatalogName;
            DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.SchemaName, string.Format("'{0}'", SchemaName));
            return dataTable;
        }

        public override DataTable GetDatabasesDataTable(string catalogName, string schemaName)
        {
            return GetDatabasesDataTable(catalogName);
        }

        //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE, IS_AUTO_INCREMENTING
        //SQLSVR:   TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,CHARACTER_OCTET_LENGTH,NUMERIC_PRECISION,NUMERIC_PRECISION_RADIX,NUMERIC_SCALE,DATETIME_PRECISION,CHARACTER_SET_CATALOG,CHARACTER_SET_SCHEMA,CHARACTER_SET_NAME,COLLATION_CATALOG
        protected override DataTable GetColumnsDataTable(string catalogName, string schemaName, string tableName)
        {
            string sql = @"
                Select *, COLUMNPROPERTY(OBJECT_ID(TABLE_NAME), COLUMN_NAME, 'IsIdentity') as IS_IDENTITY 
                From INFORMATION_SCHEMA.COLUMNS";
            if (!string.IsNullOrEmpty(catalogName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TABLE_CATALOG", catalogName);
            if (!string.IsNullOrEmpty(schemaName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TABLE_SCHEMA", schemaName);
            if (!string.IsNullOrEmpty(tableName)) SqlUtilities.AddSqlWhereParameter(ref sql, "TABLE_NAME", tableName);
            sql += " ORDER BY TABLE_CATALOG, TABLE_SCHEMA, TABLE_NAME, ORDINAL_POSITION ";
            DataTable dataTable = GetDataTable(sql);
            DataTableUtilities.AddColumn(dataTable, DBSchemaTemplate.Naming.ColumnIsAutoIncrement, "IS_IDENTITY = 1");
            return dataTable;
        }
    }
}
