using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Linq;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Firestarter.DB.Schema.Logic;
using Habanero.Base.Exceptions;
using Habanero.BO;
using Habanero.DB;
using MySql.Data.MySqlClient;
using NUnit.Framework;

namespace Firestarter.DB.Schema.Test.Logic
{
    //TODO: All the scenarios for updating - 
    //      - Add new table
    //      - Remove table
    //      - 
    [TestFixture]
    public class TestDBReverseEng_UpdateInfos : TestBase
    {
        private DBSchemaTemplates_Fake _dbSchemaTemplatesFake;
        private TestUtilsSchema _testUtilsSchema;
        private DatabaseReverseEngineer _databaseReverseEngineer;
        private DBSchemaExtractor_Fake _schemaExtractorFake;

        [TestFixtureSetUp]
        public override void TestFixtureSetup()
        {
            base.TestFixtureSetup();
            _dbSchemaTemplatesFake = new DBSchemaTemplates_Fake();
        }

        [SetUp]
        public override void SetupTest()
        {
            base.SetupTest();
            _schemaExtractorFake = new DBSchemaExtractor_Fake();
            _databaseReverseEngineer = new DatabaseReverseEngineer(_schemaExtractorFake);
            _testUtilsSchema = new TestUtilsSchema(_dbSchemaTemplatesFake);
        }

        #region TestUtilityMethods

        //[Test]
        //public void TestTables_HasNoTables_WithoutPopulateCall()
        //{
        //    //-------------Setup Test Pack ------------------
        //    const string tableName = "TestTable";
        //    _dbSchemaTemplatesFake.CreateDatabaseTableWithIDAndNameField(tableName);
        //    DBDatabase databaseInfo = new DBDatabase(_databaseConnection);
        //    //-------------Test Pre-conditions --------------
        //    //-------------Execute test ---------------------
        //    List<Table> tableInfos = databaseInfo.Tables;
        //    //-------------Test Result ----------------------
        //    Assert.AreEqual(0, tableInfos.Count, "There should be zero table infos as populate has not been called.");
        //}

        [Test]
        public void Test_ConvertDBNullToNull()
        {
            //---------------Set up test pack-------------------
            string value = TestUtils.GetRandomString();
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            object nullValue = DBUtilities.ConvertDBNullToNull(DBNull.Value);
            object nonNullValue = DBUtilities.ConvertDBNullToNull(value);
            //---------------Test Result -----------------------
            Assert.AreEqual(null, nullValue);
            Assert.AreEqual(value, nonNullValue);
        }

        [Test]
        public void Test_ConvertDBNullToNullDateTime()
        {
            //---------------Set up test pack-------------------
            DateTime? value = DateTime.Now;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            object nullValue = DBUtilities.ConvertDBNullToNullDateTime(DBNull.Value);
            object nonNullValue = DBUtilities.ConvertDBNullToNullDateTime(value);
            //---------------Test Result -----------------------
            Assert.AreEqual(null, nullValue);
            Assert.AreEqual(value, nonNullValue);
        }

        [Test]
        public void Test_GetCharDataLength_FromCharMaxLength()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            const int expectedDataLength = 12;
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, TestUtils.GetRandomString(), "CHAR(38)", expectedDataLength);

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            long dataLength = DatabaseReverseEngineer.GetCharDataLength(dataTable.Rows[0]);

            //---------------Test Result -----------------------
            Assert.AreEqual(expectedDataLength, dataLength);
        }

        [Test]
        public void Test_GetCharDataLength_FromDataType_12()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            const int expectedDataLength = 12;
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, TestUtils.GetRandomString(), "CHAR(" + expectedDataLength + ")", 0);

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            long dataLength = DatabaseReverseEngineer.GetCharDataLength(dataTable.Rows[0]);

            //---------------Test Result -----------------------
            Assert.AreEqual(expectedDataLength, dataLength);
        }

        [Test]
        public void Test_GetCharDataLength_FromDataType_33()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            const int expectedDataLength = 33;
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, TestUtils.GetRandomString(), "CHAR(" + expectedDataLength + ")", 0);

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            long dataLength = DatabaseReverseEngineer.GetCharDataLength(dataTable.Rows[0]);

            //---------------Test Result -----------------------
            Assert.AreEqual(expectedDataLength, dataLength);
        }

        [Test]
        public void Test_GetCharDataLength_FromColumnType_12()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            const int expectedDataLength = 12;
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, TestUtils.GetRandomString(), "CHAR",0);
            dataTable.Columns.Add("ColumnType");
            dataTable.Rows[0]["ColumnType"] = "CHAR(" + expectedDataLength + ")";
            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            long dataLength = DatabaseReverseEngineer.GetCharDataLength(dataTable.Rows[0]);

            //---------------Test Result -----------------------
            Assert.AreEqual(expectedDataLength, dataLength);
        }

        [Test]
        public void Test_GetCharDataLength_FromColumnType_33()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            const int expectedDataLength = 33;
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, TestUtils.GetRandomString(), "CHAR", 0);
            dataTable.Columns.Add("ColumnType");
            dataTable.Rows[0]["ColumnType"] = "CHAR(" + expectedDataLength + ")";

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            long dataLength = DatabaseReverseEngineer.GetCharDataLength(dataTable.Rows[0]);

            //---------------Test Result -----------------------
            Assert.AreEqual(expectedDataLength, dataLength);
        }

        //TODO: Check where this goes.
        [Test,Ignore("Need clarity as to where this test should go.")]
        public void Test_OpenConnection_DatabaseDoesntExist()
        {
            //-------------Setup Test Pack ------------------
            DatabaseConfig databaseConfig = new DatabaseConfig
                ("MySql", "localhost", "bobsdatabasewhichdoesntexist", "root", "root", "3306");
            DBDatabase dbDatabase = new DBDatabase(databaseConfig.GetDatabaseConnection());
            dbDatabase.Catalog = null;
            dbDatabase.Schema = databaseConfig.Database;

            _schemaExtractorFake.TablesDataTable = _dbSchemaTemplatesFake.CreateTablesDataTable();

            string errors;
            bool databaseExists = dbDatabase.DatabaseExists(out errors);
            //-------------Test Pre-conditions -------------
            Assert.IsFalse(databaseExists);

            //-------------Execute test ---------------------
            try
            {
                _databaseReverseEngineer.PopulateDatabaseInfo(dbDatabase);
                Assert.Fail("Error should be thrown");
            }

                //-------------Test Result ----------------------
            catch (MySqlException ex)
            {
                StringAssert.Contains("Unknown database", ex.Message);
            }
        }

        [Test]
        public void Test_DBTypeConverter_GetSet()
        {
            //---------------Set up test pack-------------------
            DBTypeConverter expectedDbTypeConverter = new DBTypeConverter(null);
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            _databaseReverseEngineer.DBTypeConverter=expectedDbTypeConverter;
            //---------------Test Result -----------------------
            Assert.AreSame(expectedDbTypeConverter, _databaseReverseEngineer.DBTypeConverter);
        }

        #endregion

        #region TestUpdate Column

        [Test]
        public void Test_UpdateTables_OneTable()
        {
            //---------------Set up test pack-------------------
            const string tableType = "Base Table";
            const string tableName = "TableName";
            DataTable dataTable = _dbSchemaTemplatesFake.CreateTablesDataTable();
            _dbSchemaTemplatesFake.CreateTableDataRow(dataTable, tableName, tableType);
            _schemaExtractorFake.TablesDataTable = dataTable;
            DBDatabase dbDatabase = new DBDatabase();
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateTables(dbDatabase);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, dbDatabase.Tables.Count);
            DBTable dbTable = dbDatabase.Tables[0];
            Assert.AreEqual(tableName, dbTable.TableNameDB);
            Assert.AreEqual(tableType, dbTable.TableType);
        }

        [Test]
        public void Test_UpdateTables_TwoTable()
        {
            //---------------Set up test pack-------------------
            const string tableName1 = "TableName1";
            const string tableName2 = "TableName2";
            DataTable dataTable = _dbSchemaTemplatesFake.CreateTablesDataTable();
            _dbSchemaTemplatesFake.CreateTableDataRow(dataTable, tableName1);
            _dbSchemaTemplatesFake.CreateTableDataRow(dataTable, tableName2);
            _schemaExtractorFake.TablesDataTable = dataTable;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            DBDatabase dbDatabase = new DBDatabase();
            _databaseReverseEngineer.PopulateTables(dbDatabase);

            //---------------Test Result -----------------------
            Assert.AreEqual(2, dbDatabase.Tables.Count);
            DBTable dbTable = dbDatabase.GetTableInfo(tableName1);
            Assert.IsNotNull(dbTable);
            dbTable = dbDatabase.GetTableInfo(tableName2);
            Assert.IsNotNull(dbTable);
        }

        [Test, Ignore("This does not apply anymore")]
        public void TestPopulateDatabaseInfo_nullDBConnection()
        {
            //-------------Setup Test Pack ------------------
            DBDatabase dbDatabase = new DBDatabase();

            //-------------Test Pre-conditions --------------
            //-------------Execute test ---------------------
            try
            {
                _databaseReverseEngineer.PopulateDatabaseInfo(dbDatabase);
                Assert.Fail("Should raise an error");
            }
            catch (HabaneroArgumentException ex)
            {
                StringAssert.Contains("The dbDatabase has not been set up with a database connection", ex.Message);
            }
        }

        [Test]
        public void TestPopulateDatabaseInfo_nullDBINfo()
        {
            //-------------Setup Test Pack ------------------
            const DBDatabase databaseInfo = null;
            //-------------Test Pre-conditions --------------
            //-------------Execute test ---------------------
            try
            {
                _databaseReverseEngineer.PopulateDatabaseInfo(databaseInfo);
                Assert.Fail("Should raise an error");
            }
            catch (ArgumentNullException ex)
            {
                StringAssert.Contains("dbDatabase", ex.Message);
            }
        }

        [Test]
        public void Test_PopulateColumns_NoDatabaseInfo_ThrowsException()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();
            //---------------Assert Precondition----------------
            Assert.IsNull(dbTable.Database);
            //---------------Execute Test ----------------------
            try
            {
                _databaseReverseEngineer.PopulateTableInfo(dbTable);
                Assert.Fail("Should raise an error");
            }
            catch (NullReferenceException ex)
            {
                StringAssert.Contains("does not have a reference to DBDatabase", ex.Message);
            }
        }

        [Test]
        public void Test_UpdateColumns_IdColumn()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            string columnName = TestUtils.GetRandomString() + "ID";
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, columnName, "CHAR",38);
            DBTable dbTable = new DBTable();
            //_schemaExtractorFake.ColumnsDataTable = dataTable;
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateColumns(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, dbTable.Columns.Count);
            DBColumn dbColumn = dbTable.Columns[0];
            Assert.AreEqual(columnName, dbColumn.ColumnName);
            Assert.AreEqual("char".ToUpper(), dbColumn.DataType);
            Assert.AreEqual(38, dbColumn.DataLength);
        }

        [Test]
        public void Test_UpdateColumns_TwoColumnsInTable()
        {
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            string columnName2 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, columnName1);
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, columnName2);
            //_schemaExtractorFake.ColumnsDataTable = dataTable;
            DBTable dbTable = new DBTable();

            //--------------Assert PreConditions----------------
            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateColumns(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(2, dbTable.Columns.Count);
            Assert.IsNotNull(dbTable.GetColumnInfo(columnName1));
            Assert.IsNotNull(dbTable.GetColumnInfo(columnName2));
        }

        [Test]
        public void Test_UpdateColumns_TwoColumnsInTable_OutOfMany()
        {
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            string columnName2 = TestUtils.GetRandomString();
            DBTable dbTable = new DBTable();
            dbTable.TableNameDB = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, dbTable.TableNameDB, columnName1);
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, TestUtils.GetRandomString(), columnName2);
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, dbTable.TableNameDB, columnName2);
            //_schemaExtractorFake.ColumnsDataTable = dataTable;
            //--------------Assert PreConditions----------------
            Assert.AreEqual(3, dataTable.Rows.Count);
            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateColumns(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(2, dbTable.Columns.Count);
            Assert.IsNotNull(dbTable.GetColumnInfo(columnName1));
            Assert.IsNotNull(dbTable.GetColumnInfo(columnName2));
        }

        [Test]
        public void Test_UpdateColumns_ConvertsCustomType()
        {
            //---------------Set up test pack-------------------
            string dataType = TestUtils.GetRandomString();
            string customDataType = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            string columnName = customDataType;
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, columnName, dataType, 38);
            DBTable dbTable = new DBTable();
            //_schemaExtractorFake.ColumnsDataTable = dataTable;

            BusinessObjectCollection<TypeMapping> typeMappings = new BusinessObjectCollection<TypeMapping>();
            TypeMapping typeMapping = new TypeMapping { FromType = dataType, ToType = customDataType };
            typeMappings.Add(typeMapping);
            DBTypeConverter typeConverter = new DBTypeConverter(typeMappings);
            _databaseReverseEngineer.DBTypeConverter = typeConverter;
            //---------------Assert Precondition----------------
            
            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateColumns(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, dbTable.Columns.Count);
            DBColumn dbColumn = dbTable.Columns[0];
            Assert.AreEqual(columnName, dbColumn.ColumnName);
            Assert.AreEqual(customDataType, dbColumn.DataType);
            Assert.AreEqual(38, dbColumn.DataLength);
        }

        [Test]
        public void Test_PopulateColumns_WhenCharLengthGTIntMax_ShouldConvertToIntMax()
        {
            //---------------Set up test pack-------------------
            const string dataType = "Char";
            string customDataType = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateColumnsDataTable();
            string columnName = customDataType;
            const long maxCharLength = long.MaxValue - 6;
            _dbSchemaTemplatesFake.CreateColumnDataRow(dataTable, columnName, dataType, maxCharLength);
            DBTable dbTable = new DBTable();
            BusinessObjectCollection<TypeMapping> typeMappings = new BusinessObjectCollection<TypeMapping>();
            TypeMapping typeMapping = new TypeMapping { FromType = dataType, ToType = customDataType };
            typeMappings.Add(typeMapping);
            DBTypeConverter typeConverter = new DBTypeConverter(typeMappings);
            _databaseReverseEngineer.DBTypeConverter = typeConverter;
            //---------------Assert Precondition----------------
            DataRow dataRow = dataTable.Rows.OfType<DataRow>().First();
            Assert.IsNotNull(dataRow);
            Assert.Greater(Convert.ToInt64(dataRow[DBSchemaTemplate.Naming.CharacterMaximumLength]), int.MaxValue);
            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateColumns(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, dbTable.Columns.Count);
            DBColumn dbColumn = dbTable.Columns[0];
            Assert.AreEqual(columnName, dbColumn.ColumnName);
            Assert.AreEqual(customDataType, dbColumn.DataType);
            Assert.AreEqual(int.MaxValue, dbColumn.DataLength);
        }

        #endregion


        #region PrimaryKey

        [Test]
        public void TestUpdatePrimaryKeyInfos_FieldsNotLoaded()
        {
            //This test shows that a Primary Key is not an alternate key.
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreatePrimaryKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, columnName1);
            //_schemaExtractorFake.PrimaryKeysDataTable = dataTable;
            DBTable dbTable = new DBTable();

            //--------------Assert PreConditions----------------
            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            try
            {
                _databaseReverseEngineer.PopulatePrimaryKeyInfos(dbTable, dataTable);
                Assert.Fail("expected Err");
            }
            //---------------Test Result -----------------------
            catch (HabaneroApplicationException ex)
            {
                StringAssert.Contains("You cannot get the key for the table ", ex.Message);
                StringAssert.Contains("since the tables columns have not been loaded", ex.Message);
            }
            catch (Exception)
            {
                Assert.Fail("HabaneroApplicationException not thrown");
            }

            //---------------Test Result -----------------------
        }

        [Test]
        public void TestUpdatePrimaryKeyInfos_OneKey_OneColumn()
        {
            //This test shows that a Primary Key is not an alternate key.
            //---------------Set up test pack-------------------
            string tableName = TestUtils.GetRandomString();
            string columnName1 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreatePrimaryKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, tableName, columnName1);
            //_schemaExtractorFake.PrimaryKeysDataTable = dataTable;
            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithColumn(tableName,columnName1);
            //--------------Assert PreConditions----------------
            Assert.AreEqual(1, dbTable.Columns.Count);
            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulatePrimaryKeyInfos(dbTable, dataTable);
            //---------------Test Result -----------------------
            DBPrimaryKey dbPrimaryKey = dbTable.PrimaryKeyDB;
            Assert.IsNotNull(dbPrimaryKey);
            Assert.IsNotNull(dbPrimaryKey.Table);

            BusinessObjectCollection<DBPrimaryKeyColumn> columnInfos = dbPrimaryKey.PrimaryKeyColumns;
            Assert.AreEqual(1, columnInfos.Count);
            DBPrimaryKeyColumn dbPrimaryKeyColumn = columnInfos[0];
            Assert.AreEqual(columnName1, dbPrimaryKeyColumn.Column.ColumnName);
        }

        [Test]
        public void TestUpdatePrimaryKeyInfos_OneKey_TwoColumn()
        {
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            string columnName2 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreatePrimaryKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, columnName1);
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, columnName2);
            //_schemaExtractorFake.PrimaryKeysDataTable = dataTable;

            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithTwoColumn(null, columnName1, columnName2);

            //--------------Assert PreConditions----------------
            Assert.AreEqual(2, dbTable.Columns.Count);
            Assert.AreEqual(2, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulatePrimaryKeyInfos(dbTable, dataTable);

            //---------------Test Result -----------------------
            DBPrimaryKey dbPrimaryKey = dbTable.PrimaryKeyDB;
            Assert.IsNotNull(dbPrimaryKey);
            Assert.IsNotNull(dbPrimaryKey.Table);

            BusinessObjectCollection<DBPrimaryKeyColumn> columnInfos = dbPrimaryKey.PrimaryKeyColumns;
            Assert.AreEqual(2, columnInfos.Count);

            Assert.IsNotNull(dbPrimaryKey.GetColumnInfo(columnName1));
            Assert.IsNotNull(dbPrimaryKey.GetColumnInfo(columnName2));
        }

        [Test]
        public void TestUpdatePrimaryKeyInfos_OneKey_TwoColumn_OutOfMany()
        {
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            string columnName2 = TestUtils.GetRandomString();
            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithTwoColumn(null, columnName1, columnName2);
            dbTable.TableNameDB = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreatePrimaryKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, dbTable.TableNameDB, columnName1);
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, dbTable.TableNameDB, columnName2);
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, TestUtils.GetRandomString(), columnName2);


            //--------------Assert PreConditions----------------
            Assert.AreEqual(3, dataTable.Rows.Count);
            Assert.AreEqual(2, dbTable.Columns.Count);

            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulatePrimaryKeyInfos(dbTable, dataTable);

            //---------------Test Result -----------------------
            DBPrimaryKey dbPrimaryKey = dbTable.PrimaryKeyDB;
            Assert.IsNotNull(dbPrimaryKey);
            Assert.IsNotNull(dbPrimaryKey.Table);

            BusinessObjectCollection<DBPrimaryKeyColumn> columnInfos = dbPrimaryKey.PrimaryKeyColumns;
            Assert.AreEqual(2, columnInfos.Count);

            Assert.IsNotNull(dbPrimaryKey.GetColumnInfo(columnName1));
            Assert.IsNotNull(dbPrimaryKey.GetColumnInfo(columnName2));
        }

        [Test, Ignore("Test does not apply anymore")]
        public void TestUpdatePrimaryKeyInfos_OneKey_HasAlternateKey()
        {
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            string columnName2 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreatePrimaryKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreatePrimaryKeyColumnDataRow(dataTable, columnName1);
            //_schemaExtractorFake.PrimaryKeysDataTable = dataTable;
            
            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithTwoColumn(null, columnName1, columnName2);

            //--------------Assert PreConditions----------------
            Assert.AreEqual(2, dbTable.Columns.Count);
            Assert.AreEqual(2, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulatePrimaryKeyInfos(dbTable, dataTable);

            //---------------Test Result -----------------------
            DBPrimaryKey dbPrimaryKey = dbTable.PrimaryKeyDB;
            Assert.IsNotNull(dbPrimaryKey);
            Assert.IsNotNull(dbPrimaryKey.Table);

            BusinessObjectCollection<DBPrimaryKeyColumn> columnInfos = dbPrimaryKey.PrimaryKeyColumns;
            Assert.AreEqual(1, columnInfos.Count);

            Assert.IsNotNull(dbPrimaryKey.GetColumnInfo(columnName1));
            Assert.IsNull(dbPrimaryKey.GetColumnInfo(columnName2));
        }

        [Test, Ignore("Test does not apply anymore")]
        public void TestUpdatePrimaryKeyInfos_NoPrimaryKey_HasAlternateKey()
        {
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            string columnName2 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreatePrimaryKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, "AlternateKey", columnName2);
            //_schemaExtractorFake.PrimaryKeysDataTable = dataTable;

            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithTwoColumn(null, columnName1, columnName2);

            //--------------Assert PreConditions----------------
            Assert.AreEqual(2, dbTable.Columns.Count);
            Assert.AreEqual(1, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulatePrimaryKeyInfos(dbTable, dataTable);

            //---------------Test Result -----------------------
            DBPrimaryKey dbPrimaryKey = dbTable.PrimaryKeyDB;
            Assert.IsNull(dbPrimaryKey);
        }



        #endregion

        #region UniqueKey

        [Test]
        public void TestUpdateUniqueKeyInfos_FieldsNotLoaded()
        {
            //This test shows that a Alternate Key is not an alternate key.
            //---------------Set up test pack-------------------
            string columnName1 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateUniqueKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, columnName1 + "key", columnName1);
            //_schemaExtractorFake.UniqueKeysDataTable = dataTable;
            DBTable dbTable = new DBTable();

            //--------------Assert PreConditions----------------
            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            try
            {
                _databaseReverseEngineer.PopulateUniqueKeyInfos(dbTable, dataTable);
                Assert.Fail("expected Err");
            }
            //---------------Test Result -----------------------
            catch (HabaneroApplicationException ex)
            {
                StringAssert.Contains("You cannot get the key for the table ", ex.Message);
                StringAssert.Contains("since the tables columns have not been loa", ex.Message);
            }

            //---------------Test Result -----------------------
        }

        [Test]
        public void TestUpdateUniqueKeyInfos_OneKey_OneColumn()
        {
            //This test shows that a Alternate Key is not an alternate key.
            //---------------Set up test pack-------------------
            string tableName = TestUtils.GetRandomString();
            string columnName1 = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateUniqueKeyColumnsDataTable();
            string keyName = columnName1 + "key";
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, tableName, keyName, columnName1);
            //_schemaExtractorFake.UniqueKeysDataTable = dataTable;

            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithColumn(tableName, columnName1);
            //--------------Assert PreConditions----------------
            Assert.AreEqual(1, dbTable.Columns.Count);

            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulateUniqueKeyInfos(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, dbTable.Keys.Count);
            DBKey alternateDBKey = dbTable.Keys[0];
            Assert.IsNotNull(alternateDBKey);
            Assert.IsNotNull(alternateDBKey.Table);
            Assert.AreEqual(keyName , alternateDBKey.KeyName);

            IList<DBKeyColumn> keyColumns = alternateDBKey.KeyColumns;
            Assert.AreEqual(1, keyColumns.Count, "Should be one column");
            DBKeyColumn dbKeyColumn = keyColumns[0];
            Assert.IsNotNull(dbKeyColumn.Column);
            Assert.AreEqual(columnName1, dbKeyColumn.Column.ColumnName);
        }

        [Test]
        public void TestUpdateUniqueKeyInfos_OneKey_TwoColumn()
        {
            //---------------Set up test pack-------------------
            const string columnName1 = "Column1";
            const string columnName2 = "Column2";
            const string keyName = "KeyName";
            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithTwoColumn(null, columnName1, columnName2);
            dbTable.TableNameDB = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateUniqueKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, dbTable.TableNameDB, keyName, columnName1);
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, dbTable.TableNameDB, keyName, columnName2);
            //_schemaExtractorFake.UniqueKeysDataTable = dataTable;
            //--------------Assert PreConditions----------------
            Assert.AreEqual(2, dataTable.Rows.Count);
            Assert.AreEqual(2, dbTable.Columns.Count);
            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulateUniqueKeyInfos(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, dbTable.Keys.Count);
            DBKey alternateDBKey = dbTable.Keys[0];
            Assert.IsNotNull(alternateDBKey);
            Assert.IsNotNull(alternateDBKey.Table);
            Assert.AreEqual(keyName, alternateDBKey.KeyName);

            IList<DBKeyColumn> keyColumns = alternateDBKey.KeyColumns;
            Assert.AreEqual(2, keyColumns.Count, "Should be two columns");
            Assert.IsNotNull(alternateDBKey.GetColumnInfo(columnName1));
            Assert.IsNotNull(alternateDBKey.GetColumnInfo(columnName2));
        }

        [Test]
        public void TestUpdateUniqueKeyInfos_OneKey_TwoColumn_OutOfMany()
        {
            //---------------Set up test pack-------------------
            const string columnName1 = "Column1";
            const string columnName2 = "Column2";
            const string keyName = "KeyName";
            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithTwoColumn(null, columnName1, columnName2);
            dbTable.TableNameDB = TestUtils.GetRandomString();
            DataTable dataTable = _dbSchemaTemplatesFake.CreateUniqueKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, TestUtils.GetRandomString(), TestUtils.GetRandomString(), TestUtils.GetRandomString());
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, TestUtils.GetRandomString(), TestUtils.GetRandomString(), TestUtils.GetRandomString());
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, dbTable.TableNameDB, keyName, columnName1);
            _dbSchemaTemplatesFake.CreateUniqueKeyColumnDataRow(dataTable, dbTable.TableNameDB, keyName, columnName2);
            //_schemaExtractorFake.UniqueKeysDataTable = dataTable;
            //--------------Assert PreConditions----------------
            Assert.AreEqual(4, dataTable.Rows.Count);
            Assert.AreEqual(2, dbTable.Columns.Count);
            //---------------Execute Test ----------------------
            //You must have the columns available for the table so that you
            //  can create the table.
            _databaseReverseEngineer.PopulateUniqueKeyInfos(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, dbTable.Keys.Count);
            DBKey alternateDBKey = dbTable.Keys[0];
            Assert.IsNotNull(alternateDBKey);
            Assert.IsNotNull(alternateDBKey.Table);
            Assert.AreEqual(keyName, alternateDBKey.KeyName);

            IList<DBKeyColumn> keyColumns = alternateDBKey.KeyColumns;
            Assert.AreEqual(2, keyColumns.Count, "Should be two columns");
            Assert.IsNotNull(alternateDBKey.GetColumnInfo(columnName1));
            Assert.IsNotNull(alternateDBKey.GetColumnInfo(columnName2));
        }
        
        #endregion


        #region ForeignKey

        [Test]
        public void TestUpdateForeignKeys_NoneExist()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateForeignKeyColumnsDataTable();
            //_schemaExtractorFake.ForeignKeysDataTable = dataTable;
            DBTable dbTable = new DBTable();

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateForeignKeys(dbTable, dataTable);
            //---------------Test Result -----------------------
            Assert.IsNotNull(dbTable.ForeignKeys);
            Assert.AreEqual(0, dbTable.ForeignKeys.Count);
        }

        [Test]
        public void TestUpdateForeignKeys_OneExist_MatchingTableDoesNotExist()
        {
            //---------------Set up test pack-------------------
            DataTable dataTable = _dbSchemaTemplatesFake.CreateForeignKeyColumnsDataTable();
            const string foreignkeyName = "ForeignKey";
            const string foreignKeyTableName = "FK_TableName";
            const string foreignKeyColumnName = "FK_Column";
            const string referencedTableName = "ReferencedTableName";
            const string referencedColumnName = "RefColumnName";
            _dbSchemaTemplatesFake.CreateForeignKeyColumnDataRow(dataTable, 
                foreignkeyName, foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, 1);
            //_schemaExtractorFake.ForeignKeysDataTable = dataTable;

            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithForeignKeyNoReferencedTable(foreignKeyTableName, foreignKeyColumnName);

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dbTable.Database.Tables.Count);

            //---------------Execute Test ----------------------
            try
            {
                _databaseReverseEngineer.PopulateForeignKeys(dbTable, dataTable);
                Assert.Fail("expected Err");
            }
            //---------------Test Result -----------------------
            catch (HabaneroApplicationException ex)
            {
                StringAssert.Contains("You cannot Update the Foreign Key " + foreignkeyName + " for table " + foreignKeyTableName, ex.Message);
                StringAssert.Contains("since the referenced table " + referencedTableName, ex.Message);
                StringAssert.Contains("does not exist in the schema", ex.Message);
            }

            //---------------Test Result -----------------------
        }

        [Test]
        public void TestUpdateForeignKeys_OneColumn()
        {
            //---------------Set up test pack-------------------
            const string foreignkeyName = "ForeignKey";
            const string foreignKeyTableName = "FK_TableName";
            const string foreignKeyColumnName = "FK_Column";
            const string referencedTableName = "ReferencedTableName";
            const string referencedColumnName = "RefColumnName";
            DataTable dataTable = _dbSchemaTemplatesFake.CreateForeignKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreateForeignKeyColumnDataRow(dataTable, 
                foreignkeyName, foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, 1);
            //_schemaExtractorFake.ForeignKeysDataTable = dataTable;

            DBTable referencedDBTable;
            DBTable dbTable = _testUtilsSchema.CreateTableInfoWithForeignKeyWithReferencedTable(foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, out referencedDBTable);

            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dataTable.Rows.Count);
            Assert.AreEqual(1, dbTable.Columns.Count);
            Assert.AreEqual(2, dbTable.Database.Tables.Count);

            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateForeignKeys(dbTable, dataTable);

            //---------------Test Result -----------------------
            Assert.IsNotNull(dbTable.ForeignKeys);
            Assert.AreEqual(1, dbTable.ForeignKeys.Count);
            DBForeignKey dbForeignKey = dbTable.ForeignKeys[0];
            Assert.AreEqual(foreignkeyName, dbForeignKey.ForeignKeyName);
            Assert.AreEqual(foreignKeyTableName, dbForeignKey.ParentTable.TableNameDB);
            Assert.AreEqual(1, dbForeignKey.ForeignKeyColumns.Count);
            DBForeignKeyColumn dbForeignKeyColumn = dbForeignKey.ForeignKeyColumns[0];
            Assert.IsNotNull(dbForeignKeyColumn.ParentColumn);
            Assert.IsNotNull(dbForeignKeyColumn.RelatedColumn);
            Assert.AreEqual(foreignKeyColumnName, dbForeignKeyColumn.ParentColumn.ColumnName);
            Assert.AreEqual(referencedTableName, dbForeignKey.RelatedTable.TableNameDB);
            Assert.AreEqual(referencedColumnName, dbForeignKeyColumn.RelatedColumn.ColumnName);
        }

        [Test]
        public void TestUpdateForeignKeyInfo_OneForeignKey_TwoColumns()
        {
            //---------------Set up test pack-------------------
            const string foreignkeyName = "ForeignKey";
            const string foreignKeyTableName = "FK_TableName";
            const string foreignKeyColumnName = "FK_Column";
            const string foreignKeyColumnName2 = "FK_Column_2";
            const string referencedTableName = "ReferencedTableName";
            const string referencedColumnName = "RefColumnName";
            const string referencedColumnName2 = "RefColumnName_2";
            DataTable dataTable = _dbSchemaTemplatesFake.CreateForeignKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreateForeignKeyRowsWithTwoColumns(dataTable,
                foreignkeyName, foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, foreignKeyColumnName2, referencedColumnName2);
            //_schemaExtractorFake.ForeignKeysDataTable = dataTable;

            DBTable foreignKeyDBTable = _testUtilsSchema.CreateTableInfoForeignKeyWith2Columns_WithReferencedTable(foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, foreignKeyColumnName2, referencedColumnName2);

            //---------------Assert Precondition----------------
            Assert.AreEqual(2, foreignKeyDBTable.Database.Tables.Count);
            Assert.AreEqual(2, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateForeignKeys(foreignKeyDBTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(1, foreignKeyDBTable.ForeignKeys.Count);
            DBForeignKey dbForeignKey = foreignKeyDBTable.ForeignKeys[0];
            Assert.AreEqual(2, dbForeignKey.ForeignKeyColumns.Count);

            DBForeignKeyColumn firstDBForeignKeyColumn = dbForeignKey.GetForeignKeyColumnInfo(foreignKeyColumnName);
            Assert.AreEqual(foreignKeyColumnName, firstDBForeignKeyColumn.ParentColumn.ColumnName);
            Assert.AreEqual(referencedColumnName, firstDBForeignKeyColumn.RelatedColumn.ColumnName);

            DBForeignKeyColumn secondDBForeignKeyColumn = dbForeignKey.GetForeignKeyColumnInfo(foreignKeyColumnName2);
            Assert.AreEqual(foreignKeyColumnName2, secondDBForeignKeyColumn.ParentColumn.ColumnName);
            Assert.AreEqual(referencedColumnName2, secondDBForeignKeyColumn.RelatedColumn.ColumnName);
        }


        [Test]
        public void TestUpdateForeignKeyInfo_TwoForeignKey_TwoColumns()
        {
            //---------------Set up test pack-------------------
            const string foreignkeyName = "ForeignKey";
            const string foreignKeyTableName = "FK_TableName";
            const string foreignKeyColumnName = "FK_Column";
            const string foreignKeyColumnName2 = "FK_Column_2";
            const string referencedTableName = "ReferencedTableName";
            const string referencedColumnName = "RefColumnName";
            const string referencedColumnName2 = "RefColumnName_2";
            DataTable dataTable = _dbSchemaTemplatesFake.CreateForeignKeyColumnsDataTable();
            _dbSchemaTemplatesFake.CreateForeignKeyRowsWithTwoColumns(dataTable, foreignkeyName, foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, foreignKeyColumnName2, referencedColumnName2);
            //_schemaExtractorFake.ForeignKeysDataTable = dataTable;

            const string anotherfk = "AnotherFK";
            _dbSchemaTemplatesFake.CreateForeignKeyColumnDataRow(dataTable, anotherfk, foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, 1);

            DBTable foreignKeyDBTable = _testUtilsSchema.CreateTableInfoForeignKeyWith2Columns_WithReferencedTable(foreignKeyTableName, foreignKeyColumnName, referencedTableName, referencedColumnName, foreignKeyColumnName2, referencedColumnName2);

            //---------------Assert Precondition----------------
            Assert.AreEqual(2, foreignKeyDBTable.Database.Tables.Count);
            Assert.AreEqual(3, dataTable.Rows.Count);

            //---------------Execute Test ----------------------
            _databaseReverseEngineer.PopulateForeignKeys(foreignKeyDBTable, dataTable);

            //---------------Test Result -----------------------
            Assert.AreEqual(2, foreignKeyDBTable.ForeignKeys.Count);
            Assert.AreNotEqual(foreignKeyDBTable.ForeignKeys[0], foreignKeyDBTable.ForeignKeys[1]);
            DBForeignKey compositeDBForeignKey = foreignKeyDBTable.GetForeignKeyInfo(foreignkeyName);
            Assert.AreEqual(2, compositeDBForeignKey.ForeignKeyColumns.Count);

            DBForeignKey simpleDBForeignKey = foreignKeyDBTable.GetForeignKeyInfo(anotherfk);
            Assert.AreEqual(1, simpleDBForeignKey.ForeignKeyColumns.Count);
        }
        #endregion

        #region TypeConversion

        [Test]
        public void Test_ConvertDataType_NoChangeIfConverterIsNull()
        {
            //---------------Set up test pack-------------------
            string dataType = TestUtils.GetRandomString();
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string convertedDataType = DatabaseReverseEngineer.ConvertDataType(dataType, null);

            //---------------Test Result -----------------------
            Assert.AreEqual(dataType, convertedDataType);
        }

        [Test]
        public void Test_ConvertDataType_NoChangeIfMappingNotFound()
        {
            //---------------Set up test pack-------------------
            string dataType = TestUtils.GetRandomString();
            BusinessObjectCollection<TypeMapping> typeMappings = new BusinessObjectCollection<TypeMapping>();
            DBTypeConverter typeConverter = new DBTypeConverter(typeMappings);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string convertedDataType = DatabaseReverseEngineer.ConvertDataType(dataType, typeConverter);

            //---------------Test Result -----------------------
            Assert.AreEqual(dataType, convertedDataType);
        }

        [Test]
        public void Test_ConvertDataType_ChangeToCustomMapping()
        {
            //---------------Set up test pack-------------------
            string dataType = TestUtils.GetRandomString();
            string customType = TestUtils.GetRandomString();
            BusinessObjectCollection<TypeMapping> typeMappings = new BusinessObjectCollection<TypeMapping>();
            TypeMapping typeMapping = new TypeMapping {FromType = dataType, ToType = customType};
            typeMappings.Add(typeMapping);

            DBTypeConverter typeConverter = new DBTypeConverter(typeMappings);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string convertedDataType = DatabaseReverseEngineer.ConvertDataType(dataType, typeConverter);

            //---------------Test Result -----------------------
            Assert.AreNotEqual(dataType, convertedDataType);
            Assert.AreEqual(customType, convertedDataType);
        }

        #endregion
    }
}
