using System.Data;
using Firestarter.DB.Schema.Logic;
using NUnit.Framework;

namespace Firestarter.DB.Schema.Test.Logic
{
    /// <summary>
    /// These tests test that the dbschema template is set up correctly. Each database implements slightly different names etc.
    ///   this schema template is then used to translate the schema for the various database vendors into a standard schema.
    /// </summary>
    [TestFixture]
    public class TestDBSchemaTemplate: TestBase
    {
        [Test]
        public void Test_CreateDatabasesDataTable()
        {
            //---------------Set up test pack-------------------
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DataTable dataTable = dbSchemaTemplate.CreateDatabasesDataTable();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dataTable);
            Assert.AreEqual(DBSchemaTemplate.DataTableNaming.Databases, dataTable.TableName);
            Assert.AreEqual(2, dataTable.Columns.Count);
            Assert.AreEqual(DBSchemaTemplate.Naming.CatalogName, dataTable.Columns[0].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.SchemaName, dataTable.Columns[1].ColumnName);

            //General:  CATALOG_NAME, SCHEMA_NAME
            //MYSQL:    CATALOG_NAME,database_name,DEFAULT_CHARACTER_SET_NAME,DEFAULT_COLLATION_NAME,SQL_PATH
            //SQLSVR:   database_name,dbid,create_date
            //Oracle:   (Users) NAME,ID,CREATEDATE
        }

        [Test]
        public void Test_CreateDataTypesDataTable()
        {
            //---------------Set up test pack-------------------
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DataTable dataTable = dbSchemaTemplate.CreateDataTypesDataTable();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dataTable);
            Assert.AreEqual(DBSchemaTemplate.DataTableNaming.DataTypes, dataTable.TableName);
            Assert.AreEqual(22, dataTable.Columns.Count);
            Assert.AreEqual(DBSchemaTemplate.Naming.TypeName, dataTable.Columns[0].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ProviderDbType, dataTable.Columns[1].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnSize, dataTable.Columns[2].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.CreateFormat, dataTable.Columns[3].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.CreateParameters, dataTable.Columns[4].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.SystemType, dataTable.Columns[5].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsAutoIncrementable, dataTable.Columns[6].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsBestMatch, dataTable.Columns[7].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsCaseSensitive, dataTable.Columns[8].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsFixedLength, dataTable.Columns[9].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsFixedPrecisionScale, dataTable.Columns[10].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsLong, dataTable.Columns[11].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsNullable, dataTable.Columns[12].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsSearchable, dataTable.Columns[13].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsSearchableWithLike, dataTable.Columns[14].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsUnsigned, dataTable.Columns[15].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.MaximumScale, dataTable.Columns[16].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.MinimumScale, dataTable.Columns[17].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsConcurrencyType, dataTable.Columns[18].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.IsLiteralSupported, dataTable.Columns[19].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.LiteralPrefix, dataTable.Columns[20].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.LiteralSuffix, dataTable.Columns[21].ColumnName);


            //General:  TypeName,ProviderDbType,ColumnSize,CreateFormat,CreateParameters,DataType,IsAutoincrementable,IsBestMatch,IsCaseSensitive,IsFixedLength,IsFixedPrecisionScale,IsLong,IsNullable,IsSearchable,IsSearchableWithLike,IsUnsigned,MaximumScale,MinimumScale,IsConcurrencyType,IsLiteralSupported,LiteralPrefix,LiteralSuffix
            //MYSQL:    TypeName,ProviderDbType,ColumnSize,CreateFormat,CreateParameters,DataType,IsAutoincrementable,IsBestMatch,IsCaseSensitive,IsFixedLength,IsFixedPrecisionScale,IsLong,IsNullable,IsSearchable,IsSearchableWithLike,IsUnsigned,MaximumScale,MinimumScale,IsConcurrencyType,IsLiteralsSupported,LiteralPrefix,LiteralSuffix,NativeDataType
            //SQLSVR:   TypeName,ProviderDbType,ColumnSize,CreateFormat,CreateParameters,DataType,IsAutoIncrementable,IsBestMatch,IsCaseSensitive,IsFixedLength,IsFixedPrecisionScale,IsLong,IsNullable,IsSearchable,IsSearchableWithLike,IsUnsigned,MaximumScale,MinimumScale,IsConcurrencyType,IsLiteralSupported,LiteralPrefix,LiteralSuffix
            //Oracle:   TypeName,ProviderDbType,ColumnSize,CreateFormat,CreateParameters,DataType,IsAutoIncrementable,IsBestMatch,IsCaseSensitive,IsFixedLength,IsFixedPrecisionScale,IsLong,IsNullable,IsSearchable,IsSearchableWithLike,IsUnsigned,MaximumScale,MinimumScale,IsConcurrencyType,IsLiteralSupported,LiteralPrefix,LiteralSuffix
        }

        [Test]
        public void Test_CreateTablesDataTable()
        {
            //---------------Set up test pack-------------------
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DataTable dataTable = dbSchemaTemplate.CreateTablesDataTable();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dataTable);
            Assert.AreEqual(DBSchemaTemplate.DataTableNaming.Tables, dataTable.TableName);
            Assert.AreEqual(4, dataTable.Columns.Count);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableCatalog, dataTable.Columns[0].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableSchema, dataTable.Columns[1].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableName, dataTable.Columns[2].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableType, dataTable.Columns[3].ColumnName);

            //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,TABLE_TYPE
            //MYSQL:    TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,TABLE_TYPE,ENGINE,VERSION,ROW_FORMAT,TABLE_ROWS,AVG_ROW_LENGTH,DATA_LENGTH,MAX_DATA_LENGTH,INDEX_LENGTH,DATA_FREE,AUTO_INCREMENT,CREATE_TIME,UPDATE_TIME,CHECK_TIME,TABLE_COLLATION,CHECKSUM,CREATE_OPTIONS,TABLE_COMMENT
            //SQLSVR:   TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE,CHARACTER_SET_NAME,COLLATION_NAME,COLUMN_TYPE,COLUMN_KEY,EXTRA,PRIVILEGES,COLUMN_COMMENT
            //Oracle:   OWNER,TABLE_NAME,TYPE
        }

        [Test]
        public void Test_CreateColumnsDataTable()
        {
            //---------------Set up test pack-------------------
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DataTable dataTable = dbSchemaTemplate.CreateColumnsDataTable();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dataTable);
            Assert.AreEqual(DBSchemaTemplate.DataTableNaming.Columns, dataTable.TableName);
            Assert.AreEqual(12, dataTable.Columns.Count);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableCatalog, dataTable.Columns[0].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableSchema, dataTable.Columns[1].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableName, dataTable.Columns[2].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnName, dataTable.Columns[3].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.OrdinalPosition, dataTable.Columns[4].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnDefault, dataTable.Columns[5].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnIsNullable, dataTable.Columns[6].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.DataType, dataTable.Columns[7].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.CharacterMaximumLength, dataTable.Columns[8].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.NumericPrecision, dataTable.Columns[9].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.NumericScale, dataTable.Columns[10].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnIsAutoIncrement, dataTable.Columns[11].ColumnName);
            
            //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE, IS_AUTO_INCREMENTING
            //MySQL:    TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,NUMERIC_PRECISION,NUMERIC_SCALE,CHARACTER_SET_NAME,COLLATION_NAME,COLUMN_TYPE,COLUMN_KEY,EXTRA,PRIVILEGES,COLUMN_COMMENT
            //SQLSVR:   TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION,COLUMN_DEFAULT,IS_NULLABLE,DATA_TYPE,CHARACTER_MAXIMUM_LENGTH,CHARACTER_OCTET_LENGTH,NUMERIC_PRECISION,NUMERIC_PRECISION_RADIX,NUMERIC_SCALE,DATETIME_PRECISION,CHARACTER_SET_CATALOG,CHARACTER_SET_SCHEMA,CHARACTER_SET_NAME,COLLATION_CATALOG
            //Oracle:   OWNER,TABLE_NAME,COLUMN_NAME,ID,DATATYPE,LENGTH,PRECISION,SCALE,NULLABLE
        }

        [Test]
        public void Test_CreatePrimaryKeysDataTable()
        {
            //---------------Set up test pack-------------------
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DataTable dataTable = dbSchemaTemplate.CreatePrimaryKeyColumnsDataTable();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dataTable);
            Assert.AreEqual(DBSchemaTemplate.DataTableNaming.PrimaryKeys, dataTable.TableName);
            Assert.AreEqual(8, dataTable.Columns.Count);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableCatalog, dataTable.Columns[0].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableSchema, dataTable.Columns[1].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableName, dataTable.Columns[2].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeyCatalog, dataTable.Columns[3].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeySchema, dataTable.Columns[4].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeyName, dataTable.Columns[5].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnName, dataTable.Columns[6].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.OrdinalPosition, dataTable.Columns[7].ColumnName);

            //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,KEY_CATALOG,KEY_SCHEMA,KEY_NAME,COLUMN_NAME,ORDINAL_POSITION

            //MySQL:    Indexes / IndexColumns
            //          INDEX_CATALOG,INDEX_SCHEMA,INDEX_NAME,TABLE_NAME,UNIQUE,PRIMARY
            //          INDEX_CATALOG,INDEX_SCHEMA,INDEX_NAME,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION

            //SQLSVR:   Indexes / IndexColumns
            //          constraint_catalog,constraint_schema,constraint_name,table_catalog,table_schema,table_name,index_name
            //          constraint_catalog,constraint_schema,constraint_name,table_catalog,table_schema,table_name,column_name,ordinal_position,KeyType,index_name

            //Oracle:   OWNER,CONSTRAINT_NAME,TABLE_NAME,SEARCH_CONDITION,R_OWNER,R_CONSTRAINT_NAME,DELETE_RULE,STATUS,DEFERRABLE,VALIDATED,GENERATED,BAD,RELY,LAST_CHANGE,INDEX_OWNER,INDEX_NAME
            //
        }

        [Test]
        public void Test_CreateUniqueKeysDataTable()
        {
            //---------------Set up test pack-------------------
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DataTable dataTable = dbSchemaTemplate.CreateUniqueKeyColumnsDataTable();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dataTable);
            Assert.AreEqual(DBSchemaTemplate.DataTableNaming.UniqueKeys, dataTable.TableName);
            Assert.AreEqual(8, dataTable.Columns.Count);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableCatalog, dataTable.Columns[0].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableSchema, dataTable.Columns[1].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableName, dataTable.Columns[2].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeyCatalog, dataTable.Columns[3].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeySchema, dataTable.Columns[4].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeyName, dataTable.Columns[5].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnName, dataTable.Columns[6].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.OrdinalPosition, dataTable.Columns[7].ColumnName);

            //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,KEY_CATALOG,KEY_SCHEMA,KEY_NAME,COLUMN_NAME,ORDINAL_POSITION

            //MySQL:    INDEX_CATALOG,INDEX_SCHEMA,INDEX_NAME,TABLE_NAME,UNIQUE,PRIMARY
            //          INDEX_CATALOG,INDEX_SCHEMA,INDEX_NAME,TABLE_NAME,COLUMN_NAME,ORDINAL_POSITION

            //SQLSVR:   constraint_catalog,constraint_schema,constraint_name,table_catalog,table_schema,table_name,index_name
            //          constraint_catalog,constraint_schema,constraint_name,table_catalog,table_schema,table_name,column_name,ordinal_position,KeyType,index_name

            //Oracle:   OWNER,CONSTRAINT_NAME,TABLE_NAME,SEARCH_CONDITION,R_OWNER,R_CONSTRAINT_NAME,DELETE_RULE,STATUS,DEFERRABLE,VALIDATED,GENERATED,BAD,RELY,LAST_CHANGE,INDEX_OWNER,INDEX_NAME
            //
        }

        [Test]
        public void Test_CreateForeignKeyColumnsDataTable()
        {
            //---------------Set up test pack-------------------
            DBSchemaTemplate dbSchemaTemplate = new DBSchemaTemplate();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DataTable dataTable = dbSchemaTemplate.CreateForeignKeyColumnsDataTable();
            //---------------Test Result -----------------------
            Assert.IsNotNull(dataTable);
            Assert.AreEqual(DBSchemaTemplate.DataTableNaming.ForeignKeyColumns, dataTable.TableName);
            Assert.AreEqual(12, dataTable.Columns.Count);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableCatalog, dataTable.Columns[0].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableSchema, dataTable.Columns[1].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.TableName, dataTable.Columns[2].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeyCatalog, dataTable.Columns[3].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeySchema, dataTable.Columns[4].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.KeyName, dataTable.Columns[5].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ColumnName, dataTable.Columns[6].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ReferencedTableCatalog, dataTable.Columns[7].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ReferencedTableSchema, dataTable.Columns[8].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ReferencedTableName, dataTable.Columns[9].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.ReferencedColumnName, dataTable.Columns[10].ColumnName);
            Assert.AreEqual(DBSchemaTemplate.Naming.OrdinalPosition, dataTable.Columns[11].ColumnName);


            //General:  TABLE_CATALOG,TABLE_SCHEMA,TABLE_NAME,
            //          KEY_CATALOG,KEY_SCHEMA,KEY_NAME,COLUMN_NAME, 
            //          REFERENCED_TABLE_CATALOG, REFERENCED_TABLE_SCHEMA,REFERENCED_TABLE_NAME, 
            //          REFERENCED_COLUMN_NAME, ORDINAL_POSITION
            //MySQL:    N/A
            //SQLSVR:   N/A
            //Oracle:   OWNER,CONSTRAINT_NAME,TABLE_NAME,COLUMN_NAME,POSITION
        }
    }

    
}
