using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using NUnit.Framework;

namespace Firestarter.DB.Schema.Test.Logic
{
    [TestFixture]
    public class TestTableInfo : TestBase
    {
        [Test]
        public void Test_DefaultPropertyValues()
        {
            //---------------Set up test pack-------------------
            
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            DBTable dbTable = new DBTable();
            //---------------Test Result -----------------------
            Assert.AreEqual(ChangeType.None, dbTable.PendingChangeType);
        }

        [Test]
        public void TestGetColumnInfo_NullWhenNoColumnsExist()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, dbTable.Columns.Count);
            //---------------Execute Test ----------------------
            string nonExistentColumn = TestUtils.GetRandomString();
            DBColumn dbColumn = dbTable.GetColumnInfo(nonExistentColumn);
            //---------------Test Result -----------------------
            Assert.IsNull(dbColumn);
        }

        [Test]
        public void TestGetColumnInfo_NullIfNotFound()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();

            DBColumn firstDBColumn = new DBColumn();
            firstDBColumn.ColumnName = "TestColumn1";
            dbTable.Columns.Add(firstDBColumn);

            DBColumn secondDBColumn = new DBColumn();
            secondDBColumn.ColumnName = "TestColumn2";
            dbTable.Columns.Add(secondDBColumn);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string nonExistentColumn = TestUtils.GetRandomString();
            DBColumn dbColumn = dbTable.GetColumnInfo(nonExistentColumn);
            //---------------Test Result -----------------------
            Assert.IsNull(dbColumn);
        }

        [Test]
        public void TestGetColumnInfo_ReturnsCorrectColumnInfoIfExists()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();

            DBColumn firstDBColumn = new DBColumn();
            firstDBColumn.ColumnName = "TestColumn1";
            dbTable.Columns.Add(firstDBColumn);

            DBColumn secondDBColumn = new DBColumn();
            secondDBColumn.ColumnName = "TestColumn2";
            dbTable.Columns.Add(secondDBColumn);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string existentColumnName = "TestColumn1";
            DBColumn foundDBColumn = dbTable.GetColumnInfo(existentColumnName);
            //---------------Test Result -----------------------
            Assert.IsNotNull(foundDBColumn);
            Assert.AreEqual(existentColumnName, foundDBColumn.ColumnName);
        }

        [Test]
        public void TestGetKeyInfo_NullWhenNoColumnsExist()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, dbTable.Keys.Count);
            //---------------Execute Test ----------------------
            string nonExistentKey = TestUtils.GetRandomString();
            DBKey dbKey = dbTable.GetKeyInfo(nonExistentKey);
            //---------------Test Result -----------------------
            Assert.IsNull(dbKey);
        }

        [Test]
        public void TestGetKeyInfo_NullIfNotFound()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();

            DBKey dbKeyForTable = new DBKey();
            dbKeyForTable.KeyName = "TestKey";
            dbTable.Keys.Add(dbKeyForTable);

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string nonExistentKey = TestUtils.GetRandomString();
            DBKey dbKey = dbTable.GetKeyInfo(nonExistentKey);
            //---------------Test Result -----------------------
            Assert.IsNull(dbKey);
        }

        [Test]
        public void TestGetKeyInfo_ReturnsCorrectKeyInfoIfExists()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();

            DBKey firstDBKey = new DBKey();
            firstDBKey.KeyName = "TestKey1";
            dbTable.Keys.Add(firstDBKey);

            DBKey secondDBKey = new DBKey();
            string secondKeyInfoName = "TestKey2";
            secondDBKey.KeyName = secondKeyInfoName;
            dbTable.Keys.Add(secondDBKey);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            DBKey foundDBKey = dbTable.GetKeyInfo(secondKeyInfoName);
            //---------------Test Result -----------------------
            Assert.IsNotNull(foundDBKey);
            Assert.AreEqual(secondKeyInfoName, foundDBKey.KeyName);
        }


        [Test]
        public void TestGetForeignKeyInfo_NullWhenNoColumnsExist()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();
            //---------------Assert Precondition----------------
            Assert.AreEqual(0, dbTable.ForeignKeys.Count);
            //---------------Execute Test ----------------------
            string nonExistentForeignKey = TestUtils.GetRandomString();
            DBForeignKey dbForeignKey = dbTable.GetForeignKeyInfo(nonExistentForeignKey);
            //---------------Test Result -----------------------
            Assert.IsNull(dbForeignKey);
        }

        [Test]
        public void TestGetForeignKeyInfo_NullIfNotFound()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();

            DBForeignKey dbForeignKeyForTable = new DBForeignKey();
            dbForeignKeyForTable.ForeignKeyName = "TestForeignKeyColumn1";
            dbTable.ForeignKeys.Add(dbForeignKeyForTable);

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string nonExistentForeignKey = TestUtils.GetRandomString();
            DBForeignKey dbForeignKey = dbTable.GetForeignKeyInfo(nonExistentForeignKey);
            //---------------Test Result -----------------------
            Assert.IsNull(dbForeignKey);
        }

        [Test]
        public void TestGetForeignKeyInfo_ReturnsCorrectColumnInfoIfExists()
        {
            //---------------Set up test pack-------------------
            DBTable dbTable = new DBTable();

            DBForeignKey firstDBForeignKey = new DBForeignKey();
            firstDBForeignKey.ForeignKeyName = "TestForeignKeyColumn1";
            dbTable.ForeignKeys.Add(firstDBForeignKey);

            DBForeignKey secondDBForeignKey = new DBForeignKey();
            secondDBForeignKey.ForeignKeyName = "TestForeignKeyColumn2";
            dbTable.ForeignKeys.Add(secondDBForeignKey);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            string existentForeignKeyName = "TestForeignKeyColumn2";
            DBForeignKey foundDBForeignKey = dbTable.GetForeignKeyInfo(existentForeignKeyName);
            //---------------Test Result -----------------------
            Assert.IsNotNull(foundDBForeignKey);
            Assert.AreEqual(existentForeignKeyName, foundDBForeignKey.ForeignKeyName);
        }
    }
}
