using System.Windows.Forms;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using Firestarter.DB.Schema.Logic;
using FireStarterModeller.BO;
using Firestarter.DB.Schema.UI;
using FireStarterModeller.Test;
using FireStarterModeller.UI;
using Habanero.Base;
using Habanero.Faces.Base;
using NUnit.Framework;
using Firestarter.DB.Schema.Test.Logic;
using TreeViewController=Habanero.Faces.Base.TreeViewController;

namespace Firestarter.DB.Schema.Test.UI
{
    [TestFixture]
    public class TestDatabaseInfoChangesControl: TestBase
    {
        [Test]
        public void Test_DatabaseInfo()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase= new DBDatabase();
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            Assert.AreSame(dbDatabase, databaseInfoChangesControl.Database);
        }

        [Test]
        public void Test_NodeImages()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBColumn updatedDBColumn = new DBColumn("UpdatedColumn") { PendingChangeType = ChangeType.Update };
            dbTable.Columns.Add(updatedDBColumn);
            DBColumn addedDBColumn = new DBColumn("AddedColumn") { PendingChangeType = ChangeType.Add };
            dbTable.Columns.Add(addedDBColumn);
            DBColumn deletedDBColumn = new DBColumn("DeletedColumn") { PendingChangeType = ChangeType.Delete };
            dbTable.Columns.Add(deletedDBColumn);
            DBColumn unchangedDBColumn = new DBColumn("UnchangedColumn") { PendingChangeType = ChangeType.None };
            dbTable.Columns.Add(unchangedDBColumn);

            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions -------------
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            AssertColumnImageKeyCorrect("Change", treeViewController, dbDatabase);
            AssertColumnImageKeyCorrect("Change", treeViewController, dbTable);
            AssertColumnImageKeyCorrect("Change", treeViewController, updatedDBColumn);
            AssertColumnImageKeyCorrect("Add", treeViewController, addedDBColumn);
            AssertColumnImageKeyCorrect("Delete", treeViewController, deletedDBColumn);
            AssertColumnImageKeyCorrect("None", treeViewController, unchangedDBColumn);
        }

        [Test]
        public void Test_NodeImages_ChangeWhenBoUpdated()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.None };
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            databaseInfoChangesControl.Database = dbDatabase;
            //-------------Assert Preconditions -------------
            AssertColumnImageKeyCorrect("None", treeViewController, dbDatabase);
            //---------------Execute Test ----------------------
            dbDatabase.PendingChangeType = ChangeType.Update;
            //---------------Test Result -----------------------
            AssertColumnImageKeyCorrect("Change", treeViewController, dbDatabase);
        }

        private static void AssertColumnImageKeyCorrect(string expectedImageKey, TreeViewController treeViewController, IBusinessObject businessObject)
        {
            ImageList imageList = ((TreeView)treeViewController.TreeView).ImageList;
            int expectedImageIndex = imageList.Images.IndexOfKey(expectedImageKey);
            Assert.AreEqual(expectedImageIndex, treeViewController.GetBusinessObjectTreeNode(businessObject).SelectedImageIndex);
            Assert.AreEqual(expectedImageIndex, treeViewController.GetBusinessObjectTreeNode(businessObject).ImageIndex);
        }

        [Test]
        public void Test_NodeText_Table()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);

            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(dbTable);
            Assert.AreEqual(dbTable.TableNameDB, treeNode.Text);
        }

        [Test]
        public void Test_NodeText_Column()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBColumn updatedDBColumn = new DBColumn("UpdatedColumn") { PendingChangeType = ChangeType.Update };
            dbTable.Columns.Add(updatedDBColumn);

            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(updatedDBColumn);
            Assert.AreEqual(updatedDBColumn.ColumnName, treeNode.Text);
        }

        [Test]
        public void Test_NodeText_PrimaryKey()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBColumn primaryKeyColumn = dbTableBuilder.AddPrimaryKeyColumn().Column;
            DBPrimaryKey primaryKey = dbTable.PrimaryKeyDB;
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(primaryKeyColumn);
            Assert.IsNotNull(primaryKey);
            Assert.AreEqual(1, primaryKey.Columns.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(primaryKey);
            Assert.AreEqual("Primary", treeNode.Text);
        }

        [Test]
        public void Test_NodeText_PrimaryKeyColumn()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBColumn primaryKeyColumn = dbTableBuilder.AddPrimaryKeyColumn().Column;
            DBPrimaryKey primaryKey = dbTable.PrimaryKeyDB;
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(primaryKeyColumn);
            Assert.IsNotNull(primaryKey);
            Assert.AreEqual(1, primaryKey.Columns.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(primaryKeyColumn);
            Assert.AreEqual(primaryKeyColumn.ColumnName, treeNode.Text);
        }

        [Test]
        public void Test_NodeText_UniqueKey()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBColumn uniqueKeyColumn = dbTableBuilder.AddUniqueKeyColumn("ExistingAKColumn");
            DBKey uniqueKey = dbTable.Keys[0];
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(uniqueKeyColumn);
            Assert.IsNotNull(uniqueKey);
            Assert.AreEqual(1, uniqueKey.Columns.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(uniqueKey);
            Assert.AreEqual(uniqueKey.KeyName, treeNode.Text);
        }

        [Test]
        public void Test_NodeText_UniqueKeyColumn()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBColumn uniqueKeyColumn = dbTableBuilder.AddUniqueKeyColumn("ExistingAKColumn");
            DBKey uniqueKey = dbTable.Keys[0];
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(uniqueKeyColumn);
            Assert.IsNotNull(uniqueKey);
            Assert.AreEqual(1, uniqueKey.Columns.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(uniqueKeyColumn);
            Assert.AreEqual(uniqueKeyColumn.ColumnName, treeNode.Text);
        }

        [Test]
        public void Test_NodeText_ForeignKey()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTable dbParentTable = new DBTable("MyLookupTable") { PendingChangeType = ChangeType.Update };
            DBTableBuilder dbParentTableBuilder = new DBTableBuilder(dbParentTable);
            dbParentTableBuilder.AddPrimaryKeyColumn();
            dbDatabase.Tables.Add(dbParentTable);

            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBForeignKey foreignKey = dbTableBuilder.AddForeignKey("MyForeignKey", dbParentTable);

            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(foreignKey);
            Assert.IsNotNull(foreignKey.ParentTable);
            Assert.IsNotNull(foreignKey.RelatedTable);
            Assert.AreEqual(1, dbTable.ForeignKeys.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(foreignKey);
            Assert.AreEqual(foreignKey.ForeignKeyName + " (" + foreignKey.RelatedTable.TableNameDB + ")", treeNode.Text);
        }

        [Test]
        public void Test_NodeText_ForeignKeyColumn()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTable dbLookupTable = new DBTable("MyLookupTable") { PendingChangeType = ChangeType.Update };
            DBTableBuilder lookupTableBuilder = new DBTableBuilder(dbLookupTable);
            DBColumn lookupTablePrimaryKeyColumn = lookupTableBuilder.AddPrimaryKeyColumn().Column;
            dbDatabase.Tables.Add(dbLookupTable);

            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBForeignKey dbForeignKey = dbTableBuilder.AddForeignKey("MyForeignKey", dbLookupTable);
            DBForeignKeyColumn foreignKeyColumn = dbForeignKey.ForeignKeyColumns[0];
            DBColumn foreignKeyIDColumn = foreignKeyColumn.ParentColumn;
            foreignKeyIDColumn.ColumnName = "MyForeignKeyIDColumn";
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(dbForeignKey);
            Assert.AreEqual(1, dbForeignKey.ForeignKeyColumns.Count);
            Assert.IsNotNull(foreignKeyColumn);
            Assert.IsNotNull(foreignKeyIDColumn);
            Assert.IsNotNull(lookupTablePrimaryKeyColumn);
            Assert.AreEqual(1, dbTable.ForeignKeys.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(foreignKeyColumn);
            Assert.AreEqual(string.Format("{0} -> {1}", foreignKeyIDColumn.ColumnName, lookupTablePrimaryKeyColumn.ColumnName), treeNode.Text);
        }

        [Test]
        public void Test_NodeText_ForeignKeyColumn_WhenRelatedColumnNull()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTable dbLookupTable = new DBTable("MyLookupTable") { PendingChangeType = ChangeType.Update };
            DBTableBuilder lookupTableBuilder = new DBTableBuilder(dbLookupTable);
            DBColumn lookupTablePrimaryKeyColumn = lookupTableBuilder.AddPrimaryKeyColumn().Column;
            dbDatabase.Tables.Add(dbLookupTable);

            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBForeignKey dbForeignKey = dbTableBuilder.AddForeignKey("MyForeignKey", dbLookupTable);
            DBForeignKeyColumn foreignKeyColumn = dbForeignKey.ForeignKeyColumns[0];
            DBColumn foreignKeyIDColumn = foreignKeyColumn.ParentColumn;
            foreignKeyIDColumn.ColumnName = "MyForeignKeyIDColumn";
            foreignKeyColumn.RelatedColumn = null;
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(dbForeignKey);
            Assert.AreEqual(1, dbForeignKey.ForeignKeyColumns.Count);
            Assert.IsNotNull(foreignKeyColumn);
            Assert.IsNotNull(foreignKeyIDColumn);
            Assert.IsNotNull(lookupTablePrimaryKeyColumn);
            Assert.AreEqual(1, dbTable.ForeignKeys.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(foreignKeyColumn);
            Assert.AreEqual(string.Format("{0} -> {1}", foreignKeyIDColumn.ColumnName, "(Unspecified)"), treeNode.Text);
        }

        [Test]
        public void Test_NodeText_ForeignKeyColumn_WhenParentColumnNull()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTable dbLookupTable = new DBTable("MyLookupTable") { PendingChangeType = ChangeType.Update };
            DBTableBuilder lookupTableBuilder = new DBTableBuilder(dbLookupTable);
            DBColumn lookupTablePrimaryKeyColumn = lookupTableBuilder.AddPrimaryKeyColumn().Column;
            dbDatabase.Tables.Add(dbLookupTable);

            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBForeignKey dbForeignKey = dbTableBuilder.AddForeignKey("MyForeignKey", dbLookupTable);
            DBForeignKeyColumn foreignKeyColumn = dbForeignKey.ForeignKeyColumns[0];
            DBColumn foreignKeyIDColumn = foreignKeyColumn.ParentColumn;
            foreignKeyIDColumn.ColumnName = "MyForeignKeyIDColumn";
            foreignKeyColumn.ParentColumn = null;
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(dbForeignKey);
            Assert.AreEqual(1, dbForeignKey.ForeignKeyColumns.Count);
            Assert.IsNotNull(foreignKeyColumn);
            Assert.IsNotNull(foreignKeyIDColumn);
            Assert.IsNotNull(lookupTablePrimaryKeyColumn);
            Assert.AreEqual(1, dbTable.ForeignKeys.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(foreignKeyColumn);
            Assert.AreEqual(string.Format("{0} -> {1}", "(Unspecified)", lookupTablePrimaryKeyColumn.ColumnName), treeNode.Text);
        }

        [Test]
        public void Test_NodeText_ForeignKeyColumn_WhenParentColumnNullAndRelatedColumnNull()
        {
            DBDatabase dbDatabase = new DBDatabase { PendingChangeType = ChangeType.Update };
            DBTable dbTable = new DBTable("MyUpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(dbTable);
            DBTable dbLookupTable = new DBTable("MyLookupTable") { PendingChangeType = ChangeType.Update };
            DBTableBuilder lookupTableBuilder = new DBTableBuilder(dbLookupTable);
            DBColumn lookupTablePrimaryKeyColumn = lookupTableBuilder.AddPrimaryKeyColumn().Column;
            dbDatabase.Tables.Add(dbLookupTable);

            DBTableBuilder dbTableBuilder = new DBTableBuilder(dbTable);
            DBForeignKey dbForeignKey = dbTableBuilder.AddForeignKey("MyForeignKey", dbLookupTable);
            DBForeignKeyColumn foreignKeyColumn = dbForeignKey.ForeignKeyColumns[0];
            DBColumn foreignKeyIDColumn = foreignKeyColumn.ParentColumn;
            foreignKeyIDColumn.ColumnName = "MyForeignKeyIDColumn";
            foreignKeyColumn.ParentColumn = null;
            foreignKeyColumn.RelatedColumn = null;
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController treeViewController = databaseInfoChangesControl.TreeViewController;
            //-------------Assert Preconditions ----------------
            Assert.IsNotNull(dbForeignKey);
            Assert.AreEqual(1, dbForeignKey.ForeignKeyColumns.Count);
            Assert.IsNotNull(foreignKeyColumn);
            Assert.IsNotNull(foreignKeyIDColumn);
            Assert.IsNotNull(lookupTablePrimaryKeyColumn);
            Assert.AreEqual(1, dbTable.ForeignKeys.Count);
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode treeNode = treeViewController.GetBusinessObjectTreeNode(foreignKeyColumn);
            Assert.AreEqual(string.Format("{0} -> {1}", "(Unspecified)", "(Unspecified)"), treeNode.Text);
        }

        [Test]
        public void Test_Filter_Defaults()
        {
            //---------------Set up test pack-------------------
            //-------------Assert Preconditions -------------

            //---------------Execute Test ----------------------
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            //---------------Test Result -----------------------
            IPanel panel = databaseInfoChangesControl.FilterPanel;
            Assert.IsNotNull(panel);
            Assert.AreEqual(4, panel.Controls.Count);
            AssertCheckBoxDetails("Unchanged", true, panel.Controls[0]);
            AssertCheckBoxDetails("Added", true, panel.Controls[1]);
            AssertCheckBoxDetails("Removed", true, panel.Controls[2]);
            AssertCheckBoxDetails("Updated", true, panel.Controls[3]);
        }

        private static void AssertCheckBoxDetails(string expectedText, bool checkedValue, IControlHabanero control)
        {
            Assert.IsInstanceOf(typeof(ICheckBox), control);
            ICheckBox checkBox = (ICheckBox) control;
            Assert.AreEqual(checkedValue, checkBox.Checked);
            Assert.AreEqual(expectedText, checkBox.Text);
        }

        [Test]
        public void Test_Filter_UncheckUnchanged()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable unchangedDBTable = new DBTable("UnchangedTable") { PendingChangeType = ChangeType.None };
            dbDatabase.Tables.Add(unchangedDBTable);
            DBTable addedDBTable = new DBTable("AddedTable") { PendingChangeType = ChangeType.Add };
            dbDatabase.Tables.Add(addedDBTable);
            DBTable deletedDBTable = new DBTable("DeletedTable") { PendingChangeType = ChangeType.Delete };
            dbDatabase.Tables.Add(deletedDBTable);
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeView treeView = databaseInfoChangesControl.TreeViewController.TreeView;
            ITreeNode tableInfosNode = treeView.Nodes[0].Nodes[0];
            ICheckBox checkbox = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[0];
            //-------------Assert Preconditions -------------
            Assert.AreEqual("Unchanged", checkbox.Text);
            Assert.AreEqual(true, checkbox.Checked);
            Assert.AreEqual(4, tableInfosNode.Nodes.Count);
            //---------------Execute Test ----------------------
            checkbox.Checked = false;
            //---------------Test Result -----------------------
            Assert.AreEqual(3, tableInfosNode.Nodes.Count);
            Assert.AreEqual(addedDBTable, tableInfosNode.Nodes[0].Tag);
            Assert.AreEqual(deletedDBTable, tableInfosNode.Nodes[1].Tag);
            Assert.AreEqual(updatedDBTable, tableInfosNode.Nodes[2].Tag);
        }

        [Test]
        public void Test_Filter_UncheckAdded()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable unchangedDBTable = new DBTable("UnchangedTable") { PendingChangeType = ChangeType.None };
            dbDatabase.Tables.Add(unchangedDBTable);
            DBTable addedDBTable = new DBTable("AddedTable") { PendingChangeType = ChangeType.Add };
            dbDatabase.Tables.Add(addedDBTable);
            DBTable deletedDBTable = new DBTable("DeletedTable") { PendingChangeType = ChangeType.Delete };
            dbDatabase.Tables.Add(deletedDBTable);
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeView treeView = databaseInfoChangesControl.TreeViewController.TreeView;
            ITreeNode tableInfosNode = treeView.Nodes[0].Nodes[0];
            ICheckBox checkbox = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[1];
            //-------------Assert Preconditions -------------
            Assert.AreEqual("Added", checkbox.Text);
            Assert.AreEqual(true, checkbox.Checked);
            Assert.AreEqual(4, tableInfosNode.Nodes.Count);
            //---------------Execute Test ----------------------
            checkbox.Checked = false;
            //---------------Test Result -----------------------
            Assert.AreEqual(3, tableInfosNode.Nodes.Count);
            Assert.AreEqual(unchangedDBTable, tableInfosNode.Nodes[0].Tag);
            Assert.AreEqual(deletedDBTable, tableInfosNode.Nodes[1].Tag);
            Assert.AreEqual(updatedDBTable, tableInfosNode.Nodes[2].Tag);
        }

        [Test]
        public void Test_Filter_UncheckDeleted()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable unchangedDBTable = new DBTable("UnchangedTable") { PendingChangeType = ChangeType.None };
            dbDatabase.Tables.Add(unchangedDBTable);
            DBTable addedDBTable = new DBTable("AddedTable") { PendingChangeType = ChangeType.Add };
            dbDatabase.Tables.Add(addedDBTable);
            DBTable deletedDBTable = new DBTable("DeletedTable") { PendingChangeType = ChangeType.Delete };
            dbDatabase.Tables.Add(deletedDBTable);
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeView treeView = databaseInfoChangesControl.TreeViewController.TreeView;
            ITreeNode tableInfosNode = treeView.Nodes[0].Nodes[0];
            ICheckBox checkbox = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[2];
            //-------------Assert Preconditions -------------
            Assert.AreEqual("Removed", checkbox.Text);
            Assert.AreEqual(true, checkbox.Checked);
            Assert.AreEqual(4, tableInfosNode.Nodes.Count);
            //---------------Execute Test ----------------------
            checkbox.Checked = false;
            //---------------Test Result -----------------------
            Assert.AreEqual(3, tableInfosNode.Nodes.Count);
            Assert.AreEqual(unchangedDBTable, tableInfosNode.Nodes[0].Tag);
            Assert.AreEqual(addedDBTable, tableInfosNode.Nodes[1].Tag);
            Assert.AreEqual(updatedDBTable, tableInfosNode.Nodes[2].Tag);
        }

        [Test]
        public void Test_Filter_UncheckUpdated()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable unchangedDBTable = new DBTable("UnchangedTable") { PendingChangeType = ChangeType.None };
            dbDatabase.Tables.Add(unchangedDBTable);
            DBTable addedDBTable = new DBTable("AddedTable") { PendingChangeType = ChangeType.Add };
            dbDatabase.Tables.Add(addedDBTable);
            DBTable deletedDBTable = new DBTable("DeletedTable") { PendingChangeType = ChangeType.Delete };
            dbDatabase.Tables.Add(deletedDBTable);
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeView treeView = databaseInfoChangesControl.TreeViewController.TreeView;
            ITreeNode tableInfosNode = treeView.Nodes[0].Nodes[0];
            ICheckBox checkbox = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[3];
            //-------------Assert Preconditions -------------
            Assert.AreEqual("Updated", checkbox.Text);
            Assert.AreEqual(true, checkbox.Checked);
            Assert.AreEqual(4, tableInfosNode.Nodes.Count);
            //---------------Execute Test ----------------------
            checkbox.Checked = false;
            //---------------Test Result -----------------------
            Assert.AreEqual(3, tableInfosNode.Nodes.Count);
            Assert.AreEqual(unchangedDBTable, tableInfosNode.Nodes[0].Tag);
            Assert.AreEqual(addedDBTable, tableInfosNode.Nodes[1].Tag);
            Assert.AreEqual(deletedDBTable, tableInfosNode.Nodes[2].Tag);
        }

        [Test]
        public void Test_Filter_UncheckAll()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable unchangedDBTable = new DBTable("UnchangedTable") { PendingChangeType = ChangeType.None };
            dbDatabase.Tables.Add(unchangedDBTable);
            DBTable addedDBTable = new DBTable("AddedTable") { PendingChangeType = ChangeType.Add };
            dbDatabase.Tables.Add(addedDBTable);
            DBTable deletedDBTable = new DBTable("DeletedTable") { PendingChangeType = ChangeType.Delete };
            dbDatabase.Tables.Add(deletedDBTable);
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeView treeView = databaseInfoChangesControl.TreeViewController.TreeView;
            ITreeNode tableInfosNode = treeView.Nodes[0].Nodes[0];
            ICheckBox checkbox = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[0];
            ICheckBox checkbox2 = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[1];
            ICheckBox checkbox3 = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[2];
            ICheckBox checkbox4 = (ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[3];
            //-------------Assert Preconditions -------------
            Assert.AreEqual(4, tableInfosNode.Nodes.Count);
            //---------------Execute Test ----------------------
            checkbox.Checked = false;
            checkbox2.Checked = false;
            checkbox3.Checked = false;
            checkbox4.Checked = false;
            //---------------Test Result -----------------------
            Assert.AreEqual(0, tableInfosNode.Nodes.Count);
        }

        [Test]
        public void Test_Filter_WithChildren_AddedOnly()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update };
            dbDatabase.Tables.Add(updatedDBTable);
            DBTableBuilder dbTableBuilder = new DBTableBuilder(updatedDBTable);
            dbTableBuilder.AddColumn("AddedColumn").PendingChangeType = ChangeType.Add;
            dbTableBuilder.AddColumn("UpdatedColumn").PendingChangeType = ChangeType.Update;

            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeView treeView = databaseInfoChangesControl.TreeViewController.TreeView;
            ITreeNode tablesNode = treeView.Nodes[0].Nodes[0];
            ITreeNode tableColumnsNode = tablesNode.Nodes[0].Nodes[0];
            //-------------Assert Preconditions -------------
            Assert.AreEqual("Tables", tablesNode.Text);
            Assert.AreEqual(1, tablesNode.Nodes.Count);
            Assert.AreEqual("Columns", tableColumnsNode.Text);
            Assert.AreEqual(2, tableColumnsNode.Nodes.Count);
            //---------------Execute Test ----------------------
            ((ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[0]).Checked = false;
            ((ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[2]).Checked = false;
            ((ICheckBox)databaseInfoChangesControl.FilterPanel.Controls[3]).Checked = false;
            //---------------Test Result -----------------------
            Assert.AreEqual(1, tablesNode.Nodes.Count);
            Assert.AreEqual(1, tableColumnsNode.Nodes.Count);
            Assert.AreEqual(updatedDBTable, tablesNode.Nodes[0].Tag);
            Assert.AreEqual(updatedDBTable, tablesNode.Nodes[0].Tag);
        }

        [Test]
        public void Test_SetupNode_ApprovedIsFalse()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable updatedDBTable = new DBTable("UpdatedTable")
                                         {PendingChangeType = ChangeType.Update, Approved = false };
            dbDatabase.Tables.Add(updatedDBTable);
            
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController controller = databaseInfoChangesControl.TreeViewController;
            ITreeView treeView = controller.TreeView;
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode node = controller.GetBusinessObjectTreeNode(updatedDBTable);
            Assert.IsFalse(node.Checked);
        }

        [Test]
        public void Test_SetupNode_ApprovedIsTrue()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable updatedDBTable = new DBTable("UpdatedTable")
                                         {PendingChangeType = ChangeType.Update, Approved = true };
            dbDatabase.Tables.Add(updatedDBTable);
            
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController controller = databaseInfoChangesControl.TreeViewController;
            ITreeView treeView = controller.TreeView;
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode node = controller.GetBusinessObjectTreeNode(updatedDBTable);
            Assert.IsTrue(node.Checked);
        }

        [Test]
        public void Test_NodeCheckChanged_ToTrue_ShouldUpdateApproved()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update, Approved = false };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController controller = databaseInfoChangesControl.TreeViewController;
            ITreeView treeView = controller.TreeView;
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeNode node = controller.GetBusinessObjectTreeNode(updatedDBTable);
            //---------------Assert Precondition----------------
            Assert.IsFalse(node.Checked);
            Assert.IsFalse(updatedDBTable.PendingChangeApproved);
            //---------------Execute Test ----------------------
            node.Checked = true;
            //---------------Test Result -----------------------
            Assert.IsTrue(node.Checked);
            Assert.IsTrue(updatedDBTable.PendingChangeApproved);
        }

        [Test]
        public void Test_NodeCheckChanged_ToFalse_ShouldUpdateApproved()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update, Approved = true };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController controller = databaseInfoChangesControl.TreeViewController;
            ITreeView treeView = controller.TreeView;
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeNode node = controller.GetBusinessObjectTreeNode(updatedDBTable);
            //---------------Assert Precondition----------------
            Assert.IsTrue(node.Checked);
            Assert.IsTrue(updatedDBTable.PendingChangeApproved);
            //---------------Execute Test ----------------------
            node.Checked = false;
            //---------------Test Result -----------------------
            Assert.IsFalse(node.Checked);
            Assert.IsFalse(updatedDBTable.PendingChangeApproved);
        }

        [Test]
        public void Test_SetupNode_ForRelationshipNode_WhenChildNodeApprovedIsTrue_ShouldStillBeFalse()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update, Approved = true };
            dbDatabase.Tables.Add(updatedDBTable);

            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController controller = databaseInfoChangesControl.TreeViewController;
            ITreeView treeView = controller.TreeView;
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            databaseInfoChangesControl.Database = dbDatabase;
            //---------------Test Result -----------------------
            ITreeNode tableNode = controller.GetBusinessObjectTreeNode(updatedDBTable);
            ITreeNode tablesRelationshipNode = tableNode.Parent;
            Assert.AreEqual("Tables", tablesRelationshipNode.Text, "(Precondition)Should be the correct node");
            Assert.IsFalse(tablesRelationshipNode.Checked);
        }

        [Test]
        public void Test_RelationshipNode_WhenCheckChanged_ToTrue_ShouldDoNothing()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update, Approved = false };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController controller = databaseInfoChangesControl.TreeViewController;
            ITreeView treeView = controller.TreeView;
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeNode tableNode = controller.GetBusinessObjectTreeNode(updatedDBTable);
            ITreeNode tablesRelationshipNode = tableNode.Parent;
            //---------------Assert Precondition----------------
            Assert.AreEqual("Tables", tablesRelationshipNode.Text);
            Assert.IsFalse(tablesRelationshipNode.Checked);
            Assert.IsFalse(updatedDBTable.PendingChangeApproved);
            //---------------Execute Test ----------------------
            tablesRelationshipNode.Checked = true;
            //---------------Test Result -----------------------
            Assert.IsFalse(tablesRelationshipNode.Checked);
            Assert.IsFalse(updatedDBTable.PendingChangeApproved);
        }

        [Test]
        public void Test_RelationshipNode_WhenCheckChanged_ToFalse_ShouldDoNothing()
        {
            //---------------Set up test pack-------------------
            DBDatabase dbDatabase = new DBDatabase();
            DBTable updatedDBTable = new DBTable("UpdatedTable") { PendingChangeType = ChangeType.Update, Approved = true };
            dbDatabase.Tables.Add(updatedDBTable);
            DatabaseInfoChangesControl databaseInfoChangesControl = new DatabaseInfoChangesControl();
            TreeViewController controller = databaseInfoChangesControl.TreeViewController;
            ITreeView treeView = controller.TreeView;
            databaseInfoChangesControl.Database = dbDatabase;
            ITreeNode tableNode = controller.GetBusinessObjectTreeNode(updatedDBTable);
            ITreeNode tablesRelationshipNode = tableNode.Parent;
            //---------------Assert Precondition----------------
            Assert.AreEqual("Tables", tablesRelationshipNode.Text);
            Assert.IsFalse(tablesRelationshipNode.Checked);
            Assert.IsTrue(updatedDBTable.PendingChangeApproved);
            //---------------Execute Test ----------------------
            tablesRelationshipNode.Checked = false;
            //---------------Test Result -----------------------
            Assert.IsFalse(tablesRelationshipNode.Checked);
            Assert.IsTrue(updatedDBTable.PendingChangeApproved);
        }

    }
}
