using System;
using System.Collections.Generic;
using System.Data;
using System.Text;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.BO;
using Habanero.DB;
using Habanero.Faces.Base;
using Habanero.Faces.Win;
using Habanero.BO.ClassDefinition;
using UIForm=Habanero.BO.ClassDefinition.UIForm;

namespace Firestarter.DB.Schema.UI
{
    public class ChooseDatabaseConnectionWizardStep : WizardStepWin
    {
        private IPanelInfo _databaseConnectionInfoPanelInfo;
        private ReverseEngineering _reverseEngineering;
        private SingleRelationship<DatabaseConnectionInfo> _databaseConnectionInfoRelationship;
        private IGroupBox _databaseConnectionGroupBox;

        public const string SelectConnectionMessage = "Please select a database connection.";
        
        public ChooseDatabaseConnectionWizardStep()
        {
            SetupDatabaseConnectionControl();
            LayoutControls();
        }

        private void LayoutControls()
        {
            IControlFactory factory = GlobalUIRegistry.ControlFactory;
            BorderLayoutManager layoutManager = factory.CreateBorderLayoutManager(this);
            layoutManager.AddControl(_databaseConnectionGroupBox, BorderLayoutManager.Position.Centre);
        }

        private void SetupDatabaseConnectionControl()
        {
            IControlFactory factory = GlobalUIRegistry.ControlFactory;
            PanelBuilder panelBuilder = new PanelBuilder(factory);
            IPanel databaseConnectionPanel = SetupDatabaseConnectionPanel(panelBuilder);
            _databaseConnectionGroupBox = factory.CreateGroupBox("Database Connection");
            BorderLayoutManager groupBoxLayoutManager = factory.CreateBorderLayoutManager(_databaseConnectionGroupBox);
            groupBoxLayoutManager.AddControl(databaseConnectionPanel, BorderLayoutManager.Position.Centre);
        }

        private IPanel SetupDatabaseConnectionPanel(PanelBuilder panelBuilder)
        {
            _databaseConnectionInfoPanelInfo = panelBuilder.BuildPanelForForm(GetDatabaseConnectionInfoUIForm());
            _databaseConnectionInfoPanelInfo.BusinessObject = new DatabaseConnectionInfo();
            IPanel panel = _databaseConnectionInfoPanelInfo.Panel;
            //PanelInfo.Field fieldInfo = _databaseConnectionInfoPanelInfo.FieldInfos[_databaseConnectionInfoPanelInfo.FieldInfos.Count - 1];
            //panel.Height = fieldInfo.InputControl.Bottom + 5;
            return panel;
        }

        private static IUIForm GetDatabaseConnectionInfoUIForm()
        {
            return ClassDef.Get<DatabaseConnectionInfo>().GetUIDef("default").UIForm;
        }

        public IPanelInfo DatabaseConnectionInfoPanelInfo
        {
            get { return _databaseConnectionInfoPanelInfo; }
        }

        public DatabaseConnectionInfo DatabaseConnectionInfo
        {
            get { return (DatabaseConnectionInfo)_databaseConnectionInfoPanelInfo.BusinessObject; }
        }

        public ReverseEngineering ReverseEngineering
        {
            get { return _reverseEngineering; }
            set
            {
                if (_databaseConnectionInfoRelationship != null)
                {
                    _databaseConnectionInfoRelationship.Updated -= DatabaseConnectionInfo_Changed;
                }
                _reverseEngineering = value;
                if (_reverseEngineering != null)
                {
                    _databaseConnectionInfoRelationship = _reverseEngineering
                        .Relationships.GetSingle<DatabaseConnectionInfo>("DatabaseConnectionInfo");
                    _databaseConnectionInfoRelationship.Updated += DatabaseConnectionInfo_Changed;
                }
                UpdateDatabaseConnectionInfo();
            }
        }

        private void DatabaseConnectionInfo_Changed(object sender, BOEventArgs<DatabaseConnectionInfo> e)
        {
            UpdateDatabaseConnectionInfo();
        }

        private void UpdateDatabaseConnectionInfo()
        {
            DatabaseConnectionInfo databaseConnectionInfo = null;
            if (_reverseEngineering != null)
                databaseConnectionInfo = _reverseEngineering.DatabaseConnectionInfo;
            _databaseConnectionInfoPanelInfo.BusinessObject = databaseConnectionInfo;
        }

        #region Implementation of IWizardStep

        /// <summary>
        ///             Initialises the step. Run when the step is reached.
        /// </summary>
        public override void InitialiseStep()
        {
            
        }

        /// <summary>
        ///             Verifies whether this step can be passed.
        /// </summary>
        /// <param name="message">Error message should moving on be disallowed. This message will be displayed to the user by the WizardControl.</param>
        public override bool CanMoveOn(out string message)
        {
            return IsDatabaseConnectionValid(out message);
        }

        private bool IsDatabaseConnectionValid(out string message)
        {
            if (DatabaseConnectionInfo == null)
            {
                message = SelectConnectionMessage;
                return false;
            }
            return DatabaseConnectionInfo.IsDatabaseConnectionValid(out message);
        }

        /// <summary>
        ///             Verifies whether the user can move back from this step.
        /// </summary>
        public override bool CanMoveBack()
        {
            throw new System.NotImplementedException(); //TODO Mark 21 Jan 2009: Implement this
        }

        /// <summary>
        ///             Provides an interface for the developer to implement functionality to cancel any edits made as part of this
        ///             wizard step. The default wizard controller functionality is to call all wizard steps cancelStep methods when
        ///             its Cancel method is called.
        /// </summary>
        public override void CancelStep()
        {
            if (_reverseEngineering == null) return;
            if (_reverseEngineering.DatabaseConnectionInfo == null) return;
            _reverseEngineering.DatabaseConnectionInfo.CancelEdits();
        }

        /// <summary>
        ///             The text that you want displayed at the top of the wizard control when this step is active.
        /// </summary>
        public override string HeaderText
        {
            get { return "Select a valid database connection."; }
        }

        #endregion
    }
}
