using System;
using Firestarter.DB.Schema.BO;
using Firestarter.DB.Schema.Logic;
using Firestarter.DB_DomainModel.Logic;
using FireStarterModeller.BO;
using FireStarterModeller.Test;
using FireStarterModeller.Test.BO;
using Habanero.Base;
using NUnit.Framework;
 
using Rhino.Mocks;

namespace Firestarter.DB_DomainModel.Logic.Test
{
    //TODO: need to modify so that has relationship to the 
    // database field ID.
    // Then need to modify logic for creating setting databaseFieldName 
    // so that either from Prop name if id not set else from ID prop name
    // if prop name not set.
    //TODO Mark 16 Mar 2009: Look at Single Table Inheritance import prop
    [TestFixture]
    public class TestDBToDMPropertyImporter : TestBase
    {

        private static DBToDMPropertyImporter CreateDBToDMPropertyImporter()
        {
            IDBToDMPropertyUpdater dbToDMPropertyUpdater = MockRepository.GenerateStub<IDBToDMPropertyUpdater>();
            return new DBToDMPropertyImporter(dbToDMPropertyUpdater);
        }

        [Test]
        public void Test_Construct()
        {
            //---------------Set up test pack-------------------
            IDBToDMPropertyUpdater dbToDMPropertyUpdater = MockRepository.GenerateStub<IDBToDMPropertyUpdater>();
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            DBToDMPropertyImporter dbToDMPropertyImporter = new DBToDMPropertyImporter(dbToDMPropertyUpdater);
            //---------------Test Result -----------------------
            Assert.IsNotNull(dbToDMPropertyImporter);
            Assert.AreSame(dbToDMPropertyUpdater, dbToDMPropertyImporter.DBToDMPropertyUpdater);
        }

        [Test]
        public void Test_Construct_WhenNullDBToDMPropertyUpdater_ShouldThrowError()
        {
            //---------------Set up test pack-------------------
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            try
            {
                DBToDMPropertyImporter dbToDMPropertyImporter = new DBToDMPropertyImporter(null);
                //---------------Test Result -----------------------
                Assert.Fail("Expected to throw an ArgumentNullException");
            }
            catch (ArgumentNullException ex)
            {
                Assert.AreEqual("dbToDMPropertyUpdater", ex.ParamName);
            }
        }

        [Test]
        public void Test_Import_WhenNullDMClass_ShouldThrowError()
        {
            //---------------Set up test pack-------------------
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter();
            //---------------Assert Precondition----------------
            //---------------Execute Test ----------------------
            try
            {
                dbToDMPropertyImporter.Import(null, new DBColumn());
                //---------------Test Result -----------------------
                Assert.Fail("Expected to throw an ArgumentNullException");
            }
            catch (ArgumentNullException ex)
            {
                Assert.AreEqual("dmClass", ex.ParamName);
            }
        }


        [Test]
        public void Test_Import_WhenColumnIsNull_ShouldThrowError()
        {
            //---------------Set up test pack-------------------
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter();
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            try
            {
                dbToDMPropertyImporter.Import(new DMClass(), null);
                Assert.Fail("expected ArgumentNullException");
            }
                //---------------Test Result -----------------------
            catch (ArgumentNullException ex)
            {
                StringAssert.Contains("Value cannot be null", ex.Message);
                StringAssert.Contains("dbColumn", ex.ParamName);
            }
        }

        [Test]
        public void Test_Import_WhenPropertyDoesNotExist_ShouldAddNewDMProperty()
        {
            //---------------Set up test pack-------------------
            DBColumn dbColumn = new DBColumn("FieldName");
            DMClass dmClass = new DMClass();
            DMProperty otherProperty = new DMProperty();
            dmClass.Properties.Add(otherProperty);
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter(); 
            //---------------Assert Precondition----------------
            Assert.AreEqual(1, dmClass.Properties.Count);
            //---------------Execute Test ----------------------
            DMProperty returnedDMProperty = dbToDMPropertyImporter.Import(dmClass, dbColumn);
            //---------------Test Result -----------------------
            Assert.AreEqual(2, dmClass.Properties.Count);
            Assert.AreNotSame(otherProperty, returnedDMProperty);
            Assert.IsTrue(dmClass.Properties.Contains(returnedDMProperty));
            Assert.AreEqual(dbColumn.ColumnName, returnedDMProperty.MappedDBColumnName);
            Assert.AreEqual(dbColumn.ColumnID, returnedDMProperty.MappedDBColumnID);
        }

        [Test]
        public void Test_Import_WhenPropertyDoesNotExist_ShouldCallDBToDMPropertyUpdater()
        {
            //---------------Set up test pack-------------------
            DBColumn dbColumn = new DBColumn("FieldName");
            DMClass dmClass = new DMClass();
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter();
            //---------------Assert Precondition----------------
            Assert.IsTrue(dbColumn.Nullable.Value);
            //---------------Execute Test ----------------------
            DMProperty boProp = dbToDMPropertyImporter.Import(dmClass, dbColumn);
            //---------------Test Result -----------------------
            dbToDMPropertyImporter.DBToDMPropertyUpdater.AssertWasCalled(updater => updater.UpdateValues(dbColumn, boProp));
        }

        [Test]
        public void Test_Import_WhenPropertyExists_WithOnlyMappedDBColumnSet_ShouldCallDBToDMPropertyUpdater()
        {
            //---------------Set up test pack-------------------
            DBColumn dbColumn = new DBColumn("FieldName");
            DMClass dmClass = new DMClass();
            DMProperty existingDMProperty = new DMProperty { MappedDBColumn = dbColumn };
            dmClass.Properties.Add(existingDMProperty);
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter();
            //---------------Assert Precondition----------------
            Assert.IsTrue(dbColumn.Nullable.Value);
            //---------------Execute Test ----------------------
            DMProperty returnedDMProperty = dbToDMPropertyImporter.Import(dmClass, dbColumn);
            //---------------Test Result -----------------------
            Assert.AreSame(existingDMProperty, returnedDMProperty);
            dbToDMPropertyImporter.DBToDMPropertyUpdater.AssertWasCalled(updater => updater.UpdateValues(dbColumn, existingDMProperty));
        }

        [Test]
        public void Test_Import_WhenPropertyExists_WithOnlyMappedDBColumnNameSet_ShouldCallDBToDMPropertyUpdater()
        {
            //---------------Set up test pack-------------------
            DBColumn dbColumn = new DBColumn("FieldName");
            DMClass dmClass = new DMClass();
            DMProperty existingDMProperty = new DMProperty { MappedDBColumnName = dbColumn.ColumnName };
            dmClass.Properties.Add(existingDMProperty);
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter();
            //---------------Assert Precondition----------------
            Assert.IsTrue(dbColumn.Nullable.Value);
            //---------------Execute Test ----------------------
            DMProperty returnedDMProperty = dbToDMPropertyImporter.Import(dmClass, dbColumn);
            //---------------Test Result -----------------------
            Assert.AreSame(existingDMProperty, returnedDMProperty);
            dbToDMPropertyImporter.DBToDMPropertyUpdater.AssertWasCalled(updater => updater.UpdateValues(dbColumn, existingDMProperty));
        }
        
        [Test]
        public void Test_Import_WhenPropertyExists_WithOnlyMappedDBColumnSet_ShouldUpdateMappedDBColumnName()
        {
            //---------------Set up test pack-------------------
            DBColumn dbColumn = new DBColumn("FieldName");
            DMClass dmClass = new DMClass();
            DMProperty existingDMProperty = new DMProperty { MappedDBColumnID = dbColumn.ColumnID };
            dmClass.Properties.Add(existingDMProperty);
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter();
            //---------------Assert Precondition----------------
            Assert.AreEqual(dbColumn.ColumnID, existingDMProperty.MappedDBColumnID);
            Assert.IsNull(existingDMProperty.MappedDBColumnName);
            //---------------Execute Test ----------------------
            DMProperty returnedDMProperty = dbToDMPropertyImporter.Import(dmClass, dbColumn);
            //---------------Test Result -----------------------
            Assert.AreSame(existingDMProperty, returnedDMProperty);
            Assert.AreEqual(dbColumn.ColumnID, existingDMProperty.MappedDBColumnID);
            Assert.AreEqual(dbColumn.ColumnName, existingDMProperty.MappedDBColumnName);
        }

        [Test]
        public void Test_Import_WhenPropertyExists_WithOnlyMappedDBColumnNameSet_ShouldUpdateMappedDBColumnID()
        {
            //---------------Set up test pack-------------------
            DBColumn dbColumn = new DBColumn("FieldName");
            DMClass dmClass = new DMClass();
            DMProperty existingDMProperty = new DMProperty { MappedDBColumnName = dbColumn.ColumnName };
            dmClass.Properties.Add(existingDMProperty);
            DBToDMPropertyImporter dbToDMPropertyImporter = CreateDBToDMPropertyImporter();
            //---------------Assert Precondition----------------
            Assert.AreEqual(dbColumn.ColumnName, existingDMProperty.MappedDBColumnName);
            Assert.IsNull(existingDMProperty.MappedDBColumnID);
            //---------------Execute Test ----------------------
            DMProperty returnedDMProperty = dbToDMPropertyImporter.Import(dmClass, dbColumn);
            //---------------Test Result -----------------------
            Assert.AreSame(existingDMProperty, returnedDMProperty);
            Assert.AreEqual(dbColumn.ColumnName, existingDMProperty.MappedDBColumnName);
            Assert.AreEqual(dbColumn.ColumnID, existingDMProperty.MappedDBColumnID);
        }

    }
}