using System;
using System.Globalization;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.Base.Exceptions;

namespace Firestarter.DB_DomainModel.Logic
{
    public class DBToDMClassImporter : IDBToDMClassImporter
    {
        /// <summary>
        /// Returns the table prefix to be used for this transformation this prefix may be 
        ///  a prefix set up for this particular database, for this project, a global settings for all
        ///  projects for this user or a global setting for all users of an organisation. 
        /// </summary>
        /// <returns></returns>
        public virtual string TableNamePrefix { get; set; }

        /// <summary>
        /// Returns the primary key suffix to be used for this transformation this suffix may be 
        ///  a suffix set up for this particular database, for this project, a global settings for all
        ///  projects for this user or a global setting for all users of an organisation. 
        /// </summary>
        /// <returns></returns>
        public virtual string PrimaryKeyColumnSuffix { get; set; }

        public DMClass Import(DMSolution dmSolution, DBTable dbTable)
        {
            if (dmSolution == null) throw new ArgumentNullException("dmSolution");
            if (dmSolution.DefaultAssembly == null)
            {
                const string message =
                    "You cannot create a DM Importer if the Solution does not have a default assembly";
                throw new HabaneroDeveloperException(message, message);
            }
            if (dbTable == null) throw new ArgumentNullException("dbTable");
            DMClass mappedClass = dbTable.GetMappedDMClass();
            if (mappedClass != null)
            {
                if (String.IsNullOrEmpty(mappedClass.ClassDescription)) mappedClass.ClassDescription = dbTable.TableComment;
                mappedClass.ClassNameBO = dbTable.GetClassName(TableNamePrefix, PrimaryKeyColumnSuffix);
                mappedClass.MappedTableName = dbTable.TableNameDB;
                return mappedClass;
            }
            DMClass foundDBClass = dmSolution.FindClassByMappedTableName(dbTable.TableNameDB);
            if (foundDBClass != null && foundDBClass.MappedTableID == null)
            {
                foundDBClass.MappedTable = dbTable;
                if (String.IsNullOrEmpty(foundDBClass.ClassDescription))
                {
                    foundDBClass.ClassDescription = dbTable.TableComment;
                }
                return foundDBClass;
            }

            return new DMClass
                       {
                           MappedTableID = dbTable.TableID,
                           MappedTableName = dbTable.TableNameDB,
                           ClassNameBO =  dbTable.GetClassName(TableNamePrefix, PrimaryKeyColumnSuffix),
                           Assembly = dmSolution.DefaultAssembly,
                           ClassDescription = dbTable.TableComment
                       };
        }
    }
}