using System;
using FireStarter.Base;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.Util;

namespace Firestarter.DB_DomainModel.Logic
{
    public class DBToDMPropertyUpdater : IDBToDMPropertyUpdater
    {
        public IDBTypeToDMPropertyTypeMapper DBTypeToDMPropertyTypeMapper { get; private set; }

        public DBToDMPropertyUpdater(IDBTypeToDMPropertyTypeMapper typeMapper)
        {
            if (typeMapper == null) throw new ArgumentNullException("typeMapper");
            DBTypeToDMPropertyTypeMapper = typeMapper;
        }

        public void UpdateValues(DBColumn fromDBColumn, DMProperty toDMProperty)
        {
            toDMProperty.PropertyName = PascalCaseFieldName(fromDBColumn.ColumnName.Trim());
            toDMProperty.PropertyType = DBTypeToDMPropertyTypeMapper.GetDMPropertyType(fromDBColumn.DataType, fromDBColumn.DataLength);
            toDMProperty.Description = fromDBColumn.Comment;
            toDMProperty.Compulsory = !fromDBColumn.Nullable.GetValueOrDefault(true);
            toDMProperty.AutoIncrementing = fromDBColumn.IsAutoIncrementing.GetValueOrDefault(false);
            toDMProperty.KeepValuePrivate = NameContainsPassword(fromDBColumn);
            if (toDMProperty.OrdinalPosition == null) toDMProperty.OrdinalPosition = fromDBColumn.OrdinalPosition;

            DMPropertyType propertyType = toDMProperty.PropertyType;
            if (propertyType != null && propertyType.PropertyTypeFullName == "System.String")
            {
                DMPropRuleParameter maxLengthParameter =
                    toDMProperty.DefaultPropRule.PropRuleParameters.Find(parameter => parameter.KeyName == "maxLength");
                maxLengthParameter.Value = fromDBColumn.DataLength.ToString();
            }

        }

        private static bool NameContainsPassword(DBColumn dbColumn)
        {
            return dbColumn.ColumnName.ToUpper().Contains("PASSWORD");
        }

        private static string PascalCaseFieldName(string fieldNameConverted)
        {
            return StringUtilities.PascalCaseTableName(fieldNameConverted);
        }
    }
}