using System;
using System.Collections.Generic;
using Firestarter.DB.Schema.BO;
using FireStarterModeller.BO;
using Habanero.Base.Exceptions;

namespace Firestarter.DB_DomainModel.Logic
{
    public class DBToDMUniqueConstraintImporter : IDBToDMUniqueConstraintImporter
    {
        public DMUniqueConstraint Import(DMClass dmClass, DBKey dbKey, out IList<string> errorMessages)
        {
            errorMessages = new List<string>();
            if (dmClass == null) throw new ArgumentNullException("dmClass");
            if (dbKey == null) throw new ArgumentNullException("dbKey");
            if (dmClass == null)
                throw new HabaneroArgumentException
                    ("_dmClass", "You cannot import the primary key since the business object is null");
            DMUniqueConstraint dmUniqueConstraint = GetMappedUniqueConstraint(dmClass, dbKey);
            dmUniqueConstraint.UniqueConstraintName = dbKey.KeyName;

            foreach (DMUniqueConstraintProperty uniqueConstraintProperty in dmUniqueConstraint.UniqueConstraintProperties.Clone())
            {
                DMProperty dmProperty = uniqueConstraintProperty.Property;
                if (!dbKey.KeyColumns.Exists(column =>
                    column.Column.ColumnName == dmProperty.MappedDBColumnName))
                {
                    dmUniqueConstraint.UniqueConstraintProperties.Remove(uniqueConstraintProperty);
                }
            }


            foreach (DBColumn columnInfo in dbKey.Columns)
            {
                DMProperty property = dmClass.FindBOPropertyByColumnId(columnInfo.ColumnID);
                if (property == null)
                {
                    string expectedMessage = string.Format("The Unique Constraint '{0}' for business object '{1}' could not be "
                                                           + "created since the column '{2}' do not exist on the business object",
                                                           dbKey.KeyName, dmClass.ClassNameBO, columnInfo.ColumnName);

                    errorMessages.Add(expectedMessage);
                    return null;
                }
                dmUniqueConstraint.AddPropertyInfo(property);
            }
            
            return dmUniqueConstraint;

        }

        private DMUniqueConstraint GetMappedUniqueConstraint(DMClass dmClass, DBKey dbKey)
        {
            DMUniqueConstraint uniqueConstraint = dbKey.GetMappedDMUniqueConstraint();
            //if (uniqueConstraint != null && uniqueConstraint.Class != _dmClass)
            //{
            //    uniqueConstraint.MappedDBKey = null;// objectIdentity.Class.MappedTable.PrimaryKeyDB;
            //    uniqueConstraint = null;
            //}
            if (uniqueConstraint == null)
            {
                uniqueConstraint = new DMUniqueConstraint {
                    Class = dmClass,
                    MappedDBKey = dbKey
                };
            }
            return uniqueConstraint;
        }

        //private DMObjectIdentity GetMappedObjectIdentity(DBPrimaryKey dbPrimaryKey)
        //{
        //    DMObjectIdentity objectIdentity = dbPrimaryKey.GetMappedDMObjectIdentity();
        //    if (objectIdentity != null && objectIdentity.Class != _dmClass)
        //    {
        //        objectIdentity.MappedDBPrimaryKey = null;// objectIdentity.Class.MappedTable.PrimaryKeyDB;
        //        objectIdentity = null;
        //    }
        //    if (objectIdentity == null)
        //    {
        //        objectIdentity = _dmClass.ObjectIdentity ?? new DMObjectIdentity { Class = _dmClass };
        //    }
        //    objectIdentity.MappedDBPrimaryKey = dbPrimaryKey;
        //    return objectIdentity;
        //}
    }
}