using Firestarter.Test.Base;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.BO;
using NUnit.Framework;

namespace FireStarterModeller.Test.BO
{
    [TestFixture]
    public class TestDMPropertyNameNotSameAsClassNameRule
    {
        [SetUp]
        public void SetupTest()
        {
            //Runs every time that any testmethod is executed
            TestUtilsDomain.SetupTest();
            BORegistry.DataAccessor = new DataAccessorInMemory();
        }

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            //Code that is executed before any test is run in this class. If multiple tests
            // are executed then it will still only be called once.
            TestUtilsDomain.SetupFixture();
        }

        [TearDown]
        public void TearDownTest()
        {
            //runs every time any testmethod is complete
        }

        #region TestDMPropertyNameNotSameAsClassName

        [Test]
        public void Test_CreateDMPropertyNotSameAsClassName()
        {
            //---------------Set up test pack-------------------
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IBusinessObjectRule rule = new DMPropertyNameNotSameAsClassName();
            //---------------Test Result -----------------------
            Assert.IsNotNull(rule);
            Assert.AreEqual("DMPropertyNameNotSameAsClassName", rule.Name);
            Assert.AreEqual(ErrorLevel.Error, rule.ErrorLevel);
        }

//        [Test]
//        public void Test_CreateDMPropertyNotSameAsClassName_WhenPropertyNull_ShouldRaiseError()
//        {
//            //---------------Set up test pack-------------------
//            //---------------Assert Precondition----------------
//            //---------------Execute Test ----------------------
//            try
//            {
//                new DMPropertyNameNotSameAsClassName(null);
//                Assert.Fail("expected ArgumentNullException");
//            }
//            //---------------Test Result -----------------------
//            catch (ArgumentNullException ex)
//            {
//                StringAssert.Contains("Value cannot be null", ex.Message);
//                StringAssert.Contains("dmProperty", ex.ParamName);
//            }
//        }

        [Test]
        public void Test_DMPropertyNotSameAsClassName_FailsWhenPropertyNameIsSameAsClassName()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = TestUtilsShared.CreateUnsavedValidDMProperty();
            dmProperty.PropertyName = dmProperty.Class.ClassNameBO;
            IBusinessObjectRule rule = new DMPropertyNameNotSameAsClassName();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dmProperty.Class);
            Assert.AreEqual(dmProperty.Class.ClassNameBO, dmProperty.PropertyName);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmProperty);
            //---------------Test Result -----------------------
            Assert.IsFalse(isValid);
            StringAssert.Contains("The Property '", rule.Message);
            StringAssert.Contains("'" + dmProperty + "'", rule.Message);
            StringAssert.Contains("' cannot have the same name as its Class '" + dmProperty.Class + "'.", rule.Message);
        }

        [Test]
        public void Test_DMPropertyNotSameAsClassName_PassesWhenPropertyHasNoClass()
        {
            //---------------Set up test pack-------------------
            DMProperty dmProperty = TestUtilsDMProperty.CreateUnsavedDefaultDMProperty();
            IBusinessObjectRule rule = new DMPropertyNameNotSameAsClassName();
            //---------------Assert Precondition----------------
            Assert.IsNull(dmProperty.Class);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmProperty);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }

        [Test]
        public void Test_DMPropertyNotSameAsClassName_PassesWhenPropertyNameIsNotSameAsClassName()
        {
            //---------------Set up test pack-------------------
            DMProperty  dmProperty = TestUtilsShared.CreateUnsavedValidDMProperty(); 
            dmProperty.PropertyName = "DFHGFHGF";
            IBusinessObjectRule rule = new DMPropertyNameNotSameAsClassName();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(dmProperty.Class);
            Assert.AreNotEqual(dmProperty.Class.ClassNameBO, dmProperty.PropertyName);
            //---------------Execute Test ----------------------
            bool isValid = rule.IsValid(dmProperty);
            //---------------Test Result -----------------------
            Assert.IsTrue(isValid);
        }
        #endregion
    }
}