using System;
using System.Collections.Generic;
using FireStarter.Base;
using FireStarterModeller.BO;
using Habanero.Base;
using Habanero.BO;
using Habanero.BO.ClassDefinition;
using Habanero.BO.Loaders;
using Habanero.DB;
using NUnit.Framework;

namespace FireStarterModeller.Test.BO
{

    [TestFixture]
    public class TestTempCreateDB
    {

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            //------------------------------------------------------------
            // Use this list to ignore generated tests that are failing
            // due to a unique condition in your application.
            // Remember to reimplement the test here.
            //------------------------------------------------------------
            //_ignoreList.Add("TestMethodName", "Reason for ignoring it");
            //Runs every time that any testmethod is executed
            ClassDef.ClassDefs.Clear();
            ClassDef.ClassDefs.Add(new XmlClassDefsLoader(BOBroker.GetClassDefsXml(), new DtdLoader()).LoadClassDefs());

        }

        [TearDown]
        public  void TearDownTest()
        {

        }

        [Test]
        [Ignore("This should be run on demand")]
        public void Test_TempTestToCreateDatabase()
        {
            //---------------Set up test pack-------------------
            ClassDefCol classDefCol = ClassDef.ClassDefs;
            //DatabaseConfig databaseConfig = new DatabaseConfig("Mysql", "localhost", "firestarter_temp", "root", "root",
            //                                                   "3306");
            DatabaseConfig databaseConfig = new DatabaseConfig(DatabaseConfig.SqlServer, "localhost", "firestarter_temp", "sa", "sa", "1000");
            IDatabaseConnection databaseConnection = databaseConfig.GetDatabaseConnection();
            SetupDatabase setupDatabase = new SetupDatabase(databaseConnection);
//           setupDatabase.ClearDatabase();
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            
            setupDatabase.CreateDatabase(classDefCol);
            //---------------Test Result -----------------------
        }

    }
    public class SetupDatabase
    {
        private readonly IDatabaseConnection _databaseConnection;

        public SetupDatabase(IDatabaseConnection databaseConnection)
        {
            _databaseConnection = databaseConnection;
        }

        public void ClearDatabase()
        {
            string database = _databaseConnection.GetConnection().Database;
            _databaseConnection.ExecuteRawSql(String.Format("DROP DATABASE IF EXISTS {0};CREATE DATABASE {0};", database));
        }

        public void CreateDatabase(ClassDefCol classDefCol)
        {
            List<string> statements = new List<string>();
            CreateTablesOnly(classDefCol, statements);
//            CreateRelationshipsOnly(classDefCol, statements);
            Apply(statements);
        }

        private void CreateRelationshipsOnly(ClassDefCol classDefCol, List<string> statements)
        {
            foreach (ClassDef classDef in classDefCol)
            {
                CreateRelationships(classDefCol, classDef, statements);
            }
        }

        private void CreateTablesOnly(ClassDefCol classDefCol, List<string> statements)
        {
            foreach (ClassDef classDef in classDefCol)
            {
                CreateTable(classDef, statements);
            }
//            AddSettingTableIfNeeded(classDefCol, statements);
        }

        private void AddSettingTableIfNeeded(ClassDefCol classDefCol, List<string> statements)
        {
            foreach (ClassDef classDef in classDefCol)
            {
                if (classDef.TableName.ToUpper() == "SETTING")
                {
                    return;
                }
            }
            const string sql = @"
                CREATE TABLE `setting` (
                  `SettingName` varchar(50) NOT NULL default '',
                  `SettingValue` varchar(1000) NOT NULL default '',
                  `StartDate` datetime default NULL,
                  `EndDate` datetime default NULL,
                  PRIMARY KEY  (`SettingName`)
                )";
            statements.Add(sql);
        }

        private void Apply(List<string> statements)
        {
            SqlStatementCollection sqlStatementCollection = new SqlStatementCollection();
            foreach (string statement in statements)
            {
                sqlStatementCollection.Add(new SqlStatement(_databaseConnection, statement));
            }
            _databaseConnection.ExecuteSql(sqlStatementCollection);
        }

        private void CreateTable(ClassDef classDef, List<string> statements)
        {
            //CREATE TABLE `setting` (
            //`SettingName` varchar(50) NOT NULL default '',
            //`SettingValue` varchar(1000) NOT NULL default '',
            //`StartDate` datetime default NULL,
            //`EndDate` datetime default NULL,
            //PRIMARY KEY  (`SettingName`)
            //)

            if (classDef.SuperClassDef != null && classDef.SuperClassDef.ORMapping == ORMapping.SingleTableInheritance)
            {
                return;
            }
            string sqlStatement = "CREATE TABLE ";
            sqlStatement += SqlFormattingHelper.FormatTableName(classDef.TableName, _databaseConnection);
            sqlStatement += " (";
            foreach (PropDef propDef in classDef.PropDefcol)
            {
                sqlStatement += SqlFormattingHelper.FormatFieldName(propDef.DatabaseFieldName, _databaseConnection);
                sqlStatement += " " + GetFieldType(propDef);
                if (propDef.Compulsory)
                {
                    sqlStatement += " NOT NULL";
                }
                sqlStatement += ", ";
            }
            sqlStatement += "PRIMARY KEY (";
            List<string> fields = new List<string>();
            foreach (PropDef propDef in classDef.PrimaryKeyDef)
            {
                fields.Add(SqlFormattingHelper.FormatFieldName(propDef.DatabaseFieldName, _databaseConnection));
            }
            sqlStatement += String.Join(", ", fields.ToArray());
            sqlStatement += "));";
            statements.Add(sqlStatement);
        }

        private static string GetFieldType(PropDef propDef)
        {
            string fieldType = "varchar(50)";
            switch (propDef.PropertyTypeName.ToUpper())
            {
                case "DATETIME":
                    fieldType = "datetime";
                    break;
                case "STRING":
                    int length = 50;
                    PropRuleString propRule = null;
                    if (propDef.PropRules.Count > 0)
                    {
                        propRule = propDef.PropRules[0] as PropRuleString;
                    }
                    if (propRule != null)
                    {
                        length = propRule.MaxLength;
                    }
                    if (length <= 0) length = 50;
                    fieldType = "varchar(" + length + ")";
                    break;
                case "GUID":
                    fieldType = "char(38)";
                    break;
                case "IMAGE":
                    fieldType = "blob";
                    break;
                case "BOOL":
                case "BOOLEAN":
                    fieldType = "bit";// " unsigned";
                    break;
                case "INT":
                case "INTEGER":
                    fieldType = "integer";
                    break;
                case "SINGLE":
                case "DOUBLE":
                    fieldType = "float";
                    break;
            }
            return fieldType.ToUpper();
        }

        private void CreateRelationships(ClassDefCol classDefCol, ClassDef classDef, ICollection<string> statements)
        {
            //ALTER TABLE `invoice` 
            //  ADD CONSTRAINT `Invoice_InvoiceStatus_FK` FOREIGN KEY `Invoice_InvoiceStatus_FK` (`InvoiceStatusID`)
            //    REFERENCES `invoicestatus` (`InvoiceStatusID`)
            //    ON DELETE RESTRICT
            //    ON UPDATE RESTRICT;

            if (classDef.SuperClassDef != null && classDef.SuperClassDef.ORMapping == ORMapping.SingleTableInheritance)
            {
                return;
            }
            string sqlStatement;
            sqlStatement = "ALTER TABLE ";
            sqlStatement += SqlFormattingHelper.FormatTableName(classDef.TableName, _databaseConnection);
            List<string> constraints = new List<string>();
            foreach (RelationshipDef relationshipDef in classDef.RelationshipDefCol)
            {
                if (relationshipDef is SingleRelationshipDef)
                {
                    if (!classDefCol.Contains(relationshipDef.RelatedObjectAssemblyName, relationshipDef.RelatedObjectClassName))
                    {
                        throw new Exception("Related class not found:" + relationshipDef.RelatedObjectAssemblyName + "." + relationshipDef.RelatedObjectClassName);
                    }
                    IClassDef relatedClassDef = classDefCol[relationshipDef.RelatedObjectAssemblyName, relationshipDef.RelatedObjectClassName];

                    string constraintName = SqlFormattingHelper.FormatFieldName(
                        classDef.TableName + "_" + relationshipDef.RelationshipName + "_FK", _databaseConnection);
                    string constraintSql = " ADD CONSTRAINT " + constraintName;
                    constraintSql += " FOREIGN KEY " + constraintName;
                    List<string> props = new List<string>();
                    List<string> relProps = new List<string>();
                    foreach (RelPropDef relPropDef in relationshipDef.RelKeyDef)
                    {
                        string propName = relPropDef.OwnerPropertyName;
                        string relPropName = relPropDef.RelatedClassPropName;
                        IPropDef ownerPropDef = classDef.GetPropDef(propName);
                        if (ownerPropDef != null)
                        {
                            propName = ownerPropDef.DatabaseFieldName;
                        }
                        IPropDef relatedPropDef = relatedClassDef.GetPropDef(relPropName);
                        if (relatedPropDef != null)
                        {
                            relPropName = relatedPropDef.DatabaseFieldName;
                        }
                        props.Add(SqlFormattingHelper.FormatFieldName(propName, _databaseConnection));
                        relProps.Add(SqlFormattingHelper.FormatFieldName(relPropName, _databaseConnection));
                    }
                    constraintSql += " (" + String.Join(",", props.ToArray()) + ")";
                    constraintSql += " REFERENCES ";
                    IClassDef relatedBaseClassDef = relatedClassDef;
                    while (relatedBaseClassDef.SuperClassDef != null && relatedBaseClassDef.SuperClassDef.ORMapping == ORMapping.SingleTableInheritance)
                    {
                        relatedBaseClassDef = (ClassDef)relatedBaseClassDef.SuperClassDef.SuperClassClassDef;
                    }
                    string relatedTableName = relatedBaseClassDef.TableName;
                    constraintSql += SqlFormattingHelper.FormatTableName(relatedTableName, _databaseConnection);
                    constraintSql += " (" + String.Join(",", relProps.ToArray()) + ")";
                    constraintSql += " ON DELETE RESTRICT ";
                    constraintSql += " ON UPDATE RESTRICT";
                    constraints.Add(constraintSql);
                }
            }
            sqlStatement += String.Join(", ", constraints.ToArray());
            sqlStatement += ";";
            if (constraints.Count > 0)
            {
                statements.Add(sqlStatement);
            }
        }
    }
}