using System;
using FireStarter.Base;
using FireStarterModeller.BO;
using Habanero.Base.Exceptions;
using NUnit.Framework;

namespace FireStarterModeller.Test.BO
{
    /// <summary>
    /// Provides a place to write custom tests for UIGridColumnGridColumnControlType objects.
    /// This file is only written once and can be changed.  The Def file
    /// attached to this as a dependent is rewritten with each regeneration
    /// and contains the standard tests for UIGridColumnGridColumnControlType.
    /// Regenerate this test project whenever there have been changes to the
    /// business objects.
    /// If tests are failing due to a unique setup in your application,
    /// you can either override the Create methods in TestUtils, or you
    /// can add the test to the ignore list below and reimplement it here.
    /// </summary>
    public partial class TestUIGridColumnControlType
    {
        [TestFixtureSetUp]
        public void SetupTestFixture()
        {
            //------------------------------------------------------------
            // Use this list to ignore generated tests that are failing
            // due to a unique condition in your application.
            // Remember to reimplement the test here.
            //------------------------------------------------------------
            //_ignoreList.Add("TestMethodName", "Reason for ignoring it");
            base.SetupFixture();
        }


        [Test]
        public void Test_ToString()
        {
            //---------------Set up test pack-------------------
            UIGridColumnControlType uiGridColumnControlType =
                TestUtilsUIGridColumnControlType.CreateUnsavedValidUIGridColumnControlType();
            //---------------Assert Precondition----------------
            Assert.IsNotNull(uiGridColumnControlType.Solution);
            TestUtilsShared.AssertStringNotEmpty
                (uiGridColumnControlType.AssemblyName, "uiGridColumnControlType.AssemblyName");
            TestUtilsShared.AssertStringNotEmpty(uiGridColumnControlType.TypeName, "uiGridColumnControlType.TypeName");
            //---------------Execute Test ----------------------
            string actualToString = uiGridColumnControlType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual
                (uiGridColumnControlType.Solution + " - " + uiGridColumnControlType.AssemblyName + "."
                 + uiGridColumnControlType.TypeName, actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull()
        {
            //---------------Set up test pack-------------------
            UIGridColumnControlType uiGridColumnControlType =
                TestUtilsUIGridColumnControlType.CreateUnsavedValidUIGridColumnControlType();
            uiGridColumnControlType.Solution = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiGridColumnControlType.Solution);
            TestUtilsShared.AssertStringNotEmpty
                (uiGridColumnControlType.AssemblyName, "uiGridColumnControlType.AssemblyName");
            //---------------Execute Test ----------------------
            string actualToString = uiGridColumnControlType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual
                (" - " + uiGridColumnControlType.AssemblyName + "." + uiGridColumnControlType.TypeName, actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull_AssemblyNameNull()
        {
            //---------------Set up test pack-------------------
            UIGridColumnControlType uiGridColumnControlType =
                TestUtilsUIGridColumnControlType.CreateUnsavedValidUIGridColumnControlType();
            uiGridColumnControlType.AssemblyName = null;
            uiGridColumnControlType.Solution = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiGridColumnControlType.Solution);
            TestUtilsShared.AssertStringEmpty
                (uiGridColumnControlType.AssemblyName, "uiGridColumnControlType.AssemblyName");
            //---------------Execute Test ----------------------
            string actualToString = uiGridColumnControlType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(" - ." + uiGridColumnControlType.TypeName, actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull_AssemblyNameNull_TypeNameNull()
        {
            //---------------Set up test pack-------------------
            UIGridColumnControlType uiGridColumnControlType =
                TestUtilsUIGridColumnControlType.CreateUnsavedValidUIGridColumnControlType();
            uiGridColumnControlType.AssemblyName = null;
            uiGridColumnControlType.Solution = null;
            uiGridColumnControlType.TypeName = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiGridColumnControlType.Solution);
            TestUtilsShared.AssertStringEmpty
                (uiGridColumnControlType.AssemblyName, "uiGridColumnControlType.AssemblyName");
            TestUtilsShared.AssertStringEmpty(uiGridColumnControlType.TypeName, "uiGridColumnControlType.TypeName");
            //---------------Execute Test ----------------------
            string actualToString = uiGridColumnControlType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual(uiGridColumnControlType.UIGridColumnControlTypeID.ToString(), actualToString);
        }

        [Test]
        public void Test_ToString_SolutionNull_AssemblyNameNull_UIGridColumnControlTypeIDNull()
        {
            //---------------Set up test pack-------------------
            UIGridColumnControlType uiGridColumnControlType =
                TestUtilsUIGridColumnControlType.CreateUnsavedValidUIGridColumnControlType();
            uiGridColumnControlType.Solution = null;
            uiGridColumnControlType.AssemblyName = null;
            uiGridColumnControlType.TypeName = null;
            uiGridColumnControlType.UIGridColumnControlTypeID = null;
            //---------------Assert Precondition----------------
            Assert.IsNull(uiGridColumnControlType.Solution);
            Assert.IsNull(uiGridColumnControlType.AssemblyName);
            Assert.IsNull(uiGridColumnControlType.UIGridColumnControlTypeID);
            //---------------Execute Test ----------------------
            string actualToString = uiGridColumnControlType.ToString();
            //---------------Test Result -----------------------
            Assert.AreEqual("-unnamed UIGridColumnControlType-", actualToString);
        }
        [Test]
        public void Test_GetParent()
        {
            //---------------Set up test pack-------------------
            DMSolution dmSolution = new DMSolution();
            UIGridColumnControlType uiGridColumnControlType = new UIGridColumnControlType();
            dmSolution.UIGridColumnControlTypes.Add(uiGridColumnControlType);
            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            IModelElement parent = ((IModelElement)uiGridColumnControlType).Parent;
            //---------------Test Result -----------------------
            Assert.AreSame(dmSolution, parent);
        }

        [Test]
        public void Test_SetParentExceptionThrown()
        {
            //---------------Set up test pack-------------------
            IModelElement prop = new UIGridColumnControlType();

            //---------------Assert Precondition----------------

            //---------------Execute Test ----------------------
            Exception exceptionThrown = null;
            try
            {
                prop.Parent = new DMSolution();
            }
            catch (Exception ex)
            {
                exceptionThrown = ex;
            }
            //---------------Test Result -----------------------
            Assert.IsNotNull(exceptionThrown);
            Assert.IsInstanceOf(typeof(HabaneroDeveloperException), exceptionThrown);
            StringAssert.Contains
                ("Set the parent of the UIGridColumnControlType by setting the related Solution object",
                 exceptionThrown.Message);
        }
    }
}