using FireStarterModeller.BO;
using Habanero.BO;
using NUnit.Framework;

namespace FireStarterModeller.Test.BO
{
    [TestFixture]
    public class TestUITabInfo : TestBase
    {
        [SetUp]
        public override void SetupTest()
        {
            //Runs every time that any testmethod is executed
            base.SetupTest();
        }

        [TestFixtureSetUp]
        public void TestFixtureSetup()
        {
            //Code that is executed before any test is run in this class. If multiple tests
            // are executed then it will still only be called once.
        }

        [TearDown]
        public override void TearDownTest()
        {
            //runs every time any testmethod is complete
            base.TearDownTest();
        }


        [Test]
        public void Test_InsertNewUITabInfo_Invalid()
        {
            //---------------Set up test pack-------------------
            UITab uiTab = new UITab();

            //---------------Assert Precondition----------------
            Assert.IsTrue(uiTab.Status.IsNew);

            //---------------Execute Test ----------------------
            try
            {
                uiTab.Save();
                Assert.Fail("expected Err");
            }
            //---------------Test Result -----------------------
            catch (BusObjectInAnInvalidStateException ex)
            {
                StringAssert.Contains("Name' is a compulsory field and has no value", ex.Message);
            }
            Assert.IsTrue(uiTab.Status.IsNew);
        }

        [Test]
        public void Test_InsertNewUITabInfo()
        {
            //---------------Set up test pack-------------------
            UITab uiTabInfo = TestUtils.CreateUnsavedValidUITabInfo();

            //---------------Assert Precondition----------------
            Assert.IsTrue(uiTabInfo.Status.IsNew);

            //---------------Execute Test ----------------------
            uiTabInfo.Save();

            //---------------Test Result -----------------------
            Assert.IsFalse(uiTabInfo.Status.IsNew);
        }

        [Test]
        public void Test_DeleteUITabInfo()
        {
            //---------------Set up test pack-------------------
            UITab uiTabInfo = TestUtils.CreateSavedUITabInfo();

            //---------------Assert Precondition----------------
            Assert.IsFalse(uiTabInfo.Status.IsNew);
            string message;
            Assert.IsTrue(uiTabInfo.IsDeletable(out message));

            //---------------Execute Test ----------------------
            uiTabInfo.MarkForDelete();
            uiTabInfo.Save();

            //---------------Test Result -----------------------  
            Assert.IsTrue(uiTabInfo.Status.IsNew);
            Assert.IsTrue(uiTabInfo.Status.IsDeleted);
        }

        [Test]
        public void Test_UpdateUITabInfo()
        {
            //---------------Set up test pack-------------------
            UITab uiTabInfo = TestUtils.CreateSavedUITabInfo();

            //---------------Assert Precondition----------------
            Assert.IsFalse(uiTabInfo.Status.IsNew);
            string message;
            Assert.IsTrue(uiTabInfo.IsEditable(out message));

            //---------------Execute Test ----------------------
            TestUtils.UpdateUITabInfoWithValidData(uiTabInfo);

            //---------------Test Result -----------------------
            Assert.IsFalse(uiTabInfo.Status.IsNew);
        }
    }
}